/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <vector>
# include <algorithm>

using namespace std;

# include "Structures.h"
# include "AtomUtils.h"
# include "IOUtils.h"
# include "VectorManip.h"
# include "PrintPdb.h"

const int C_LENG=0;
const int C_ANG=1;
const int C_DIH=2;
const int N_LENG=4;
const int N_ANG=5;
const int N_DIH=6;

const int CA=0;
const int C=1;
const int N=2;
const int O=3;

void addBackboneToHead(vector< vector<AtomStruct> > &Atoms, Array4D &cancbins)
{
	int i=27, j=27, k=22;
	Real ang, length, dih;
	Real commonDihedral=1.04;
	Real commonAngle=1.58;
	Real CAtoCLength=3.813;
	Real CAtoNLength=1.46;
	Real C_CA_N_Angle=111.008*DEGREE_TO_RAD;
	AtomStruct tempAtom;

	ang=commonAngle;
	length=CAtoCLength;
	dih=commonDihedral;

	icToXYZ(Atoms[2][CA], Atoms[1][CA], Atoms[0][CA], tempAtom, ang, length, dih);

	ang=cancbins[C_ANG][i][j][k];
	length=cancbins[C_LENG][i][j][k];
	dih=cancbins[C_DIH][i][j][k]*DEGREE_TO_RAD;

	icToXYZ(tempAtom, Atoms[1][CA], Atoms[0][CA], Atoms[0][C], ang, length, dih);

	ang=cancbins[N_ANG][i][j][k];
	length=cancbins[N_LENG][i][j][k];
	dih=cancbins[N_DIH][i][j][k];

	icToXYZ(Atoms[2][CA], Atoms[0][CA], Atoms[1][CA], Atoms[1][N], ang, length, dih);

	ang=C_CA_N_Angle;
	length=CAtoNLength;
	dih=180.0*DEGREE_TO_RAD;

	icToXYZ(Atoms[1][N], Atoms[0][C], Atoms[0][CA], Atoms[0][N], ang, length, dih);
}

void addBackboneToTail(vector< vector<AtomStruct> > &Atoms, Array4D &cancbins)
{
	int i=26, j=26, k=21;
	int nresidue=Atoms.size();
	Real ang, length, dih;
	Real commonDihedral=1.04;
	Real commonAngle=1.58;
	Real CAtoCLength=3.813;
	AtomStruct tempAtom;
	
	ang=commonAngle;
	length=CAtoCLength;
	dih=commonDihedral;

	icToXYZ(Atoms[nresidue-3][CA], Atoms[nresidue-2][CA], Atoms[nresidue-1][CA], tempAtom, ang, length, dih);
	
	ang=cancbins[C_ANG][i][j][k];
	length=cancbins[C_LENG][i][j][k];
	dih=cancbins[C_DIH][i][j][k];

	icToXYZ(Atoms[nresidue-3][CA], Atoms[nresidue-1][CA], Atoms[nresidue-2][CA], Atoms[nresidue-2][C], ang, length, dih);

	ang=cancbins[N_ANG][i][j][k];
	length=cancbins[N_LENG][i][j][k];
	dih=cancbins[N_DIH][i][j][k];

	icToXYZ(tempAtom, Atoms[nresidue-2][CA], Atoms[nresidue-1][CA], Atoms[nresidue-1][N], ang, length, dih);

	ang=111.008*DEGREE_TO_RAD;
	length=1.252;
	dih=180.0*DEGREE_TO_RAD;
	
	icToXYZ(Atoms[nresidue-2][C], Atoms[nresidue-1][N], Atoms[nresidue-1][CA], Atoms[nresidue-1][C], ang, length, dih);
}

void addBackboneToResidue(vector< vector<AtomStruct> > &Atoms, int residue, Array4D &cancbins)
{
	int i, j, k;
	int Size, maxAngBin1, maxAngBin2, maxAngBin3;
	Real ang, length, dih;
	Real ang1, ang2;
	Real dihedralBinSize=1.0/25.0;
	Real angleBinSize=1.0/50.0;
	Real angShift=53.0;
	AtomStruct tempAtom;

	Get4DVectorSize(cancbins, Size, maxAngBin1, maxAngBin2, maxAngBin3, "cancbins");
	ang1=calcAngle(Atoms[residue][CA], Atoms[residue+1][CA], Atoms[residue+2][CA]);
	ang2=calcAngle(Atoms[residue+1][CA], Atoms[residue+2][CA], Atoms[residue+3][CA]);
	dih=calcDihedralAngle(Atoms[residue][CA], Atoms[residue+1][CA], Atoms[residue+2][CA], Atoms[residue+3][CA]);
	if (ang1<0) ang1+=2.0*pi;
	if (ang2<0) ang2+=2.0*pi;
	if (dih<0) dih+=2.0*pi;

	i=int(ang1/angleBinSize-angShift);
	j=int(ang2/angleBinSize-angShift);
	k=int(dih/dihedralBinSize);

	if (i<0) i=0;
	else if (i>=maxAngBin1) i=maxAngBin1-1;
	if (j<0) j=0;
	else if (j>=maxAngBin2) j=maxAngBin2-1;
	if (k<0) k=0;
	else if (k>=maxAngBin3) k=maxAngBin3-1;


	ang=cancbins[C_ANG][i][j][k];
	length=cancbins[C_LENG][i][j][k];
	dih=cancbins[C_DIH][i][j][k]*DEGREE_TO_RAD;
	
	icToXYZ(Atoms[residue][CA], Atoms[residue+2][CA], Atoms[residue+1][CA], Atoms[residue+1][C], ang, length, dih);

	ang=0.275;
	length=1.46;
	dih=180.0*DEGREE_TO_RAD;

	icToXYZ(Atoms[residue+1][C], Atoms[residue+1][CA], Atoms[residue+2][CA], Atoms[residue+2][N], ang, length, dih);
}

void addBackboneToMain(vector< vector<AtomStruct> > &Atoms, vector< Array3D > &cancbins)
{
	int nresidue=Atoms.size();
	for (int i=0;i<nresidue-3;i++)
	{
		addBackboneToResidue(Atoms, i, cancbins);
	}
}

void addOxygen(vector< vector<AtomStruct> > &Atoms)
{
	int nresidue=Atoms.size();
	Real ang, length, dih;

	ang=2.0961;
	length=1.229;
	dih=179.6715*DEGREE_TO_RAD;

	for (int i=1;i<nresidue;i++)
	{
		icToXYZ(Atoms[i][N], Atoms[i-1][CA], Atoms[i-1][C], Atoms[i-1][O], ang, length, dih);
		cout <<endl;
		cout <<"ang= "<<ang<<endl;
		cout <<calcAngle(Atoms[i-1][CA], Atoms[i-1][C], Atoms[i-1][O])<<endl;
		cout <<endl;
	}
	icToXYZ(Atoms[nresidue-1][N], Atoms[nresidue-2][CA], Atoms[nresidue-2][C], Atoms[nresidue-2][O], 1.2439, 0, 2.0855);
}

void proteinToAtomMatrix(ProteinStruct &Protein, vector< vector<AtomStruct> > &Atoms)
{
	int nresidue=Protein.Atoms.size();
	AtomStruct tempAtom;

	Safe2DAlloc(Atoms, tempAtom, nresidue, 4, "Atoms");

	for (int i=0;i<nresidue;i++)
	{
		for (int j=0;j<4;j++)
		{
			Atoms[i][j]=Protein.Atoms[i];
		}
		Atoms[i][CA].AtomName="CA";
		Atoms[i][C].AtomName="C";
		Atoms[i][N].AtomName="N";
		Atoms[i][O].AtomName="O";
	}
}

void atomMatrixToProtein(vector< vector<AtomStruct> > &Atoms, ProteinStruct &Protein)
{
	int nresidue=Atoms.size();

	Protein.Atoms.clear();
	for (int i=0;i<nresidue;i++)
	{
		for (int j=0;j<4;j++)
		{
			SafePushBack(Protein.Atoms, Atoms[i][j], "Protein.Atoms");
		}
	}
}

void addBackbone(ProteinStruct &Protein, vector< Array3D > &cancbins)
{
	vector< vector<AtomStruct> > Atoms;

	proteinToAtomMatrix(Protein, Atoms);
	addBackboneToHead(Atoms, cancbins);
	addBackboneToTail(Atoms, cancbins);
	addBackboneToMain(Atoms, cancbins);
	addOxygen(Atoms);
	atomMatrixToProtein(Atoms, Protein);
}

void readCancFile(string cancFile, Array4D &cancbins)
{
	fstream file;
	string line;
	vector<string> str;
	int i, j, k;
	int Size;

	OpenFile(cancFile, file, "cancFile");
	getline(file, line);

	Safe4DAlloc(cancbins, 15, 107, 107, 162, "cancbins");

	while (true)
	{
		if (file.eof()) break;
		Tokenize2(line, " ", str);
		Size=str.size();
		if (Size>=15)
		{
			i=toInt(str[0]);
			j=toInt(str[1]);
			k=toInt(str[2]);

			cancbins[C_LENG][i][j][k]=toReal(str[4]);
			cancbins[C_ANG][i][j][k]=toReal(str[6]);
			cancbins[C_DIH][i][j][k]=toReal(str[8]);
			cancbins[N_LENG][i][j][k]=toReal(str[10]);
			cancbins[N_ANG][i][j][k]=toReal(str[12]);
			cancbins[N_DIH][i][j][k]=toReal(str[14]);
		}
		getline(file, line);
	}
	file.close();
}

void addBackbone(ProteinStruct &Protein, string cancFile)
{
	vector< Array3D > cancbins;

	readCancFile(cancFile, cancbins);
	addBackbone(Protein, cancbins);
}
