/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <vector>
# include <algorithm>
# include <stdlib.h>
# include <stdio.h>
# include <map>

using namespace std;

# include "TypeDef.h"
# include "Structures.h"
# include "AtomCode.h"
# include "AtomIDs.h"
# include "AtomUtils.h"
# include "AssignAtomIDs.h"
# include "ASA.h"
# include "Constants.h"
# include "error.h"
# include "GetAtomType.h"
# include "GetAtomType2.h"
# include "MathUtils.h"
# include "ReadGro.h"
# include "ReadPdb.h"
# include "ReadPrm.h"
# include "ReadPsf.h"
# include "ReadRtf.h"
# include "ResidueCode.h"
# include "ResidueID.h"
# include "VectorManip.h"
# include "MinMax.h"
# include "ResidueID.h"

void copyAtom(const AtomStruct &Atom, AtomStruct &Atom2)
{
	Atom2.HetAtom=Atom.HetAtom;
	Atom2.solute=Atom.solute;
	Atom2.AtomNumber=Atom.AtomNumber;
	Atom2.ResidueNum=Atom.ResidueNum;
	Atom2.atomid=Atom.atomid;
	Atom2.AtomType=Atom.AtomType;
	Atom2.AtomType2=Atom.AtomType2;
	Atom2.ParmType=Atom.ParmType;
	Atom2.residueid=Atom.residueid;
	Atom2.weight=Atom.weight;
	Atom2.Electrons=Atom.Electrons;
	Atom2.charge=Atom.charge;
	Atom2.mass=Atom.mass;
	Atom2.x=Atom.x;
	Atom2.y=Atom.y;
	Atom2.z=Atom.z;
	Atom2.vdw=Atom.vdw;
	//Atom2.epsilon=Atom.epsilon;
	Atom2.vdw14=Atom.vdw14;
	Atom2.epsilon14=Atom.epsilon14;
	Atom2.Occupancy=Atom.Occupancy;
	Atom2.BFactor=Atom.BFactor;
	Atom2.AtomName=Atom.AtomName;
	Atom2.CharmmAtomName=Atom.CharmmAtomName;
	Atom2.ResidueName=Atom.ResidueName;
	Atom2.ChainName=Atom.ChainName;
	Atom2.SegID=Atom.SegID;
	Atom2.ID=Atom.ID;
	Atom2.HydrationRadius=Atom.HydrationRadius;
}

void initializeAtom(AtomStruct &Atom)
{
	Atom.HetAtom=false;
	Atom.solute=true;
	Atom.AtomNumber=0;
	Atom.ResidueNum=0;
	Atom.atomid=0;
	Atom.AtomType=0;
	Atom.AtomType2=0;
	Atom.ParmType=0;
	Atom.residueid=0;
	//Atom.atomtype=0;
	Atom.weight=1.0;
	Atom.Electrons=0;
	Atom.charge=0;
	Atom.mass=0;
	Atom.x=0;
	Atom.y=0;
	Atom.z=0;
	Atom.vdw=0;
	//Atom.epsilon=0;
	Atom.vdw14=0;
	Atom.epsilon14=0;
	Atom.Occupancy=0;
	Atom.BFactor=0;
	Atom.AtomName="";
	Atom.CharmmAtomName="";
	Atom.ResidueName="";
	Atom.ChainName="A";
	Atom.SegID="";
	Atom.ID="";
}

void setAtomTypeNames(vector<string> &AtomTypeName)
{
	string name;
	string AtomName[NumAtomTypes1], ResidueName[NumAtomTypes1];
	AtomTypeToAtomName(AtomName);
	AtomTypeToResidueName(ResidueName);
	AtomTypeName.clear();
	for (int i=0;i<NumAtomTypes1;i++)
	{
		name=ResidueName[i]+"\t"+AtomName[i];
		SafePushBack(AtomTypeName, name, "AtomTypeName");
	}
}

void moveAtoms(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, Real dx, Real dy, Real dz)
{
	Atoms2=Atoms1;
	moveAtoms(Atoms2, dx, dy, dz);
}

//void moveAtoms(vector<AtomStruct> &Atoms, Real dx, Real dy, Real dz, bool boolPrint=true)
void moveAtoms(vector<AtomStruct> &Atoms, Real dx, Real dy, Real dz, bool boolPrint)
{
	int natom=Atoms.size();
	if (boolPrint) cout <<"MoveX= "<<dx<<" MoveY= "<<dy<<" MoveZ= "<<dz<<endl;
	for (int i=0;i<natom;i++)
	{
		Atoms[i].x+=dx;
		Atoms[i].y+=dy;
		Atoms[i].z+=dz;
	}
}

void moveAtoms(vector<AtomStruct> &Atoms, Vector &d)
{
	int natom=Atoms.size();
	int Size=d.size();

	if (Size!=3)
	{
		string errorStr=" The size of vector d is "+IntToStr(Size)+" ";
		errorStr+="instead of 3";
		error(errorStr, __LINE__, __FILE__);
	}
	for (int i=0;i<natom;i++)
	{
		Atoms[i].x+=d[X];
		Atoms[i].y+=d[Y];
		Atoms[i].z+=d[Z];
	}
}

void moveAtom(AtomStruct &Atom, Real dx, Real dy, Real dz)
{
	Atom.x+=dx;
	Atom.y+=dy;
	Atom.z+=dz;
}

void moveAtom(AtomStruct &Atom, VectorStruct &v)
{
	moveAtom(Atom, v.x, v.y, v.z);
}

void moveAtom(AtomStruct &Atom, Real v[3])
{
	moveAtom(Atom, v[X], v[Y], v[Z]);
}

void moveAtom(AtomStruct &Atom, vector<Real> &v)
{
	moveAtom(Atom, v[X], v[Y], v[Z]);
}

void mergeProteins(const vector<ProteinStruct> &Proteins, vector<AtomStruct> &Atoms)
{
	int i, nprotein=Proteins.size();
	DeleteVector(Atoms);

	for (i=0;i<nprotein;i++) AppendVector(Proteins[i].Atoms, Atoms);
}

void calcDistMatrix(vector< vector<Real> > &distMatrix, vector<AtomStruct> &Atoms)
{
	int i, j, natom=Atoms.size();
	Real dist;
	Real dx, dy, dz;
	int Size1, Size2;

	Get2DVectorSize(distMatrix, Size1, Size2);
	if (Size1!=natom || Size2!=natom)
	{
		Safe2DAlloc(distMatrix, natom, natom, "distMatrix");
	}
	for (i=0;i<natom;i++)
	{
		for (j=i+1;j<natom;j++)
		{
			dx=Atoms[i].x-Atoms[j].x;
			dy=Atoms[i].y-Atoms[j].y;
			dz=Atoms[i].z-Atoms[j].z;
			dist=sqrt(dx*dx+dy*dy+dz*dz);
			distMatrix[i][j]=dist;
			distMatrix[j][i]=dist;
		}
	}
}

void calcDistanceMatrix(vector<AtomStruct> &Atoms, Matrix &DistanceMatrix)
{
	int natom=Atoms.size();
	int Size1, Size2;
	Real dist;

	Get2DVectorSize(DistanceMatrix, Size1, Size2);
	if (Size1!=natom || Size2!=natom)
	{
		Safe2DAlloc(DistanceMatrix, natom, natom, "DistanceMatrix");
	}
	for (int i=0;i<natom;i++)
	{
		//printAtomInfo(Atoms[i]);
		for (int j=i+1;j<natom;j++)
		{
			dist=atomDistance(Atoms[i], Atoms[j]);
			DistanceMatrix[i][j]=dist;
			DistanceMatrix[j][i]=dist;
		}
	}
}

Real atomSquareDistance(AtomStruct &Atom1, AtomStruct &Atom2)
{
	Real dx, dy, dz;
	dx=Atom1.x-Atom2.x;
	dy=Atom1.y-Atom2.y;
	dz=Atom1.z-Atom2.z;
	return dx*dx+dy*dy+dz*dz;
}

Real atomDistance(AtomStruct &Atom1, AtomStruct &Atom2)
{
	Real dx, dy, dz;
	dx=Atom1.x-Atom2.x;
	dy=Atom1.y-Atom2.y;
	dz=Atom1.z-Atom2.z;
	return sqrt(dx*dx+dy*dy+dz*dz);
}

Real atomDistance(vector<AtomStruct> &Atoms, int atom1, int atom2)
{
	Real dx, dy, dz;
	dx=Atoms[atom1].x-Atoms[atom2].x;
	dy=Atoms[atom1].y-Atoms[atom2].y;
	dz=Atoms[atom1].z-Atoms[atom2].z;
	return sqrt(dx*dx+dy*dy+dz*dz);
}

Real atomDistance(Real x, Real y, Real z, AtomStruct &Atom)
{
	Real dx, dy, dz;
	dx=x-Atom.x;
	dy=y-Atom.y;
	dz=z-Atom.z;
	return sqrt(dx*dx+dy*dy+dz*dz);
}


void updateDistanceMatrix(vector<AtomStruct> &Atoms, Matrix &DistanceMatrix, int start1, int end1, int start2, int end2)
{
	int natom=Atoms.size();
	int Size1, Size2;
	Real dist;

	Get2DVectorSize(DistanceMatrix, Size1, Size2);
	if (Size1!=natom || Size2!=natom)
	{
		string errorStr="natom= "+toStr(natom);
		errorStr+=" Size1= "+toStr(Size1);
		errorStr+=" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}
	if (start1>=natom || end1>=natom || start2>=natom || end2>=natom)
	{
		string errorStr="start1= "+toStr(start1);
		errorStr+=" end1= "+toStr(end1);
		errorStr+=" start2= "+toStr(start2);
		errorStr+=" end2= "+toStr(end2);
		errorStr+=" natom= "+toStr(natom);
		error(errorStr, __LINE__, __FILE__);
	}
	for (int i=start1;i<=end1;i++)
	{
		for (int j=start2;j<=end2;j++)
		{
			dist=atomDistance(Atoms[i], Atoms[j]);
			DistanceMatrix[i][j]=dist;
			DistanceMatrix[j][i]=dist;
		}
	}
}

void updateDistanceMatrix(ProteinStruct &Protein, vector<int> &start, vector<int> &end)
{
	int nstart=start.size();
	int nend=end.size();

	if (nstart!=nend)
	{
		cout <<"ERROR: nstart!=nend"<<endl;
		exit(EXIT_FAILURE);
	}
	for (int i=0;i<nstart;i++)
	{
		for (int j=i+1;j<nstart;j++)
		{
			updateDistanceMatrix(Protein.Atoms, Protein.DistMatrix, start[i], end[i], start[j], end[j]);
		}
	}
}

void updateAtomDistances(vector<AtomStruct> &Atoms, Matrix &DistMatrix, int atomNum)
{
	int natom=Atoms.size();
	int Size1, Size2;
	Real dist;

	Get2DVectorSize(DistMatrix, Size1, Size2);

	if (Size1!=natom || Size2!=natom)
	{
		string errorStr="natom= "+toStr(natom);
		errorStr+=" Size1= "+toStr(Size1);
		errorStr+=" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}
	if (atomNum>=natom)
	{
		string errorStr="natom= "+toStr(natom);
		errorStr+=" atomNum= "+toStr(atomNum);
		error(errorStr, __LINE__, __FILE__);
	}
	for (int i=0;i<natom;i++)
	{
		if (i!=atomNum)
		{
			dist=atomDistance(Atoms[i], Atoms[atomNum]);
			DistMatrix[i][atomNum]=dist;
			DistMatrix[atomNum][i]=dist;
		}
	}
}

void updateLocalDistanceMatrix(ProteinStruct &Protein, int residue1, int residue2)
{
	int natom=Protein.Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Protein.Atoms[i].ResidueNum>=residue1 && Protein.Atoms[i].ResidueNum<=residue2)
		{
			updateAtomDistances(Protein.Atoms, Protein.DistMatrix, i);
		}
	}
}

void updateDistanceMatrixForPivot(ProteinStruct &Protein, int residue, int angleType)
{
	int atom1, atom2, atom3, atom4;
	vector<int> start, end;
	start.resize(2);
	end.resize(2);
	getDihedralAtoms(Protein.Atoms, atom1, atom2, atom3, atom4, angleType, residue);
	start[0]=0;
	end[0]=atom2;
	start[1]=atom3;
	end[1]=Protein.Atoms.size()-1;
	updateDistanceMatrix(Protein, start, end);
}

int findFirstAtomWithResidueNum(vector<AtomStruct> &Atoms, int residue)
{
	int natom=Atoms.size();
	int firstAtom=0;
	for (int i=0;i<natom;i++)
	{
		//cout <<"residue= "<<Atoms[i].ResidueNum<<endl;
		if (Atoms[i].ResidueNum==residue) 
		{
			firstAtom=i;
			break;
		}
	}
	return firstAtom;
}

int findLastAtomWithResidueNum(vector<AtomStruct> &Atoms, int residue)
{
	int natom=Atoms.size();
	int lastAtom=0;
	for (int i=natom-1;i>=0;i--)
	{
		if (Atoms[i].ResidueNum==residue) 
		{
			lastAtom=i;
			break;
		}
	}
	return lastAtom;
}

void updateDistanceMatrix(ProteinStruct &Protein, int residue1, int residue2)
{
	int atom1, atom2, natom=Protein.Atoms.size();
	vector<int> start, end;
	//cout <<"residue1= "<<residue1<<" residue2= "<<residue2<<endl;
	atom1=findFirstAtomWithResidueNum(Protein.Atoms, residue1);
	atom2=findLastAtomWithResidueNum(Protein.Atoms, residue2);
	SafePushBack(start, 0, "start");
	SafePushBack(end, atom1-1, "end");
	for (int i=atom1;i<=atom2;i++)
	{
		SafePushBack(start, i, "start");
		SafePushBack(end, i, "end");
	}
	SafePushBack(start, atom2+1, "start");
	SafePushBack(end, natom-1, "end");
	updateDistanceMatrix(Protein, start, end);
}

void rotateAroundX(AtomStruct &Atom, Real theta)
{
	Real tempY, tempZ;
	tempY=Atom.y*cos(theta)-Atom.z*sin(theta);
	tempZ=Atom.y*sin(theta)+Atom.z*cos(theta);
	Atom.y=tempY;
	Atom.z=tempZ;
}

void rotateAroundX(vector<AtomStruct> &Atoms, Real theta)
{
	int natom=Atoms.size();
	Real tempY, tempZ;
	Real costheta=cos(theta);
	Real sintheta=sin(theta);
	for (int i=0;i<natom;i++)
	{
		tempY=Atoms[i].y*costheta-Atoms[i].z*sintheta;
		tempZ=Atoms[i].y*sintheta+Atoms[i].z*costheta;
		Atoms[i].y=tempY;
		Atoms[i].z=tempZ;
	}
}

void rotateAroundY(AtomStruct &Atom, Real theta)
{
	Real tempX, tempZ;
	tempX=Atom.x*cos(theta)+Atom.z*sin(theta);
	tempZ=-Atom.x*sin(theta)+Atom.z*cos(theta);
	Atom.x=tempX;
	Atom.z=tempZ;
}

void rotateAroundY(vector<AtomStruct> &Atoms, Real theta)
{
	int natom=Atoms.size();
	Real tempX, tempZ;
	Real costheta=cos(theta);
	Real sintheta=sin(theta);
	for (int i=0;i<natom;i++)
	{
		tempX=Atoms[i].x*costheta+Atoms[i].z*sintheta;
		tempZ=-Atoms[i].x*sintheta+Atoms[i].z*costheta;
		Atoms[i].x=tempX;
		Atoms[i].z=tempZ;
	}
}

void rotateAroundZ(AtomStruct &Atom, Real theta)
{
	Real tempX, tempY;
	tempX=Atom.x*cos(theta)-Atom.y*sin(theta);
	tempY=Atom.x*sin(theta)+Atom.y*cos(theta);
	Atom.x=tempX;
	Atom.y=tempY;
}

void rotateAroundZ(vector<AtomStruct> &Atoms, Real theta)
{
	int natom=Atoms.size();
	Real tempX, tempY;
	Real costheta=cos(theta);
	Real sintheta=sin(theta);
	for (int i=0;i<natom;i++)
	{
		tempX=Atoms[i].x*costheta-Atoms[i].y*sintheta;
		tempY=Atoms[i].x*sintheta+Atoms[i].y*costheta;
		Atoms[i].x=tempX;
		Atoms[i].y=tempY;
	}
}

void calcRotationMatrix(VectorStruct &v, Real angle, Real RotationMatrix[3][3])
{
	vectorNormalize(v);
	RotationMatrix[0][0]=cos(angle)+v.x*v.x*(1.0-cos(angle));
	RotationMatrix[0][1]=v.x*v.y*(1.0-cos(angle))-v.z*sin(angle);
	RotationMatrix[0][2]=v.x*v.z*(1.0-cos(angle))+v.y*sin(angle);

	RotationMatrix[1][0]=v.x*v.y*(1.0-cos(angle))+v.z*sin(angle);
	RotationMatrix[1][1]=cos(angle)+v.y*v.y*(1.0-cos(angle));
	RotationMatrix[1][2]=v.y*v.z*(1.0-cos(angle))-v.x*sin(angle);

	RotationMatrix[2][0]=v.x*v.z*(1.0-cos(angle))-v.y*sin(angle);
	RotationMatrix[2][1]=v.y*v.z*(1.0-cos(angle))+v.x*sin(angle);
	RotationMatrix[2][2]=cos(angle)+v.z*v.z*(1.0-cos(angle));

	for (int i=0;i<3;i++)
	{
		for (int j=0;j<3;j++)
		{
			//cout <<"rmat= "<<RotationMatrix[i][j]<<endl;
		}
	}
}

void calcRotationMatrix(VectorStruct &v, Real angle, Matrix &RotationMatrix)
{
	int Size1, Size2;
	vectorNormalize(v);

	Get2DVectorSize(RotationMatrix, Size1, Size2);
	if (Size1!=3 || Size2!=3) Safe2DAlloc(RotationMatrix, 3, 3, "RotationMatrix");

	RotationMatrix[0][0]=cos(angle)+v.x*v.x*(1.0-cos(angle));
	RotationMatrix[0][1]=v.x*v.y*(1.0-cos(angle))-v.z*sin(angle);
	RotationMatrix[0][2]=v.x*v.z*(1.0-cos(angle))+v.y*sin(angle);

	RotationMatrix[1][0]=v.x*v.y*(1.0-cos(angle))+v.z*sin(angle);
	RotationMatrix[1][1]=cos(angle)+v.y*v.y*(1.0-cos(angle));
	RotationMatrix[1][2]=v.y*v.z*(1.0-cos(angle))-v.x*sin(angle);

	RotationMatrix[2][0]=v.x*v.z*(1.0-cos(angle))-v.y*sin(angle);
	RotationMatrix[2][1]=v.y*v.z*(1.0-cos(angle))+v.x*sin(angle);
	RotationMatrix[2][2]=cos(angle)+v.z*v.z*(1.0-cos(angle));
}

void calcRotationMatrixDerivative(VectorStruct &v, Matrix &RotationMatrix)
{
	/*
	Real angle=0;
	RotationMatrix[0][0]=-sin(angle)+v.x*v.x*sin(angle);
	RotationMatrix[0][1]=v.x*v.y*sin(angle)-v.z*cos(angle);
	RotationMatrix[0][2]=v.x*v.z*sin(angle)+v.y*cos(angle);

	RotationMatrix[1][0]=v.x*v.y*(sin(angle))+v.z*cos(angle);
	RotationMatrix[1][1]=-sin(angle)+v.y*v.y*sin(angle);
	RotationMatrix[1][2]=v.y*v.z*sin(angle)-v.x*cos(angle);

	RotationMatrix[2][0]=v.x*v.z*sin(angle)-v.y*cos(angle);
	RotationMatrix[2][1]=v.y*v.z*sin(angle)+v.x*cos(angle);
	RotationMatrix[2][2]=-sin(angle)+v.z*v.z*sin(angle);
	*/
	RotationMatrix[0][0]=0;
	RotationMatrix[0][1]=-v.z;
	RotationMatrix[0][2]=v.y;

	RotationMatrix[1][0]=v.z;
	RotationMatrix[1][1]=0;
	RotationMatrix[1][2]=-v.x;

	RotationMatrix[2][0]=-v.y;
	RotationMatrix[2][1]=v.x;
	RotationMatrix[2][2]=0;
}

void applyRotationMatrix(Real &x, Real &y, Real &z, Real RotationMatrix[3][3])
{
	Real tempX, tempY, tempZ;

	tempX=RotationMatrix[0][0]*x+RotationMatrix[0][1]*y+RotationMatrix[0][2]*z;
	tempY=RotationMatrix[1][0]*x+RotationMatrix[1][1]*y+RotationMatrix[1][2]*z;
	tempZ=RotationMatrix[2][0]*x+RotationMatrix[2][1]*y+RotationMatrix[2][2]*z;

	x=tempX;
	y=tempY;
	z=tempZ;
}

void applyRotationMatrix(AtomStruct &Atom, Real RotationMatrix[3][3])
{
	applyRotationMatrix(Atom.x, Atom.y, Atom.z, RotationMatrix);
}

void applyRotationMatrix(Real &x, Real &y, Real &z, Matrix &RotationMatrix)
{
	Real tempX, tempY, tempZ;

	tempX=RotationMatrix[0][0]*x+RotationMatrix[0][1]*y+RotationMatrix[0][2]*z;
	tempY=RotationMatrix[1][0]*x+RotationMatrix[1][1]*y+RotationMatrix[1][2]*z;
	tempZ=RotationMatrix[2][0]*x+RotationMatrix[2][1]*y+RotationMatrix[2][2]*z;

	x=tempX;
	y=tempY;
	z=tempZ;
}

void applyRotationMatrix(AtomStruct &Atom, Matrix &RotationMatrix)
{
	applyRotationMatrix(Atom.x, Atom.y, Atom.z, RotationMatrix);
}

void applyRotationMatrixToVector(VectorStruct &v, Real RotationMatrix[3][3])
{
	applyRotationMatrix(v.x, v.y, v.z, RotationMatrix);
}

VectorStruct atomVector(AtomStruct &Atom1, AtomStruct &Atom2)
{
	VectorStruct v1;
	//printAtomInfo(Atom1);
	//printAtomInfo(Atom2);
	v1.x=Atom1.x-Atom2.x;
	v1.y=Atom1.y-Atom2.y;
	v1.z=Atom1.z-Atom2.z;

	return v1;
}

void applyRotationMatrixToAtom(vector<AtomStruct> &Atoms, int atom1, int atom2, Real RotationMatrix[3][3])
{
	VectorStruct v;

	v=atomVector(Atoms[atom2], Atoms[atom1]);
	applyRotationMatrixToVector(v, RotationMatrix);
	Atoms[atom2].x=Atoms[atom1].x+v.x;
	Atoms[atom2].y=Atoms[atom1].y+v.y;
	Atoms[atom2].z=Atoms[atom1].z+v.z;
}

void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, int Atom, Real RotationMatrix[3][3])
{
	int natom=Atoms.size();
	VectorStruct v;

	for (int i=Atom+1;i<natom;i++)
	{
		v=atomVector(Atoms[i], Atoms[Atom]);
		applyRotationMatrixToVector(v, RotationMatrix);
		Atoms[i].x=Atoms[Atom].x+v.x;
		Atoms[i].y=Atoms[Atom].y+v.y;
		Atoms[i].z=Atoms[Atom].z+v.z;
	}
}

void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Real RotationMatrix[3][3])
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		applyRotationMatrix(Atoms[i].x, Atoms[i].y, Atoms[i].z, RotationMatrix);
	}
}

void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Matrix &RotationMatrix)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		applyRotationMatrix(Atoms[i].x, Atoms[i].y, Atoms[i].z, RotationMatrix);
	}
}

void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Real x, Real y, Real z, Real RotationMatrix[3][3])
{
	int natom=Atoms.size();
	VectorStruct v;

	for (int i=0;i<natom;i++)
	{
		v.x=Atoms[i].x-x;
		v.y=Atoms[i].y-y;
		v.z=Atoms[i].z-z;
		applyRotationMatrixToVector(v, RotationMatrix);
		Atoms[i].x=x+v.x;
		Atoms[i].y=y+v.y;
		Atoms[i].z=z+v.z;
	}
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, Real angle)
{
	Real RotationMatrix[3][3];
	Real x, y, z;

	calcCenter(Atoms, x, y, z);
	calcRotationMatrix(v, angle, RotationMatrix);
	applyRotationMatrixToAtoms(Atoms, x, y, z, RotationMatrix);

	if (isnan(angle))
	{
		error("angle= nan", __LINE__, __FILE__);
	}
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, Real x, Real y, Real z, Real angle)
{
	Real RotationMatrix[3][3];

	calcRotationMatrix(v, angle, RotationMatrix);
	applyRotationMatrixToAtoms(Atoms, x, y, z, RotationMatrix);

	if (isnan(angle))
	{
		error("angle= nan", __LINE__, __FILE__);
	}
}

void rotateAtomAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, int atom1, int atom2, Real angle)
{
	Real RotationMatrix[3][3];

	calcRotationMatrix(v, angle, RotationMatrix);
	applyRotationMatrixToAtom(Atoms, atom1, atom2, RotationMatrix);
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, int Atom, Real angle)
{
	Real RotationMatrix[3][3];

	calcRotationMatrix(v, angle, RotationMatrix);
	applyRotationMatrixToAtoms(Atoms, Atom, RotationMatrix);
}

VectorStruct rotateAroundVector(VectorStruct v, Real angle, VectorStruct &rotationV)
{
	Real RotationMatrix[3][3];
	VectorStruct vout;

	vout.x=v.x;
	vout.y=v.y;
	vout.z=v.z;
	calcRotationMatrix(rotationV, angle, RotationMatrix);
	applyRotationMatrixToVector(vout, RotationMatrix);
	return vout;
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, vector<bool> &use, Real phi, Real psi, Real theta)
{
	Real RotationMatrix[3][3];
	Real x, y, z;
	VectorStruct v;

	v.x=sin(phi);
	v.y=cos(phi)*sin(psi);
	v.z=cos(phi)*cos(psi);
	
        calcRotationMatrix(v, theta, RotationMatrix);
	calcCenter(Atoms, use, x, y, z);
	moveAtoms(Atoms, -x, -y, -z);
        applyRotationMatrixToAtoms(Atoms, 0, 0, 0, RotationMatrix);
	moveAtoms(Atoms, x, y, z);
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, Real phi, Real psi, Real theta)
{
	Real RotationMatrix[3][3];
	Real x, y, z;
	VectorStruct v;

	v.x=sin(phi);
	v.y=cos(phi)*sin(psi);
	v.z=cos(phi)*cos(psi);
	
        calcRotationMatrix(v, theta, RotationMatrix);
	calcCenter(Atoms, x, y, z);
	moveAtoms(Atoms, -x, -y, -z);
        applyRotationMatrixToAtoms(Atoms, 0, 0, 0, RotationMatrix);
	moveAtoms(Atoms, x, y, z);
}

Real calcAngle(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3)
{
	VectorStruct v1, v2;
	Real DotProduct;

	v1=atomVector(Atom2, Atom1);
	v2=atomVector(Atom3, Atom2);
	//cout <<"v1.x= "<<v1.x<<" v1.y= "<<v1.y<<" v1.z= "<<v1.z<<endl;
	//cout <<"v2.x= "<<v2.x<<" v2.y= "<<v2.y<<" v2.z= "<<v2.z<<endl;
	DotProduct=v1*v2;
	//cout <<"DotProduct= "<<DotProduct<<endl;
	return acos(DotProduct/(vectorLength(v1)*vectorLength(v2)));
}

Real calcAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3)
{
	return calcAngle(Atoms[atom1], Atoms[atom2], Atoms[atom3]);
}

VectorStruct calcCrossProduct(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3)
{
	VectorStruct v1, v2, product;

	v1.x=Atom2.x-Atom1.x;
	v1.y=Atom2.y-Atom1.y;
	v1.z=Atom2.z-Atom1.z;

	v2.x=Atom3.x-Atom2.x;
	v2.y=Atom3.y-Atom2.y;
	v2.z=Atom3.z-Atom2.z;

	//printAtomInfo(Atom1);
	//printAtomInfo(Atom2);
	//printAtomInfo(Atom3);

	product=calcCrossProduct(v1, v2);

	return product;
}

Real calcSign(VectorStruct &v1, VectorStruct &plane1, VectorStruct &plane2)
{
	VectorStruct CrossProduct;
	Real DotProduct;
	CrossProduct=calcCrossProduct(plane1, plane2);
	DotProduct=v1*CrossProduct;
	if (DotProduct>0) return 1.0;
	else return -1.0;
}

Real calcDihedralAngle(VectorStruct &v1, VectorStruct &v2, VectorStruct &v3)
{
	VectorStruct plane1, plane2;
	Real DotProduct, costheta, sign;
	plane1=calcCrossProduct(v2, v1);
	plane2=calcCrossProduct(v3, v2);
	DotProduct=plane1*plane2;
	sign=calcSign(v2, plane1, plane2);
	costheta=DotProduct/(vectorLength(plane1)*vectorLength(plane2));
	if (costheta>1.0) costheta=1.0;
	else if (costheta<-1.0) costheta=-1.0;
	if (sign>0) return acos(costheta);
	if (sign<=0) return -acos(costheta);
	cout <<"ERROR: Should not be at the end of calcDihedralAngle"<<endl;
	return 0;
}

Real calcDihedralAngle(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3, AtomStruct &Atom4)
{
	VectorStruct v1, v2, v3;

	v1=atomVector(Atom2, Atom1);
	v2=atomVector(Atom3, Atom2);
	v3=atomVector(Atom4, Atom3);
	
	return calcDihedralAngle(v1, v2, v3);
}

Real calcDihedralAngle(int Atom1, int Atom2, int Atom3, int Atom4, vector<AtomStruct> &Atoms)
{
	return calcDihedralAngle(Atoms[Atom1], Atoms[Atom2], Atoms[Atom3], Atoms[Atom4]);
}

Real calcDihedralAngle(vector<AtomStruct> &Atoms, int dihedralIndex)
{
	int residue, angleType;
	dihedralIndexToResidueAndAngleType(dihedralIndex, residue, angleType);
	return calcDihedralAngle(Atoms, angleType, residue); 
}

Real calcDihedralAngle(vector<AtomStruct> &Atoms, int AngleType, int ResidueNum)
{
	if (AngleType==PHI) return calcPhi(Atoms, ResidueNum);
	else if (AngleType==PSI) return calcPsi(Atoms, ResidueNum);
	else if (AngleType==OMEGA) return calcOmega(Atoms, ResidueNum);
	else
	{
		cout <<"ERROR: Unknown AngleType "<<AngleType<<endl;
		cout <<"Acceptable values are "<<PHI<<", "<<PSI<<", and "<<OMEGA<<endl;
		exit(EXIT_FAILURE);
	}
}

Real calcPhi(vector<AtomStruct> &Atoms, int ResidueNum)
{
	int natom=Atoms.size();
	int c=UNK_INT, n=UNK_INT, ca=UNK_INT, c2=UNK_INT;
	int firstResidue;
	getFirstResidue(Atoms, firstResidue);
	//cout <<"firstResidue= "<<firstResidue<<" ResidueNum= "<<ResidueNum<<endl;
	if (ResidueNum<=firstResidue)
	{
		
		error("No phi angle for residue", __LINE__, __FILE__);
	}
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum-1 && Atoms[i].AtomName=="C") c=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="N") n=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") ca=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") c2=i;
	}
	if (c==UNK_INT || n==UNK_INT || ca==UNK_INT || c2==UNK_INT) 
	{
		cout <<"ERROR: Unable to get atom index in calcPhi"<<endl;
		exit(EXIT_FAILURE);
	}
	return calcDihedralAngle(c, n, ca, c2, Atoms);
}

Real calcPsi(vector<AtomStruct> &Atoms, int ResidueNum)
{
	int natom=Atoms.size();
	int n=UNK_INT, ca=UNK_INT, c=UNK_INT, n2=UNK_INT;
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="N") n=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") ca=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") c=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="N") n2=i;
	}
	if (n==UNK_INT || ca==UNK_INT || c==UNK_INT || n2==UNK_INT) 
	{
		cout <<"ResidueNum= "<<ResidueNum<<" n= "<<n<<" ca= "<<ca
		<<" c= "<<c<<" n2= "<<n2<<endl;
		error("Unable to get atom index in calcPsi", __LINE__, __FILE__);
	}
	return calcDihedralAngle(n, ca, c, n2, Atoms);
}

Real calcOmega(vector<AtomStruct> &Atoms, int ResidueNum)
{
	int natom=Atoms.size();
	int ca=UNK_INT, c=UNK_INT, n=UNK_INT, ca2=UNK_INT;
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") ca=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") c=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="N") n=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="CA") ca2=i;
	}
	if (ca==UNK_INT || c==UNK_INT || n==UNK_INT || ca2==UNK_INT) 
	{
		error("Unable to get atom index in calcOmega", __LINE__, __FILE__);
	}
	return calcDihedralAngle(ca, c, n, ca2, Atoms);
}


void calcDihedrals(vector<AtomStruct> &Atoms, Vector &dihedral)
{
	int nresidue;
	Real phi, psi, omega;
	nresidue=getNumResidues(Atoms);
	dihedral.clear();
	for (int i=1;i<=nresidue;i++)
	{
		if (i>1) 
		{
			phi=calcPhi(Atoms, i);
			SafePushBack(dihedral, phi, "dihedral");
		}
		if (i<nresidue) 
		{
			psi=calcPsi(Atoms, i);
			SafePushBack(dihedral, psi, "dihedral");
		}
		if (i<nresidue) 
		{
			omega=calcOmega(Atoms, i);
			SafePushBack(dihedral, omega, "dihedral");
		}
	}
}

void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, int Atom1, int Atom2, Real angle)
{
	VectorStruct v;

	v.x=Atoms[Atom2].x-Atoms[Atom1].x;
	v.y=Atoms[Atom2].y-Atoms[Atom1].y;
	v.z=Atoms[Atom2].z-Atoms[Atom1].z;
	vectorNormalize(v);

	rotateAtomsAroundVector(Atoms, v, Atom2, angle);
}

void changeDihedralAngle(vector<AtomStruct> &Atoms, int atom2, int atom3, vector<bool> &moved, VectorStruct &v, Real dAngle)
{
	int nconnect;
	if (!moved[atom3])
	{
		rotateAtomAroundVector(Atoms, v, atom2, atom3, dAngle);
		moved[atom3]=true;
		nconnect=Atoms[atom3].connectivity.size();
		for (int i=0;i<nconnect;i++)
		{
			changeDihedralAngle(Atoms, atom2, Atoms[atom3].connectivity[i], moved, v, dAngle);
		}
	}
	/*
	int natom=Atoms.size();
	for (int i=atom3;i<natom;i++)
	{
		rotateAtomAroundVector(Atoms, v, atom2, i, dAngle);
	}
	*/
}

void changeDihedralAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, int atom4, Real dAngle)
{
	vector<bool> moved;
	int natom=Atoms.size();
	VectorStruct v;

	SafeAlloc(moved, natom, "moves");
	moved[atom2]=true;
	v=atomVector(Atoms[atom3], Atoms[atom2]);
	vectorNormalize(v);
	changeDihedralAngle(Atoms, atom3, atom4, moved, v, dAngle);
}

void setDihedralAngle(vector<AtomStruct> &Atoms, int Atom1, int Atom2, int Atom3, int Atom4, Real DihedralAngle)
{
	Real OldDihedralAngle=calcDihedralAngle(Atom1, Atom2, Atom3, Atom4, Atoms);
	Real deltaAngle=DihedralAngle-OldDihedralAngle;
	changeDihedralAngle(Atoms, Atom1, Atom2, Atom3, Atom4, deltaAngle);
	OldDihedralAngle=calcDihedralAngle(Atom1, Atom2, Atom3, Atom4, Atoms);
}

void rotateDihedralAngleBy(vector<AtomStruct> &Atoms, int atom1, int atom2, Real deltaAngle)
{
	rotateAtomsAroundVector(Atoms, atom1, atom2, deltaAngle);
}

void rotateDihedralAngleBy(vector<AtomStruct> &Atoms, int dihedralIndex, Real deltaAngle)
{
	int atom1, atom2, atom3, atom4;
	int angleType, residue;
	dihedralIndexToResidueAndAngleType(dihedralIndex, residue, angleType);
	getDihedralAtoms(Atoms, atom1, atom2, atom3, atom4, angleType, residue);
	rotateDihedralAngleBy(Atoms, atom2, atom3, deltaAngle); 
}

void getPhiDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum)
{
	int natom=Atoms.size();
	int firstResidue;
	getFirstResidue(Atoms, firstResidue);
	if (ResidueNum<=firstResidue)
	{
		error("No phi angle for residue", __LINE__, __FILE__);
	}

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum-1 && Atoms[i].AtomName=="C") Atom1=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="N") Atom2=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") Atom3=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") Atom4=i;
	}
}

void getPsiDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="N") Atom1=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") Atom2=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") Atom3=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="N") Atom4=i;
	}
}

void getOmegaDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="CA") Atom1=i;
		else if (Atoms[i].ResidueNum==ResidueNum && Atoms[i].AtomName=="C") Atom2=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="N") Atom3=i;
		else if (Atoms[i].ResidueNum==ResidueNum+1 && Atoms[i].AtomName=="CA") Atom4=i;
	}
}

void getDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int AngleType, int ResidueNum)
{
	if (AngleType==PHI) getPhiDihedralAtoms(Atoms, Atom1, Atom2, Atom3, Atom4, ResidueNum);
	else if (AngleType==PSI) getPsiDihedralAtoms(Atoms, Atom1, Atom2, Atom3, Atom4, ResidueNum);
	else if (AngleType==OMEGA) getOmegaDihedralAtoms(Atoms, Atom1, Atom2, Atom3, Atom4, ResidueNum);
	else
	{
		cout <<"ERROR: Unknown dihedral angle type"<<endl;
		exit(EXIT_FAILURE);
	}
}

void setDihedralAngle(vector<AtomStruct> &Atoms, int AngleType, int ResidueNum, Real DihedralAngle)
{
	int Atom1, Atom2, Atom3, Atom4;
	getDihedralAtoms(Atoms, Atom1, Atom2, Atom3, Atom4, AngleType, ResidueNum);
	setDihedralAngle(Atoms, Atom1, Atom2, Atom3, Atom4, DihedralAngle);
}

void dihedralIndexToResidueAndAngleType(int dihedralIndex, int &residue, int &angleType)
{
	residue=int((dihedralIndex+1)/3)+1;
	if (dihedralIndex%3==0) angleType=PSI;
	if (dihedralIndex%3==1) angleType=OMEGA;
	if (dihedralIndex%3==2) angleType=PHI;
}

void printDihedral(vector<Real> &dihedral)
{
	int ndihedral=dihedral.size();
	int residue;

	for (int i=0;i<ndihedral;i++)
	{
		residue=int((i+1)/3)+1; 
		if (i%3==0) cout <<"PSI\t";
		if (i%3==1) cout <<"OMEGA\t";
		if (i%3==2) cout <<"PHI\t";
		//cout <<residue<<"\t"<<dihedral[i]*RAD_TO_DEGREE<<endl;
		cout <<residue<<"\t"<<dihedral[i]<<endl;
	}
	cout <<endl;
}

void printDihedral(vector<AtomStruct> &Atoms)
{
	vector<Real> dihedrals;

	calcDihedrals(Atoms, dihedrals);
	printDihedral(dihedrals);
}

void setDihedralAngle(vector<AtomStruct> &Atoms, int dihedralIndex, Real DihedralAngle)
{
	int angleType, residue;
	dihedralIndexToResidueAndAngleType(dihedralIndex, residue, angleType);
	setDihedralAngle(Atoms, angleType, residue, DihedralAngle);
}

void setDihedralAngles(vector<AtomStruct> &Atoms, vector<Real> &dihedrals)
{
	int FirstResidue=UNK_INT, LastResidue=UNK_INT;
	int numDihedral=0;
	getFirstLastResidue(Atoms, FirstResidue, LastResidue);
	for (int i=FirstResidue;i<=LastResidue;i++)
	{
		if (i!=FirstResidue)
		{
			setDihedralAngle(Atoms, PHI, i, dihedrals[numDihedral]);
			numDihedral++;
		}
		if (i!=LastResidue)
		{
			setDihedralAngle(Atoms, PSI, i, dihedrals[numDihedral]);
			numDihedral++;
			setDihedralAngle(Atoms, OMEGA, i, dihedrals[numDihedral]);
			numDihedral++;
		}
	}
}

int phiPsiToBasin(Real phi, Real psi)
{
	//This is made up.  Use real numbers latter
	phi*=RAD_TO_DEGREE;
	psi*=RAD_TO_DEGREE;
	if (((phi>=-180.0 && phi<=0.0) || (phi>=140.0 && phi<=180.0)))
	{
		if (psi>=-110.0 && psi<=45.0) return HELIX;
		if (phi>=-90.0 && phi<=0) return PP2;
		return BETA;
	}
	if (psi>=-30.0 && psi<=135.0) return LEFT_HANDED_HELIX;
	if (psi<=-110.0 || (psi>=135.0 && psi<=170.0)) return OTHER;
	return FORBIDDEN;
}

string basinTypeToString(int basinType)
{
	if (basinType==HELIX) return "H";
	if (basinType==PP2) return "P";
	if (basinType==BETA) return "B";
	if (basinType==LEFT_HANDED_HELIX) return "L";
	if (basinType==OTHER) return "O";
	if (basinType==FORBIDDEN) return "F";
	cout <<"ERROR: Unknown basin type"<<endl;
	exit(EXIT_FAILURE);
}

string phiPsiToBasinString(vector<Real> &phi, vector<Real> &psi)
{
	string basinString="";
	int nangle=phi.size();

	for (int i=0;i<nangle;i++)
	{
		basinString+=basinTypeToString(phiPsiToBasin(phi[i], psi[i]));
	}
	return basinString;
}

void rotateAtoms(vector<AtomStruct> &Atoms, vector<bool> &use, Real theta, Real phi, Real psi)
{
	int natom=Atoms.size();
	Real x, y, z;

	calcCenter(Atoms, use, x, y, z);
	moveAtoms(Atoms, -x, -y, -z);
	for (int i=0;i<natom;i++)
	{
		rotateAroundX(Atoms[i], theta);
		rotateAroundY(Atoms[i], phi);
		rotateAroundZ(Atoms[i], psi);
	}
	moveAtoms(Atoms, x, y, z);
} 

void rotateAtoms(vector<AtomStruct> &Atoms, Real theta, Real phi, Real psi)
{
	Real x, y, z;

	calcCenter(Atoms, x, y, z);
	moveAtoms(Atoms, -x, -y, -z);
	if (theta!=0) rotateAroundX(Atoms, theta);
	if (phi!=0) rotateAroundY(Atoms, phi);
	if (psi!=0) rotateAroundZ(Atoms, psi);
	moveAtoms(Atoms, x, y, z);
} 

void rotateAtomsNoCenter(vector<AtomStruct> &Atoms, Real theta, Real phi, Real psi)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		rotateAroundX(Atoms[i], theta);
		rotateAroundY(Atoms[i], phi);
		rotateAroundZ(Atoms[i], psi);
	}
}

void rotateAtom(AtomStruct &Atom, Real theta, Real phi, Real psi)
{
	rotateAroundX(Atom, theta);
	rotateAroundY(Atom, phi);
	rotateAroundZ(Atom, psi);
} 
/*
   void allignBondWithZAxis(vector<AtomStruct> &Atoms, BondStruct &bond)
   {
   angleWithRespectoToXAxis=calcAngleWithRespectToXAxis(Atoms[bond.atom1], Atoms[bond.atom2]);
   rotateAroundZ(Atoms, angleWithRespectToXAxis);
   anngleWithRespectToZAxis=calcAngleWithRespectToZAxis(Atoms[bond.atom1], Atoms[bond.atom2]);
   rotateAroundX(Atoms, angleWithRespectToZAxis);
   }

   Real calcProjection(vector<AtomStruct> &Atoms, BondStruct &bond1, BondStruct &bond2)
   {
   VectorStruct v1, v2;

   v1=atomVector(Atoms[bond1.atom1], Atoms[bond1.atom2]);
   v2=atomVector(Atoms[bond2.atom1], Atoms[bond2.atom2]);
   return calcVectorAngle(v1, v2);
   }

   Real calcProjection(vector<AtomStruct> &Atoms, BondStruct &mainBond, BondStruct &bond1, BondStruct &bond2)
   {
   Real projection;
   allignBondWithZAxis(Atoms, mainBond);
   projection=calcProjection(Atoms, bond1, bond2);
   return projection;
   }

   void MakeProjectionsOfBondsPerpendicular(vector<AtomStruct> &Atoms, BondStruct &mainBond, BondStruct &bond1, BondStruct &bond2)
   {
   projection=calcProjection(Atoms, mainBond, bond1, bond2);

   }
 */

void doubleCrank(vector<AtomStruct> &Atoms, int Residue, Real dAngle)
{
	int FirstResidue, LastResidue;
	Real phi, psi;
	getFirstLastResidue(Atoms, FirstResidue, LastResidue);
	if (Residue!=FirstResidue)
	{
		phi=calcPhi(Atoms, Residue);
		setDihedralAngle(Atoms, PHI, Residue, phi+dAngle);
		psi=calcPsi(Atoms, Residue-1);
		setDihedralAngle(Atoms, PSI, Residue-1, psi-dAngle);
	}
}

void doubleCrank(vector<AtomStruct> &Atoms, int Residue, int AngleType, Real dAngle)
{
	int FirstResidue=UNK_INT, LastResidue=UNK_INT;
	Real phi, psi;
	getFirstLastResidue(Atoms, FirstResidue, LastResidue);
	if (AngleType==PHI && Residue!=FirstResidue)
	{
		phi=calcPhi(Atoms, Residue);
		setDihedralAngle(Atoms, PHI, Residue, phi+dAngle);
		psi=calcPsi(Atoms, Residue-1);
		setDihedralAngle(Atoms, PSI, Residue-1, psi-dAngle);
	}
	if (AngleType==PSI && Residue!=LastResidue)
	{
		psi=calcPsi(Atoms, Residue);
		setDihedralAngle(Atoms, PSI, Residue, psi+dAngle);
		phi=calcPhi(Atoms, Residue+1);
		setDihedralAngle(Atoms, PHI, Residue+1, phi-dAngle);
	}
}

void pivot(vector<AtomStruct> &Atoms, int Residue, int AngleType, Real dAngle)
{
	Real angle;
	angle=calcDihedralAngle(Atoms, AngleType, Residue);
	setDihedralAngle(Atoms, AngleType, Residue, angle+dAngle);
}

VectorStruct findNormVector(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3)
{
	VectorStruct v1, v2, crossProduct;

	v1=atomVector(Atoms[atom2], Atoms[atom1]);
	v2=atomVector(Atoms[atom3], Atoms[atom2]);
	crossProduct=calcCrossProduct(v1, v2);
	vectorNormalize(crossProduct);
	return crossProduct;
}

void getAngles(ProteinStruct &Protein)
{
	int natom=Protein.Atoms.size();
	int nconnect;
	BondAngleStruct tempBond;

	tempBond.angle=UNK_REAL;
	for (int i=0;i<natom;i++)
	{
		nconnect=Protein.Atoms[i].connectivity.size();
		tempBond.atom2=i;
		for (int j=0;j<nconnect;j++)
		{
			for (int k=j+1;k<nconnect;k++)
			{
				tempBond.atom1=Protein.Atoms[i].connectivity[j];
				tempBond.atom3=Protein.Atoms[i].connectivity[k];
				SafePushBack(Protein.bondAngles, tempBond, "bondAngles");
			}
		}
	}
}

void changeBondAngle(vector<AtomStruct> &Atoms, int atom2, int atom3, vector<bool> &moved, VectorStruct &v, Real dAngle)
{
	int nconnect;
	if (!moved[atom3])
	{
		rotateAtomAroundVector(Atoms, v, atom2, atom3, dAngle);
		moved[atom3]=true;
		nconnect=Atoms[atom3].connectivity.size();
		for (int i=0;i<nconnect;i++)
		{
			changeBondAngle(Atoms, atom2, Atoms[atom3].connectivity[i], moved, v, dAngle);
		}
	}
}

void changeBondAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, Real dAngle)
{
	vector<bool> moved;
	int natom=Atoms.size();
	VectorStruct v;

	SafeAlloc(moved, natom, "moved");
	moved[atom1]=true;
	v=findNormVector(Atoms, atom1, atom2, atom3);
	changeBondAngle(Atoms, atom2, atom3, moved, v, dAngle);

	//rotateAtomsAroundVector(Atoms, v, atom2, dAngle);
}

void changeBondAngle(ProteinStruct &Protein, int bondIndex, Real dAngle)
{
	int nBondAngles=Protein.bondAngles.size();
	int atom1, atom2, atom3;

	if (bondIndex<nBondAngles)
	{
		atom1=Protein.bondAngles[bondIndex].atom1;
		atom2=Protein.bondAngles[bondIndex].atom2;
		atom3=Protein.bondAngles[bondIndex].atom3;
		changeBondAngle(Protein.Atoms, atom1, atom2, atom3, dAngle);
	}
}

void setBondAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, Real angle)
{
	Real oldAngle=calcAngle(Atoms[atom1], Atoms[atom2], Atoms[atom3]);
	Real dAngle=angle-oldAngle;
	changeBondAngle(Atoms, atom1, atom2, atom3, dAngle);
	cout <<calcAngle(Atoms[atom1], Atoms[atom2], Atoms[atom3])<<endl;
}

int getAtomIndex(vector<AtomStruct> &Atoms, int residue, string AtomName)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].AtomName);
		if (Atoms[i].ResidueNum==residue && Atoms[i].AtomName==AtomName)
		{
			return i;
		}
	}
	return UNK_INT;
}

int getNumResidues(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	return Atoms[natom-1].ResidueNum-Atoms[0].ResidueNum+1;
}

void getResidues(vector<AtomStruct> &Atoms, vector<int> &ResidueTypes)
{
	int natom=Atoms.size();
	int ResidueType;
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName=="CA")
		{
			ResidueType=GetResidueID(Atoms[i].ResidueName);                   
			SafePushBack(ResidueTypes, ResidueType, "ResidueTypes");
		}
	}
}

void getWaterMolecule(vector<AtomStruct> &Atoms, int index, vector<AtomStruct> &WaterMolecule)
{
	bool BoolPrint=false;
	int natom=Atoms.size();
	int ResidueNum;
	int min, max;
	cout <<"index= "<<index<<endl;
	cout <<"natom= "<<natom<<endl;
	if (index<natom && index>=0)
	{
		ResidueNum=Atoms[index].ResidueNum;
		WaterMolecule.clear();
		min=index-3;
		max=index+3;
		if (min<0) min=0;
		if (max>=natom) max=natom;
		if (BoolPrint) cout <<endl;
		//cout <<"In GetWaterMolecule"<<endl;
		for (int i=min;i<max;i++)
		{
			//cout <<"i= "<<i<<" ResidueNum= "<<ResidueNum<<endl;
			if (BoolPrint) printAtomInfo(Atoms[i]);
			if (Atoms[i].ResidueNum==ResidueNum && isWater(Atoms[i]))
			{
				SafePushBack(WaterMolecule, Atoms[i], "WaterMolecule in GetWaterMolecule");
				if (BoolPrint) cout <<"Added atom to water molecule"<<endl;
			}
		}
		if (BoolPrint) cout <<endl;
	}
}

bool isWater(AtomStruct &Atom)
{
	if (Atom.ResidueName=="HOH") return true;
	if (Atom.ResidueName=="TIP") return true;
	if (Atom.ResidueName=="TIP3") return true;
	if (Atom.ResidueName=="TIP4") return true;
	if (Atom.ResidueName=="TIP5") return true;
	if (Atom.ResidueName=="WAT") return true;
	if (Atom.ResidueName=="SPC") return true;
	if (Atom.ResidueName=="SOL") return true;
	return false;
}

bool isWaterOxygen(AtomStruct &Atom)
{
	if (Atom.atomid==OXYGEN && isWater(Atom)) return true;
	else return false;
}

bool isIon(AtomStruct &Atom)
{
	if (Atom.ResidueName=="SOD") return true;
	else if (Atom.ResidueName=="CLA") return true;
	else if (Atom.ResidueName=="IP") return true;
	else if (Atom.ResidueName=="IM") return true;
	else if (Atom.ResidueName=="MG") return true;
	return false;
}

bool isProtein(AtomStruct &Atom)
{
	if (Atom.ResidueName=="GLY") return true;
	else if (Atom.ResidueName=="ALA") return true;
	else if (Atom.ResidueName=="VAL") return true;
	else if (Atom.ResidueName=="LEU") return true;
	else if (Atom.ResidueName=="ILE") return true;
	else if (Atom.ResidueName=="SER") return true;
	else if (Atom.ResidueName=="THR") return true;
	else if (Atom.ResidueName=="CYS") return true;
	else if (Atom.ResidueName=="MET") return true;
	else if (Atom.ResidueName=="MSE") return true;
	else if (Atom.ResidueName=="ASP") return true;
	else if (Atom.ResidueName=="GLU") return true;
	else if (Atom.ResidueName=="ASN") return true;
	else if (Atom.ResidueName=="GLN") return true;
	else if (Atom.ResidueName=="LYS") return true;
	else if (Atom.ResidueName=="ARG") return true;
	else if (Atom.ResidueName=="PHE") return true;
	else if (Atom.ResidueName=="TYR") return true;
	else if (Atom.ResidueName=="PRO") return true;
	else if (Atom.ResidueName=="HIS") return true;
	else if (Atom.ResidueName=="HSE") return true;
	else if (Atom.ResidueName=="HIE") return true;
	else if (Atom.ResidueName=="HSD") return true;
	else if (Atom.ResidueName=="TRP") return true;
	else return false;
}

bool isCB(AtomStruct &Atom)
{
	if (isProtein(Atom))
	{
		if (Atom.AtomName=="CB") return true;
		if (Atom.AtomName=="CA") return true;
		if (Atom.AtomName=="C") return true;
		if (Atom.AtomName=="N") return true;
		if (Atom.AtomName=="O") return true;
	}
	return false;
}

bool isBackbone(AtomStruct &Atom)
{
	if (isProtein(Atom))
	{
		if (Atom.AtomName=="CA") return true;
		if (Atom.AtomName=="C") return true;
		if (Atom.AtomName=="N") return true;
		if (Atom.AtomName=="O") return true;
	}
	return false;
}

bool isDuplicate(AtomStruct &Atom, vector<AtomStruct> &Atoms, int index)
{
	int natom=Atoms.size();
	int NumAppearances=0;
	for (int i=index+1;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==Atom.ResidueNum && Atoms[i].AtomName==Atom.AtomName)
		{
			//cout <<"ChainName "<<Atoms[i].ChainName<<" "<<Atom.ChainName<<endl;
			//cout <<"ID "<<Atoms[i].ID<<" "<<Atom.ID<<endl;
			//cout <<"SegID "<<Atoms[i].SegID<<" "<<Atom.SegID<<endl;
			//cout <<"duplicate "<<Atoms[i].duplicate<<" "<<Atom.duplicate<<endl;
			//printAtomInfo(Atoms[i]);
			//printAtomInfo(Atom);
			//cout <<endl;
			if (Atoms[i].ChainName==Atom.ChainName && Atoms[i].ID==Atom.ID && Atoms[i].SegID==Atom.SegID && Atoms[i].duplicate!=Atom.duplicate)
			{
				if (Atoms[i].ResidueName==Atom.ResidueName)
				{
					NumAppearances++;
					//cout <<endl;
					//printAtomInfo(Atom);
					//printAtomInfo(Atoms[i]);
					//cout <<endl;
					return true;
				}
			}
		}
	}
	return false;
}

bool isSameResidue(AtomStruct &Atom1, AtomStruct &Atom2)
{
	if (Atom1.ResidueNum==Atom2.ResidueNum && Atom1.ChainName==Atom2.ChainName)
	{
		return true;
	}
	return false;
}

bool isNextResidue(AtomStruct &Atom1, AtomStruct &Atom2)
{
	if (Atom1.ResidueNum+1==Atom2.ResidueNum && Atom1.ChainName==Atom2.ChainName)
	{
		return true;
	}
	return false;
}

bool isPreviousResidue(AtomStruct &Atom1, AtomStruct &Atom2)
{
	if (Atom1.ResidueNum-1==Atom2.ResidueNum && Atom1.ChainName==Atom2.ChainName)
	{
		return true;
	}
	return false;
}

void removeNonHetatm(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> TempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].HetAtom) 
		{
			SafePushBack(TempAtoms, Atoms[i], "in removeNonHetatm");
		}
	}
	Atoms=TempAtoms;
}

void removeNonCA(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> TempAtoms;
	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].AtomName);
		if (Atoms[i].AtomName=="CA")
		{
			SafePushBack(TempAtoms, Atoms[i], "in RemoveNonCa");
		}
	}
	Atoms=TempAtoms;
}

void removeNonCaAndOp(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> TempAtoms;
	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].AtomName);
		if (Atoms[i].AtomName=="CA" || Atoms[i].AtomName=="O1P" || Atoms[i].AtomName=="OP1")
		{
			SafePushBack(TempAtoms, Atoms[i], "in RemoveNonCaAndOp");
		}
	}
	Atoms=TempAtoms;
}

void removeDuplicateAtoms(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> TempAtoms;
	for (int i=0;i<natom;i++)
	{
		if (!isDuplicate(Atoms[i], Atoms, i)) 
		{
			SafePushBack(TempAtoms, Atoms[i], "TempAtoms in removeDuplicateAtoms");
		}
	}
	Atoms=TempAtoms;
}

void removeProteinAtoms(vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> TempAtoms;
	int i, natom=Atoms.size();
	for (i=0;i<natom;i++)
	{
		if (isWater(Atoms[i])) SafePushBack(TempAtoms, Atoms[i], "TempAtoms in RemoveProteinAtoms");
	}
	Atoms=TempAtoms;
}

void removeNonProteinAtoms(vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> TempAtoms;
	int natom=Atoms.size();
	for (int i=0;i<natom;i++)
	{
		if (isProtein(Atoms[i])) SafePushBack(TempAtoms, Atoms[i], "TempAtoms in RemoveNonProteinAtoms");
	}
	Atoms=TempAtoms;
}

void removeWaters(vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> TempAtoms;
	int i, natom=Atoms.size();
	for (i=0;i<natom;i++)
	{
		if (!isWater(Atoms[i])) SafePushBack(TempAtoms, Atoms[i], "TempAtoms in removeWaters");
	}
	Atoms=TempAtoms;
}

void removeUnknownResidues(vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> TempAtoms;
	int i, natom=Atoms.size();
	for (i=0;i<natom;i++)
	{
		if (isProtein(Atoms[i]) || isWater(Atoms[i])) SafePushBack(TempAtoms, Atoms[i], "TempAtoms");
	}
	Atoms=TempAtoms;
}

vector<AtomStruct> removeHydrogensVector(vector<AtomStruct> &Atoms1)
{
	int natom=Atoms1.size();
	vector<AtomStruct> Atoms2;

	for (int i=0;i<natom;i++)
	{
		if (Atoms1[i].AtomName.substr(0,1)!="H")
		{
			SafePushBack(Atoms2, Atoms1[i], "removeHydrogens");
		}
	}
	return Atoms2;
}

void removeHydrogens(vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> tempAtoms;

	tempAtoms=removeHydrogensVector(Atoms);
	Atoms=tempAtoms;
}

void removeSideChains(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName=="C" || Atoms[i].AtomName=="CA" || Atoms[i].AtomName=="N" || Atoms[i].AtomName=="O" || Atoms[i].AtomName=="CB")
		{
			if (Atoms[i].ResidueName!="CA")
			{
				SafePushBack(tempAtoms, Atoms[i], "RemoveSideChains");
			}
		}
	}
	Atoms=tempAtoms;
}

void removeNonBackbone(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName=="C" || Atoms[i].AtomName=="CA" || Atoms[i].AtomName=="N" || Atoms[i].AtomName=="O")
		{
			if (Atoms[i].ResidueName!="CA")
			{
				SafePushBack(tempAtoms, Atoms[i], "RemoveNonBackbone");
			}
		}
	}
	Atoms=tempAtoms;

}

void removeResiduesWithName(vector<AtomStruct> &Atoms, string residue)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueName!=residue)
		{
			SafePushBack(tempAtoms, Atoms[i], "tempAtoms");
		}
	}
	Atoms=tempAtoms;
}

void removeResiduesOtherThan(vector<AtomStruct> &Atoms, string residue)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueName==residue)
		{
			SafePushBack(tempAtoms, Atoms[i], "tempAtoms");
		}
	}
	Atoms=tempAtoms;
}

/*
void splitIntoChains(ProteinStruct &Protein, vector<ProteinStruct> &Proteins)
{
        int natom=Protein.Atoms.size();
        ProteinStruct tempProtein;

        tempProtein=Protein;
        tempProtein.Atoms.clear();
        for (int i=0;i<natom;i++)
        {
                if (i!=0)
                {
                        if (Protein.Atoms[i].ChainName!=Protein.Atoms[i-1].ChainName)
                        {
                                SafePushBack(Proteins, tempProtein, "Proteins");
                                tempProtein.Atoms.clear();
                        }
                }
                SafePushBack(tempProtein.Atoms, Protein.Atoms[i], "tempProtein");
        }
        SafePushBack(Proteins, tempProtein, "Proteins");
}
*/

void splitIntoChains(ProteinStruct &Protein, vector<ProteinStruct> &Proteins)
{
	int natom=Protein.Atoms.size();
	string chain;
	ProteinStruct tempProtein;
	map<string, vector<AtomStruct> > Chains;

	Proteins.clear();

	for (int i=0;i<natom;i++)
	{
		chain=Protein.Atoms[i].ChainName;
		SafePushBack(Chains[chain], Protein.Atoms[i], "Chains");
	}
	map<string, vector<AtomStruct> >::iterator iter;
	for (iter=Chains.begin();iter!=Chains.end();++iter)
	{
		tempProtein=Protein;
		tempProtein.Atoms=iter->second;
		SafePushBack(Proteins, tempProtein, "splitIntoChains");
	}
}

void getChain(vector<AtomStruct> &Atoms, string chain)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ChainName==chain)
		{
			SafePushBack(tempAtoms, Atoms[i], "tempAtoms");
		}
	}
	Atoms=tempAtoms;
}

int findNumProteinAtoms(vector<AtomStruct> &Atoms)
{
	int TotalParticles=Atoms.size();
	int numProteinAtoms=0;
	for (int n=0;n<TotalParticles;n++) 
	{
		if (Atoms[n].solute) numProteinAtoms++;
	}
	return numProteinAtoms;
}

int countIons(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	int nion=0;
	for (int i=0;i<natom;i++)
	{
		if (isIon(Atoms[i])) nion++;
	}
	return nion;
}

void setSolute(vector<AtomStruct> &Atoms)
{
	int TotalParticles=Atoms.size();
	for (int n=0;n<TotalParticles;n++) 
	{
		Atoms[n].solute=true;
		if (isWater(Atoms[n])) Atoms[n].solute=false;
		if (!isProtein(Atoms[n])) Atoms[n].solute=false;
		if (Atoms[n].atomid==HydrationShell) Atoms[n].solute=false;
		if (Atoms[n].atomid==ExcludedVolume) Atoms[n].solute=false;
	}
}

void setAllToSolute(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Atoms[i].solute=true;
	}
}

bool calcCenterCa(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave)
{
	Real xsum, ysum, zsum;
	Real NumAve=0;
	int i, natom=Atoms.size();
	xsum=ysum=zsum=0;
	//cout <<"natom= "<<natom;
	for (i=0;i<natom;i++)
	{
		//cout <<Atoms[i].AtomName<<endl;
		if (Atoms[i].AtomName=="CA")
		{
			xsum+=Atoms[i].x;
			ysum+=Atoms[i].y;
			zsum+=Atoms[i].z;
			NumAve+=1.0;
		}
	}
	if (NumAve==0)
	{
		cout <<"ERROR: In calcCenter.  No CA atoms."<<endl;
		cout <<"In file "<<__FILE__<<" line # "<<__LINE__<<endl;
		xave=0;
		yave=0;
		zave=0;
		return false;
	}
	xave=xsum/NumAve;
	yave=ysum/NumAve;
	zave=zsum/NumAve;

	return true;
}

bool calcCenter(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave)
{
	Real xsum, ysum, zsum;
	Real NumAve=0;
	int natom=Atoms.size();
	xsum=ysum=zsum=0;
	//cout <<"natom= "<<natom<<endl;
	//cout <<"In calcCenter"<<endl;
	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].AtomName);
		//printAtomInfo(Atoms[i]);
		//cout <<Atoms[i].AtomName<<"."<<endl;
		if (Atoms[i].AtomName=="CA" || Atoms[i].AtomName=="O1P" || Atoms[i].AtomName=="OP1")
		{
			//cout <<Atoms[i].AtomName<<"."<<endl;
			xsum+=Atoms[i].x;
			ysum+=Atoms[i].y;
			zsum+=Atoms[i].z;
			NumAve+=1.0;
			//cout <<"NumAve= "<<NumAve<<endl;
		}
	}
	if (NumAve==0)
	{
		cout <<"natom= "<<natom<<endl;
		cout <<"ERROR: In calcCenter.  No CA or O1P atoms."<<endl;
		cout <<"In file "<<__FILE__<<" in line # "<<__LINE__<<endl;
		xave=0;
		yave=0;
		zave=0;
		return false;
	}
	xave=xsum/NumAve;
	yave=ysum/NumAve;
	zave=zsum/NumAve;

	if (isnan(xave) || isnan(yave) || isnan(zave))
	{
		for (int i=0;i<natom;i++) printAtomInfo(Atoms[i]);
		exit(EXIT_FAILURE);
	}

	return true;
}

bool calcCenter(vector<AtomStruct> &Atoms, vector<bool> &use, Real &xave, Real &yave, Real &zave)
{
	Real xsum, ysum, zsum;
	Real NumAve=0;
	int natom=Atoms.size();
	xsum=ysum=zsum=0;

	for (int i=0;i<natom;i++)
	{
		if (use[i])
		{
			xsum+=Atoms[i].x;
			ysum+=Atoms[i].y;
			zsum+=Atoms[i].z;
			NumAve+=1.0;
		}
	}
	if (NumAve==0)
	{
		cout <<"ERROR: In calcCenter.  No selected atoms."<<endl;
		cout <<"In file "<<__FILE__<<" in line # "<<__LINE__<<endl;
		xave=0;
		yave=0;
		zave=0;
		return false;
	}
	xave=xsum/NumAve;
	yave=ysum/NumAve;
	zave=zsum/NumAve;

	return true;
}

bool calcCenterSelection(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave, string selection)
{
	Real xsum, ysum, zsum;
	Real NumAve=0;
	int i, natom=Atoms.size();
	xsum=ysum=zsum=0;
	//cout <<"natom= "<<natom;
	for (i=0;i<natom;i++)
	{
		//cout <<Atoms[i].AtomName<<endl;
		if (Atoms[i].AtomName==selection)
		{
			xsum+=Atoms[i].x;
			ysum+=Atoms[i].y;
			zsum+=Atoms[i].z;
			NumAve+=1.0;
		}
	}
	if (NumAve==0)
	{
		cout <<"ERROR: In calcCenterSelection.  No "<<selection
		<<" atoms."<<endl;
		cout <<"In file "<<__FILE__<<" in line # "<<__LINE__<<endl;
		xave=0;
		yave=0;
		zave=0;
		return false;
	}
	xave=xsum/NumAve;
	yave=ysum/NumAve;
	zave=zsum/NumAve;

	return true;
}

void calcCenterAll(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave)
{
	Real xsum, ysum, zsum;
	Real TotalMass=0;
	int natom=Atoms.size();
	xsum=ysum=zsum=0;
	for (int i=0;i<natom;i++)
	{
		xsum+=Atoms[i].x*Atoms[i].Electrons;
		ysum+=Atoms[i].y*Atoms[i].Electrons;
		zsum+=Atoms[i].z*Atoms[i].Electrons;
		TotalMass+=Atoms[i].Electrons;
	}
	if (TotalMass==0)
	{
		cout <<"natom= "<<natom<<endl;
		error("In calcCenterAll.  TotalMass=0.", __LINE__, __FILE__);
	}
	xave=xsum/TotalMass;
	yave=ysum/TotalMass;
	zave=zsum/TotalMass;

}

Real calcDipoleMoment(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	VectorStruct v;

	v.x=v.y=v.z=0;

	for (int i=0;i<natom;i++)
	{
		v.x+=Atoms[i].x*Atoms[i].charge;
		v.y+=Atoms[i].y*Atoms[i].charge;
		v.z+=Atoms[i].z*Atoms[i].charge;
	}
	return vectorLength(v);
}

void calcMomentOfInertia(vector<AtomStruct> &Atoms, Matrix &momentOfInertia)
{
	int natom=Atoms.size();
	int Size1=0, Size2=0;
	Real xave, yave, zave;
	Real dx, dy, dz;

	Size1=momentOfInertia.size();
	if (Size1==3) Size2=momentOfInertia[0].size();

	if (Size1==3 && Size2==3) ZeroMatrix(momentOfInertia);
	else Safe2DAlloc(momentOfInertia, 3, 3, "momentOfInertia");

	setAtomElectrons(Atoms);
	calcCenterAll(Atoms, xave, yave, zave);

	for (int i=0;i<natom;i++)
	{
		dx=Atoms[i].x-xave;
		dy=Atoms[i].y-yave;
		dz=Atoms[i].z-zave;
		momentOfInertia[X][X]+=dx*dx;
		momentOfInertia[Y][Y]+=dy*dy;
		momentOfInertia[Z][Z]+=dz*dz;
		momentOfInertia[X][Y]+=dx*dy;
		momentOfInertia[X][Z]+=dx*dz;
		momentOfInertia[Y][Z]+=dy*dz;
		
	}
	momentOfInertia[Y][X]=momentOfInertia[X][Y];
	momentOfInertia[Z][Y]=momentOfInertia[Y][Z];
	momentOfInertia[Z][X]=momentOfInertia[X][Z];
}

void alignWithYAxis(vector<AtomStruct> &Atoms)
{
        Vector eigenvector;
        Matrix momentOfInertia;
        VectorStruct v;

        Safe2DAlloc(momentOfInertia, 3, 3, "momentOfInertia");
        calcMomentOfInertia(Atoms, momentOfInertia);
        calcEigenvector(momentOfInertia, eigenvector);
        v.x=eigenvector[X];
        v.y=eigenvector[Y];
        v.z=eigenvector[Z];
        rotateToAlignWithYAxis(Atoms, v);
}

VectorStruct findLongestVector(vector<AtomStruct> &Atoms, int &atom1, int &atom2)
{
	//Finds the longest vector in a protein
	int natom=Atoms.size();
	Real dist, max=0;
	VectorStruct v;

	if (natom==0) error("No atoms in protein.", __LINE__, __FILE__);

	atom1=atom2=0;

	for (int i=0;i<natom;i++)
	{
		for (int j=i+1;j<natom;j++)
		{
			dist=atomDistance(Atoms[i], Atoms[j]);
			if (dist>max)
			{
				atom1=i;
				atom2=j;
				max=dist;
			}
		}
	}
	v=atomVector(Atoms[atom2], Atoms[atom1]);

	return v;
}

PosStruct findLongestOrthogonalAxes(vector<AtomStruct> &Atoms, int atom1, int atom2)
{
	//Finds the atom which is furthest away from a vector formed by two 
	//atoms.  Returns the vector connecting the atom and the first vector
	//en.wikipedia.org/wiki/Vector_projection
	int natom=Atoms.size();
	int atom3=0;
	Real dp1, dp2;
	Real theta, length, dist, max=0;
	PosStruct a, b, c, v;

	if (natom==0) error("No atoms in protein.", __LINE__, __FILE__);

	SafeAlloc(a.Pos, 3, "a");
	SafeAlloc(b.Pos, 3, "b");
	SafeAlloc(c.Pos, 3, "c");
	SafeAlloc(v.Pos, 3, "c");

	for (int i=0;i<natom;i++)
	{
		theta=calcAngle(Atoms, atom2, atom1, i);
		length=atomDistance(Atoms, atom1, i);
		dist=length*sin(theta);
		if (dist>max)
		{
			atom3=i;
			max=dist;
		}
	}
	a.Pos[X]=Atoms[atom3].x-Atoms[atom1].x;	
	a.Pos[Y]=Atoms[atom3].y-Atoms[atom1].y;	
	a.Pos[Z]=Atoms[atom3].z-Atoms[atom1].z;	

	b.Pos[X]=Atoms[atom2].x-Atoms[atom1].x;
	b.Pos[Y]=Atoms[atom2].y-Atoms[atom1].y;
	b.Pos[Z]=Atoms[atom2].z-Atoms[atom1].z;

	dp1=calcDotProduct(a.Pos, b.Pos);
	dp2=calcDotProduct(b.Pos, b.Pos);
	
	v.Pos[X]=a.Pos[X]-dp1*b.Pos[X]/dp2;
	v.Pos[Y]=a.Pos[Y]-dp1*b.Pos[Y]/dp2;
	v.Pos[Z]=a.Pos[Z]-dp1*b.Pos[Z]/dp2;

	return v;
}

Real calcUnitCellVolume(ProteinStruct &Protein)
{
	return calcVolume(Protein.a, Protein.b, Protein.c);
}

void center(vector<AtomStruct> &Atoms)
{
	Real xave, yave, zave;
	calcCenter(Atoms, xave, yave, zave);
	moveAtoms(Atoms, -xave, -yave, -zave);
}

void centerAll(vector<AtomStruct> &Atoms)
{
	Real xave, yave, zave;
	setAtomElectrons(Atoms);
	calcCenterAll(Atoms, xave, yave, zave);
	moveAtoms(Atoms, -xave, -yave, -zave);
}

void center(vector<AtomStruct> &Atoms, vector<bool> &use)
{
	Real xave, yave, zave;
	calcCenter(Atoms, use, xave, yave, zave);
	moveAtoms(Atoms, -xave, -yave, -zave);
}

void centerAtom(vector<AtomStruct> &Atoms, int index)
{
	moveAtoms(Atoms, -Atoms[index].x, -Atoms[index].y, -Atoms[index].z);
}

void selectAtomsByName(vector<AtomStruct> &Atoms, vector<AtomStruct> &SelectedAtoms, string SelectionName)
{
	int natom=Atoms.size();
	SelectedAtoms.clear();
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName==SelectionName)
		{
			SafePushBack(SelectedAtoms, Atoms[i], "SelectedAtoms");
		}
	}
}

void selectAtomsByName(vector<AtomStruct> &Atoms, string SelectionName)
{
	vector<AtomStruct> tempAtoms;

	selectAtomsByName(Atoms, tempAtoms, SelectionName);
	Atoms=tempAtoms;
}

void selectAtomsByName(vector<AtomStruct> &Atoms, string selectionName, vector<bool> &use)
{
	int natom=Atoms.size();

	SafeAlloc(use, natom, "use");

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName==selectionName) use[i]=true;
		else use[i]=false;
	}
}

void removeSelection(vector<AtomStruct> &Atoms, string selection)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].AtomName);
		//printAtomInfo(Atoms[i]);
		if (Atoms[i].AtomName!=selection)
		{
			//cout <<"Added Atom"<<endl;
			SafePushBack(tempAtoms, Atoms[i], "tempAtoms in removeSelection");
		}
	}
	Atoms=tempAtoms;
}

void removeResidue(vector<AtomStruct> &Atoms, string selection)
{
	int natom=Atoms.size();
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		Trim(Atoms[i].ResidueName);
		if (Atoms[i].ResidueName!=selection)
		{
			SafePushBack(tempAtoms, Atoms[i], "tempAtoms in removeResidue");
		}
	}
	Atoms=tempAtoms;
}

void changeBondLength(vector<AtomStruct> &Atoms, int atom, VectorStruct &v, vector<bool> &moved)
{
	int nconnect;
	if (!moved[atom])
	{
		moveAtom(Atoms[atom], v);
		moved[atom]=true;
		nconnect=Atoms[atom].connectivity.size();
		for (int i=0;i<nconnect;i++)
		{
			changeBondLength(Atoms, Atoms[atom].connectivity[i], v, moved);
		}
	}
}

void changeBondLength(vector<AtomStruct> &Atoms, int atom1, int atom2, Real dLength)
{
	vector<bool> moved;
	int natom=Atoms.size();
	VectorStruct v1, v2;
	if (atom1>atom2) swap(atom1, atom2);
	v1=atomVector(Atoms[atom2], Atoms[atom1]);
	vectorNormalize(v1);
	v2=v1*dLength;
	SafeAlloc(moved, natom, "moved");
	moved[atom1]=true;
	changeBondLength(Atoms, atom2, v2, moved);
}

void setBondLength(vector<AtomStruct> &Atoms, int atom1, int atom2, Real d)
{
	//printAtomInfo(Atoms[atom1]);
	//printAtomInfo(Atoms[atom2]);
	Real oldLength=atomDistance(Atoms[atom1], Atoms[atom2]);
	Real dLength=d-oldLength;
	changeBondLength(Atoms, atom1, atom2, dLength);
	cout <<"dLength= "<<dLength<<endl;
	//printAtomInfo(Atoms[atom1]);
	//printAtomInfo(Atoms[atom2]);
}

void centerAtomsInBox(vector<AtomStruct> &Atoms, string SelectionName)
{
	Real XMax, YMax, ZMax;
	Real XMin, YMin, ZMin;
	Real XMove, YMove, ZMove;
	vector<AtomStruct> tempAtoms;
	selectAtomsByName(Atoms, tempAtoms, SelectionName);
	MinMaxProtein(XMin, YMin, ZMin, XMax, YMax, ZMax, tempAtoms);
	cout <<"XMin= "<<XMin<<" YMin= "<<YMin<<" ZMin= "<<ZMin<<endl;
	cout <<"XMax= "<<XMax<<" YMax= "<<YMax<<" ZMax= "<<ZMax<<endl;
	XMove=-(XMax+XMin)*0.5;
	YMove=-(YMax+YMin)*0.5;
	ZMove=-(ZMax+ZMin)*0.5;
	//PrintPdb("/home/jouko/Density/TempPdb/reference.pdb", Atoms);
	moveAtoms(Atoms, XMove, YMove, ZMove);
	cout <<"XMove= "<<XMove<<" YMove= "<<YMove<<" ZMove= "<<ZMove<<endl;
	//PrintPdb("/home/jouko/Density/TempPdb/referenceb.pdb", Atoms);
}

void centerAtomsInBox(vector<AtomStruct> &Atoms, Real &XMove, Real &YMove, Real &ZMove)
{
	Real XMax, YMax, ZMax;
	Real XMin, YMin, ZMin;
	MinMaxProtein(XMin, YMin, ZMin, XMax, YMax, ZMax, Atoms);
	cout <<"XMin= "<<XMin<<" YMin= "<<YMin<<" ZMin= "<<ZMin<<endl;
	cout <<"XMax= "<<XMax<<" YMax= "<<YMax<<" ZMax= "<<ZMax<<endl;
	XMove=-(XMax+XMin)*0.5;
	YMove=-(YMax+YMin)*0.5;
	ZMove=-(ZMax+ZMin)*0.5;
	//PrintPdb("/home/jouko/Density/TempPdb/reference.pdb", Atoms);
	moveAtoms(Atoms, XMove, YMove, ZMove);
	cout <<"XMove= "<<XMove<<" YMove= "<<YMove<<" ZMove= "<<ZMove<<endl;
	//PrintPdb("/home/jouko/Density/TempPdb/referenceb.pdb", Atoms);
}

void centerAtomsInBox(vector<AtomStruct> &Atoms)
{
	Real XMove, YMove, ZMove;

	centerAtomsInBox(Atoms, XMove, YMove, ZMove);
}

void allignCenters(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
	Real xave1, yave1, zave1;
	Real xave2, yave2, zave2;
	Real XMove, YMove, ZMove;
	calcCenter(Atoms1, xave1, yave1, zave1);
	calcCenter(Atoms2, xave2, yave2, zave2);
	printAtomInfo(Atoms1[0]);
	printAtomInfo(Atoms2[0]);
	XMove=xave1-xave2;
	YMove=yave1-yave2;
	ZMove=zave1-zave2;
	cout <<"xave1= "<<xave1<<" yave1= "<<yave1<<" zave1= "<<zave1<<endl;
	cout <<"xave2= "<<xave2<<" yave2= "<<yave2<<" zave2= "<<zave2<<endl;
	cout <<"XMove= "<<XMove<<" YMove= "<<YMove<<" ZMove= "<<ZMove<<endl;
	moveAtoms(Atoms2, XMove, YMove, ZMove);
}

Real calcRg(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	Real dist2, distsqr=0, TotalMass, rg;
	Real dx, dy, dz;
	Real xave, yave, zave;

	TotalMass=0;
	calcCenterAll(Atoms, xave, yave, zave);
	cout <<"xave= "<<xave<<" yave= "<<yave<<" zave= "<<zave<<endl;
	for (int i=0;i<natom;i++)
	{
		dx=xave-Atoms[i].x;
		dy=yave-Atoms[i].y;
		dz=zave-Atoms[i].z;
		dist2=dx*dx+dy*dy+dz*dz;
		distsqr+=dist2*Atoms[i].Electrons;
		TotalMass+=Atoms[i].Electrons;
	}

	if (distsqr/TotalMass>0) rg=sqrt(distsqr/TotalMass);
	else rg=0;

	return rg;
}

void setNumElectrons(Real NumElectrons[])
{
	for (int i=0;i<NumAtomTypes;i++) NumElectrons[i]=0;
	NumElectrons[HYDROGEN]=1.0;
	NumElectrons[CARBON]=6.0;
	NumElectrons[NITROGEN]=7.0;
	NumElectrons[OXYGEN]=8.0;
	NumElectrons[SULFUR]=16.0;
	NumElectrons[IRON]=26.0;
	NumElectrons[ExcludedVolume]=0.0;
	NumElectrons[HydrationShell]=0.0;
	NumElectrons[PHOSPHORUS]=15.0;
	NumElectrons[CL]=18.0;
	NumElectrons[NA_Plus]=10.0;
}

void setAtomElectrons(vector<AtomStruct> &Atoms, Real NumElectrons[])
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Atoms[i].Electrons=NumElectrons[Atoms[i].atomid];
	}
}

void setAtomElectrons(vector<AtomStruct> &Atoms)
{
	Real NumElectrons[NumAtomTypes];
	setNumElectrons(NumElectrons);
	setAtomElectrons(Atoms, NumElectrons);
}

Real calcProteinElectrons(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	Real electrons=0;

	setAtomElectrons(Atoms);

	for (int i=0;i<natom;i++)
	{
		electrons+=Atoms[i].Electrons;
	}
	return electrons;
}

void applyPeriodicBoundaryConditions(vector<AtomStruct> &Atoms, Real xmin, Real ymin, Real zmin, Real xmax, Real ymax, Real zmax)
{
	int TotalParticles=Atoms.size(), times;
	Real XBoxLength, YBoxLength, ZBoxLength;

	XBoxLength=xmax-xmin;
	YBoxLength=ymax-ymin;
	ZBoxLength=zmax-zmin;
	if (XBoxLength<=0 || YBoxLength<=0 || ZBoxLength<=0)
	{
		cout <<"ERROR: One of the box dimensions is 0 or less."<<endl;
		cout <<"XBoxLength= "<<XBoxLength
		<<" YBoxLength= "<<YBoxLength
		<<" ZBoxLength= "<<ZBoxLength<<endl;
		exit(EXIT_FAILURE);
	}
	for (int n=0;n<TotalParticles;n++)
	{
		if (Atoms[n].x>xmax) 
		{
			times=int((Atoms[n].x-xmax)/XBoxLength);
			Atoms[n].x-=XBoxLength*Real(times+1);
		}
		if (Atoms[n].y>ymax) 
		{
			times=int((Atoms[n].y-ymax)/YBoxLength);
			Atoms[n].y-=YBoxLength*Real(times+1);
		}
		if (Atoms[n].z>zmax) 
		{
			times=int((Atoms[n].z-zmax)/ZBoxLength);
			Atoms[n].z-=ZBoxLength*Real(times+1);
		}
		if (Atoms[n].x<xmin)
		{
			times=int((xmin-Atoms[n].x)/XBoxLength);
			Atoms[n].x+=XBoxLength*Real(times+1);
		}
		if (Atoms[n].y<ymin)
		{
			times=int((ymin-Atoms[n].y)/YBoxLength);
			Atoms[n].y+=YBoxLength*Real(times+1);
		}
		if (Atoms[n].z<zmin)
		{
			times=int((zmin-Atoms[n].z)/ZBoxLength);
			Atoms[n].z+=ZBoxLength*Real(times+1);
		}
	}
}

void applyPeriodicBoundaryConditions(vector<AtomStruct> &Atoms, Real XBoxLength, Real YBoxLength, Real ZBoxLength)
{
	Real xmin, ymin, zmin;
	Real xmax, ymax, zmax;

	xmin=-0.5*XBoxLength;
	ymin=-0.5*YBoxLength;
	zmin=-0.5*ZBoxLength;
	xmax=0.5*XBoxLength;
	ymax=0.5*YBoxLength;
	zmax=0.5*ZBoxLength;

	applyPeriodicBoundaryConditions(Atoms, xmin, ymin, zmin, xmax, ymax, zmax);
}

void applyPeriodicBoundaryConditionsRectalinear(ProteinStruct &Protein)
{
	applyPeriodicBoundaryConditions(Protein.Atoms, Protein.XBoxLength, Protein.YBoxLength, Protein.ZBoxLength);
}


void applyPeriodicBoundaryConditionsToFractionalCoordinates(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].frac.x>1.0) Atoms[i].frac.x-=Real(int(Atoms[i].frac.x));
		if (Atoms[i].frac.y>1.0) Atoms[i].frac.y-=Real(int(Atoms[i].frac.y));
		if (Atoms[i].frac.z>1.0) Atoms[i].frac.z-=Real(int(Atoms[i].frac.z));
		if (Atoms[i].frac.x<0) Atoms[i].frac.x+=Real(int(abs(Atoms[i].frac.x)+1));
		if (Atoms[i].frac.y<0) Atoms[i].frac.y+=Real(int(abs(Atoms[i].frac.y)+1));
		if (Atoms[i].frac.z<0) Atoms[i].frac.z+=Real(int(abs(Atoms[i].frac.z)+1));
	}
}

void applyPeriodicBoundaryConditionsToFractionalCoordinates(ProteinStruct &Protein)
{
	applyPeriodicBoundaryConditionsToFractionalCoordinates(Protein.Atoms);
}

void applyPeriodicBoundaryConditions(ProteinStruct &Protein)
{
	convertToFractionalCoordinates(Protein);
	applyPeriodicBoundaryConditionsToFractionalCoordinates(Protein);
	convertToCartesianCoordinates(Protein);
}

void applyPeriodicBoundaryConditions(vector<ProteinStruct> &Proteins)
{
	int nprot=Proteins.size();

	for (int i=0;i<nprot;i++)
	{
		applyPeriodicBoundaryConditions(Proteins[i]);
	}
}

void getAtomsWithinRadius(vector<AtomStruct> &Atoms, Real xgrid, Real ygrid, Real zgrid, Real radius)
{
	int natom=Atoms.size();
	Real dist, r2;
	Real dx, dy, dz;
	vector<AtomStruct> TempAtoms;

	r2=radius*radius;
	cout <<"xgrid= "<<xgrid<<" ygrid= "<<ygrid<<" zgrid= "<<zgrid<<" r2= "<<r2<<endl;
	for (int i=0;i<natom;i++)
	{
		dx=xgrid-Atoms[i].x;
		dy=ygrid-Atoms[i].y;
		dz=zgrid-Atoms[i].z;
		dist=dx*dx+dy*dy+dz*dz;
		if (dist<r2) SafePushBack(TempAtoms, Atoms[i], "in GetAtomsWithinRadius");
	}
	Atoms=TempAtoms;
}

void image(vector<AtomStruct> &Atoms, Real &XBoxLength, Real &YBoxLength, Real &ZBoxLength, int NumPerEdge)
{
	//Takes a system and replicates it.
	AtomStruct Atom;
	int TotalParticles=Atoms.size();
	int a, b, c;
	Real Xlength, Ylength, Zlength;
	vector<AtomStruct> TempAtoms;

	Xlength=Real(NumPerEdge)*XBoxLength;
	Ylength=Real(NumPerEdge)*YBoxLength;
	Zlength=Real(NumPerEdge)*ZBoxLength;

	for (a=1;a<=NumPerEdge;a++)
	{
		for (b=1;b<=NumPerEdge;b++)
		{
			for (c=1;c<=NumPerEdge;c++)
			{
				for (int n=0;n<TotalParticles;n++)
				{
					copyAtom(Atoms[n], Atom);
					Atom.x=Atoms[n].x+Real(a)*XBoxLength;
					Atom.y=Atoms[n].y+Real(b)*YBoxLength;
					Atom.z=Atoms[n].z+Real(c)*ZBoxLength;
					SafePushBack(TempAtoms, Atom, "TempAtoms in Image");
				}
			}
		}
	}
	Atoms=TempAtoms;

	cout <<"XBoxLength="<<XBoxLength<<" YBoxLength="<<YBoxLength<<" ZBoxLength="<<ZBoxLength<<endl;
	moveAtoms(Atoms, -Real(NumPerEdge-1)*XBoxLength*0.5, -Real(NumPerEdge-1)*YBoxLength*0.5, -Real(NumPerEdge-1)*ZBoxLength*0.5);
	applyPeriodicBoundaryConditions(Atoms, -0.5*Xlength, -0.5*Ylength, -0.5*Zlength, Xlength, Ylength, Zlength);
	XBoxLength=Xlength;
	YBoxLength=Ylength;
	ZBoxLength=Zlength;
	cout <<"XBoxLength="<<XBoxLength<<" YBoxLength="<<YBoxLength<<" ZBoxLength="<<ZBoxLength<<endl;
	cout <<"TotalParticles= "<<Atoms.size()<<endl;
}

void image(vector<AtomStruct> &Atoms, int axis, Real &Length, int NumPerEdge)
{
	//Takes a system and replicates it.
	AtomStruct Atom;
	int TotalParticles2;
	int TotalParticles=Atoms.size();
	int natom;
	Real NewLength, MoveBy;
	vector<AtomStruct> TempAtoms;

	TotalParticles2=NumPerEdge*TotalParticles;
	NewLength=Real(NumPerEdge)*Length;
	SafeAlloc(TempAtoms, Atom, TotalParticles2, "in Image");
	natom=0;
	for (int a=0;a<NumPerEdge;a++)
	{
		for (int n=0;n<TotalParticles;n++)
		{
			copyAtom(Atoms[n], Atom);
			if (axis==X) Atom.x=Atoms[n].x+Real(a)*Length;
			if (axis==Y) Atom.y=Atoms[n].y+Real(a)*Length;
			if (axis==Z) Atom.z=Atoms[n].z+Real(a)*Length;
			TempAtoms[natom]=Atom;
			natom++;
		}
	}
	Atoms=TempAtoms;
	//PrintPDB("/home/jouko/Density/Pdb/BeforeMove.pdb", Atoms);
	MoveBy=-Real(NumPerEdge-1)*Length*0.5;
	if (axis==X) moveAtoms(Atoms, MoveBy, 0, 0);
	if (axis==Y) moveAtoms(Atoms, 0, MoveBy, 0);
	if (axis==Z) moveAtoms(Atoms, 0, 0, MoveBy);
	//PrintPDB("/home/jouko/Density/Pdb/AfterMove.pdb", Atoms);
	Length=NewLength;
	cout <<"TotalParticles= "<<Atoms.size()<<endl;
}

void rotateAnisou(Matrix &rotationMat, Matrix &anisouMat, Matrix &anisouOut)
{
	Matrix rotMat, inverseRotMat, transposeMat, mult;

	Safe2DAlloc(rotMat, 3, 3, "rotMat");
	Safe2DAlloc(inverseRotMat, 3, 3, "inverseRotMat");
	Safe2DAlloc(transposeMat, 3, 3, "transposeMat");
	Safe2DAlloc(mult, 3, 3, "mult");
	for (int i=0;i<3;i++)
	{
		for (int j=0;j<3;j++)
		{
			rotMat[i][j]=rotationMat[i][j];
		}
	}
	gaussJordanElimination(rotMat, inverseRotMat);
	transpose(inverseRotMat, transposeMat);
	matrixMultiply(anisouMat, inverseRotMat, mult);
	matrixMultiply(transposeMat, mult, anisouOut);
}

void rotateAnisou(vector<Real> &anisou, Matrix &rotationMat)
{
	int nanisou=anisou.size();
	Matrix anisouMat, anisouOut;

	if (nanisou==6)
	{
		Safe2DAlloc(anisouMat, 3, 3, "anisouMat");
		Safe2DAlloc(anisouOut, 3, 3, "anisouOut");
		anisouMat[0][0]=anisou[0]; 
		anisouMat[1][1]=anisou[1];
		anisouMat[2][2]=anisou[2];

		anisouMat[0][1]=anisou[3]; anisouMat[1][0]=anisou[3];
		anisouMat[0][2]=anisou[4]; anisouMat[2][0]=anisou[4];
		anisouMat[1][2]=anisou[5]; anisouMat[2][1]=anisou[5];

		rotateAnisou(rotationMat, anisouMat, anisouOut);
		
		anisou[0]=anisouOut[0][0];
		anisou[1]=anisouOut[1][1];
		anisou[2]=anisouOut[2][2];
		anisou[3]=anisouOut[0][1];
		anisou[4]=anisouOut[0][2];
		anisou[5]=anisouOut[1][2];
	}
}

void applySymmetryOperation(Vector &vIn, SymmetryOperationStruct &symmetryOperation, Vector &vOut)
{
	matrixMultiply(symmetryOperation.rotationMat, vIn, vOut); 
	for (int i=0;i<3;i++) vOut[i]+=symmetryOperation.translationVector[i];
}

AtomStruct applySymmetryOperation(AtomStruct Atom, SymmetryOperationStruct &symmetryOperation)
{
	applyRotationMatrix(Atom, symmetryOperation.rotationMat);
	moveAtom(Atom, symmetryOperation.translationVector);
	rotateAnisou(Atom.anisou, symmetryOperation.rotationMat);
	return Atom;
}

void applySymmetryOperationFast(AtomStruct &Atom, SymmetryOperationStruct &symmetryOperation)
{
	applyRotationMatrix(Atom, symmetryOperation.rotationMat);
	moveAtom(Atom, symmetryOperation.translationVector);
}

void applySymmetryOperation(AtomStruct Atom1, SymmetryOperationStruct &symmetryOperation, AtomStruct &Atom2)
{
	Atom2.x=Atom1.x;
	Atom2.y=Atom1.y;
	Atom2.z=Atom1.z;
	Atom2.BFactor=Atom1.BFactor;
	Atom2.anisou=Atom1.anisou;
	applyRotationMatrix(Atom2, symmetryOperation.rotationMat);
	moveAtom(Atom2, symmetryOperation.translationVector);
	rotateAnisou(Atom2.anisou, symmetryOperation.rotationMat);
}

void applySymmetryOperation(vector<AtomStruct> &Atoms1, SymmetryOperationStruct &symmetryOperation, vector<AtomStruct> &Atoms2)
{
	int natom=Atoms1.size();

	for (int i=0;i<natom;i++)
	{
		applySymmetryOperation(Atoms1[i], symmetryOperation, Atoms2[i]);
	}
}

void applySymmetryOperation(vector<AtomStruct> &Atoms, SymmetryOperationStruct &symmetryOperation)
{
	int natom=Atoms.size();
	AtomStruct tempAtom;
	vector<AtomStruct> tempAtoms;

	for (int i=0;i<natom;i++)
	{
		tempAtom=applySymmetryOperation(Atoms[i], symmetryOperation);
		SafePushBack(tempAtoms, tempAtom);
	}
	Atoms=tempAtoms;
}

void applySymmetryOperationFast(vector<AtomStruct> &Atoms, SymmetryOperationStruct &symmetryOperation)
{
	int natom=Atoms.size();

	if (symmetryOperation.rotationMat[X][X]==1 && symmetryOperation.rotationMat[Y][Y]==1 && symmetryOperation.rotationMat[Z][Z]==1)
	{
		moveAtoms(Atoms, symmetryOperation.translationVector);
	}
	else if (symmetryOperation.rotationMat[X][X]==-1 && symmetryOperation.rotationMat[Y][Y]==-1 && symmetryOperation.rotationMat[Z][Z]==1)
	{
		for (int i=0;i<natom;i++)
		{
			Atoms[i].x=-Atoms[i].x;
			Atoms[i].y=-Atoms[i].y;
		}
		moveAtoms(Atoms, symmetryOperation.translationVector);
	}
	else if (symmetryOperation.rotationMat[X][X]==-1 && symmetryOperation.rotationMat[Y][Y]==1 && symmetryOperation.rotationMat[Z][Z]==-1)
	{
		for (int i=0;i<natom;i++)
		{
			Atoms[i].x=-Atoms[i].x;
			Atoms[i].z=-Atoms[i].z;
		}
		moveAtoms(Atoms, symmetryOperation.translationVector);
	}
	else if (symmetryOperation.rotationMat[X][X]==1 && symmetryOperation.rotationMat[Y][Y]==-1 && symmetryOperation.rotationMat[Z][Z]==-1)
	{
		for (int i=0;i<natom;i++)
		{
			Atoms[i].y=-Atoms[i].y;
			Atoms[i].z=-Atoms[i].z;
		}
		moveAtoms(Atoms, symmetryOperation.translationVector);
	}
	else
	{
		for (int i=0;i<natom;i++)
		{
			Atoms[i]=applySymmetryOperation(Atoms[i], symmetryOperation);
		}
	}
}

vector<AtomStruct> makeUnitCell(ProteinStruct &Protein)
{
	int noperations=Protein.symmetryOperations.size();
	int natom=Protein.Atoms.size();
	int count=0;
	AtomStruct tempAtom;
	ProteinStruct UnitCell;
	//for (int i=0;i<=0;i++)
	SafeAlloc(UnitCell.Atoms, tempAtom, natom*noperations, "UnitCell");
	for (int i=0;i<noperations;i++)
	{
		for (int j=0;j<natom;j++)
		{
			UnitCell.Atoms[count]=applySymmetryOperation(Protein.Atoms[j], Protein.symmetryOperations[i]);
			UnitCell.Atoms[count].ChainName=IntToStr(i+1);
			count++;
		}
	}
	if (noperations==0) return Protein.Atoms;
	return UnitCell.Atoms;
}

void makeUnitCell(ProteinStruct &Protein, ProteinStruct &UnitCell)
{
	UnitCell=Protein;

	UnitCell.Atoms=makeUnitCell(Protein);
}

Real calcFurthestDistance(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	Real max, r2;
	Real x, y, z;
	Real dx, dy, dz;

	calcCenter(Atoms, x, y, z);
	max=0;
	for (int i=0;i<natom;i++)
	{
		dx=x-Atoms[i].x;
		dy=y-Atoms[i].y;
		dz=z-Atoms[i].z;
		r2=dx*dx+dy*dy+dz*dz;
		if (r2>max) max=r2;
	}
	return sqrt(max);
}

void makeUnitCellAndImagesLib(vector<ProteinStruct> &Proteins, vector<ProteinStruct> &allProteins)
{
	int nprot=Proteins.size();
	int noperations=Proteins[0].symmetryOperations.size();
	int index=0;
	Real r, r2, cutoff, buffer=3.0;
	Real x, y, z;
	Real dx, dy, dz;
	Real centerx, centery, centerz;
	VectorStruct a, b, c;

	a=Proteins[0].a;
	b=Proteins[0].b;
	c=Proteins[0].c;
	allProteins.resize(noperations*nprot*27);
	for (int i=0;i<noperations*nprot*27;i++)
	{
		SafeAlloc(allProteins[i].center, 3, "center");
	}
	for (int i=0;i<nprot;i++)
	{
		SafeAlloc(Proteins[i].center, 3, "center");
		calcCenter(Proteins[i].Atoms, Proteins[i].center[X], Proteins[i].center[Y], Proteins[i].center[Z]);
		for (int j=0;j<noperations;j++)
		{
			allProteins[index].Atoms=Proteins[i].Atoms;
			applySymmetryOperation(allProteins[index].Atoms, Proteins[i].symmetryOperations[j]);
			SafeAlloc(allProteins[index].center, 3, "center");
			calcCenter(allProteins[index].Atoms, allProteins[index].center[X], allProteins[index].center[Y], allProteins[index].center[Z]);
			index++;
		}
	}
	r=calcFurthestDistance(Proteins[0].Atoms);
	r+=buffer;
	cutoff=4.0*r*r;
	for (Real h=-1.0;h<1.1;h+=1.0)
	{
		for (Real k=-1.0;k<1.1;k+=1.0)
		{
			for (Real l=-1.0;l<1.1;l+=1.0)
			{
				if (!(h==0 && k==0 && l==0))
				{
					dx=h*a.x+k*b.x+l*c.x;
					dy=h*a.y+k*b.y+l*c.y;
					dz=h*a.z+k*b.z+l*c.z;
					for (int i=0;i<nprot*noperations;i++)
					{
						for (int j=0;j<nprot;j++)
						{
							centerx=allProteins[i].center[X]+dx;
							centery=allProteins[i].center[Y]+dy;
							centerz=allProteins[i].center[Z]+dz;
							x=centerx-allProteins[j].center[X];
							y=centery-allProteins[j].center[Y];
							z=centerz-allProteins[j].center[Z];
							//r2=sqrt(x*x+y*y+z*z)+2.0*buffer;
							//r2=r2*r2;
							r2=x*x+y*y+z*z;
							if (r2<cutoff)
							{
								//SafeAlloc(allProteins[index].center, 3, "center");
								allProteins[index]=allProteins[i];
								allProteins[index].center[X]=centerx;
								allProteins[index].center[Y]=centery;
								allProteins[index].center[Z]=centerz;
								moveAtoms(allProteins[index].Atoms, dx, dy, dz);
								index++;
							}
						}
					}
				}
			}
		}
	}
}

void moveIntoUnitCell(ProteinStruct &Protein)
{
	int h, k, l;
	Real dx, dy, dz;
	Real aveX, aveY, aveZ;
	Real fracX, fracY, fracZ;
	Real fracX2, fracY2, fracZ2;
	VectorStruct a, b, c;	

	a=Protein.a;
	b=Protein.b;
	c=Protein.c;

	//moveAtoms(Protein.Atoms, -1000.0, -1000.0, -1000.0);
	calcCenter(Protein.Atoms, aveX, aveY, aveZ);
	convertToFractionalCoordinates(aveX, aveY, aveZ, a, b, c, fracX, fracY, fracZ);
	convertToFractionalCoordinates(Protein.Atoms[0].x, Protein.Atoms[0].y, Protein.Atoms[0].z, a, b, c, fracX2, fracY2, fracZ2);
	h=int(abs(fracX)+0.5); k=int(abs(fracY)+0.5); l=int(abs(fracZ)+0.5);
	if (fracX<0) h=-h; if (fracY<0) k=-k; if (fracZ<0) l=-l;
	//if (fracX<0) h--; if (fracY<0) k--; if (fracZ<0) l--;
	dx=Real(h)*a.x+Real(k)*b.x+Real(l)*c.x;	
	dy=Real(h)*a.y+Real(k)*b.y+Real(l)*c.y;	
	dz=Real(h)*a.z+Real(k)*b.z+Real(l)*c.z;	

	moveAtoms(Protein.Atoms, -dx, -dy, -dz);
	convertToFractionalCoordinates(Protein.Atoms[0].x, Protein.Atoms[0].y, Protein.Atoms[0].z, a, b, c, fracX2, fracY2, fracZ2);
}

void calcBondLengths(vector<AtomStruct> &Atoms, Vector &lengths)
{
	int natom=Atoms.size(), nconnect, index;
	Real dist;
	lengths.clear();

	for (int i=0;i<natom;i++)
	{
		nconnect=Atoms[i].connectivity.size();
		for (int j=0;j<nconnect;j++)
		{
			index=Atoms[i].connectivity[j];
			dist=atomDistance(Atoms[i], Atoms[index]);
			SafePushBack(lengths, dist, "lengths");
		}
	}
}

void charmmAlias(AtomStruct &Atom)
{
	
	if (Atom.ResidueName=="HIS") Atom.ResidueName="HSE";
	if (Atom.ResidueName=="HIE") Atom.ResidueName="HSE";
	if (Atom.ResidueName=="MSE") Atom.ResidueName="MET";
	if (Atom.AtomName=="HW1") Atom.AtomName="HT1";
	if (Atom.AtomName=="HW2") Atom.AtomName="HT2";
	if (Atom.AtomName=="HW3") Atom.AtomName="HT3";
	if (Atom.AtomName=="OW1") Atom.AtomName="OT1";
	if (Atom.AtomName=="OW2") Atom.AtomName="OT2";
	if (Atom.AtomName=="OXT") Atom.AtomName="OT";
	if (Atom.ResidueName=="ILE" && Atom.AtomName=="CD1")
	{
		Atom.AtomName="CD";
	}
}

void atomAlias(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		charmmAlias(Atoms[i]);
	}
}

void setCharmmAtomNames(vector<AtomStruct> &CharmmAtoms, vector<AtomStruct> &Atoms)
{
	bool BoolPrint=false;
	int NumCharmmAtoms=CharmmAtoms.size(), NumAtoms=Atoms.size();
	cout <<"In setCharmmAtomNames"<<endl;
	cout <<"NumAtoms= "<<NumAtoms<<endl;
	for (int i=0;i<NumAtoms;i++)
	{
		//if (Atoms[i].AtomName=="H5\'\'") BoolPrint=true;
		//else BoolPrint=false;
		//printAtomInfo(Atoms[i]);
		charmmAlias(Atoms[i]);
		//if (Atoms[i].AtomName=="H5T") BoolPrint=true;
		//else BoolPrint=false;
		//printAtomInfo(Atoms[i]);
		for (int j=0;j<NumCharmmAtoms;j++)
		{
			if (BoolPrint)
			{
				cout <<endl;
				cout <<"CharmmAtomInfo"<<endl;
				printAtomInfo(CharmmAtoms[j]);
				printAtomInfo(Atoms[i]);
				cout <<"CharmmAtomName= "<<Atoms[i].CharmmAtomName<<endl;
				cout <<endl;
			}
			if (CharmmAtoms[j].ResidueName==Atoms[i].ResidueName)
			{
				if (WildCardCompare(Atoms[i].AtomName, CharmmAtoms[j].AtomName))
				{
					Atoms[i].CharmmAtomName=CharmmAtoms[j].CharmmAtomName;
					if (Atoms[i].charge==0) Atoms[i].charge=CharmmAtoms[j].charge;
					if (BoolPrint) printAtomInfo(Atoms[i]);
					if (BoolPrint) cout <<"CharmmAtomName= "<<Atoms[i].CharmmAtomName<<endl;
					break;
				}
			}
			if (BoolPrint)
			{
				cout <<"About to leave setCharmmAtomNames"<<endl;
				printAtomInfo(Atoms[i]);
				cout <<"CharmmAtomName= "<<Atoms[i].CharmmAtomName<<endl;
			}
		}
	}
	cout <<"Leaving setCharmmAtomNames"<<endl;
}

void charmmAtomNameToVDW(vector<AtomStruct> &CharmmAtoms, AtomStruct &Atom)
{
	bool BoolPrint=false;
	int NumCharmmAtoms;
	Real DefaultVDW=1.5;
	Atom.vdw=UNK_REAL;
	//if (Atom.AtomName.substr(0,2)=="HT") Atom.CharmmAtomName="HC";
	//if (Atom.AtomName.substr(0,2)=="OT") Atom.CharmmAtomName="OC";
	NumCharmmAtoms=CharmmAtoms.size();
	//cout <<"In charmmAtomNameToVDW"<<endl;
	//if (Atom.ResidueName=="TRP" && Atom.AtomName=="CD1") BoolPrint=true;
	//else BoolPrint=false;

	for (int j=0;j<NumCharmmAtoms;j++)
	{
		if (BoolPrint) 
		{
			printAtomInfo(Atom);
			cout <<"Atom.CharmmAtomName= "<<Atom.CharmmAtomName<<endl;
			printAtomInfo(CharmmAtoms[j]);
			cout <<"CharmmAtoms["<<j<<"].CharmmAtomName= "<<CharmmAtoms[j].CharmmAtomName<<endl;
			cout <<endl;
		}

		//cout <<"Atom.CharmmAtomName= "<<Atom.CharmmAtomName<<" CharmmAtoms["<<j<<"].CharmmAtomName== "<<CharmmAtoms[j].CharmmAtomName<<endl;
		if (WildCardCompare(CharmmAtoms[j].CharmmAtomName, Atom.CharmmAtomName))
		{
			Atom.vdw=CharmmAtoms[j].vdw;
			//cout <<"ERROR: Need to uncomment out next line"<<endl;
			//exit(EXIT_FAILURE);
			//Atom.epsilon=CharmmAtoms[j].epsilon;
			Atom.vdw14=CharmmAtoms[j].vdw14;
			Atom.epsilon14=CharmmAtoms[j].epsilon14;
			//Atom.charge=CharmmAtoms[j].charge;
			break;
		}
	}
	if (Atom.vdw==UNK_REAL)
	{
		Atom.vdw=DefaultVDW;
		cout <<"Warning.  Unable to set vdw for atom "<<endl;
		printAtomInfo(Atom);
		cout <<"Atom.CharmmAtomName= "<<Atom.CharmmAtomName<<endl;
		cout <<"Setting vdw to "<<DefaultVDW<<endl;
	}
}

void charmmAtomNamesToVDW(vector<AtomStruct> &CharmmAtoms, vector<AtomStruct> &Atoms)
{
	int NumAtoms=Atoms.size();
	for (int i=0;i<NumAtoms;i++)
	{
		charmmAtomNameToVDW(CharmmAtoms, Atoms[i]);
		//cout <<"vdw= "<<Atoms[i].vdw<<endl;
		//printAtomInfo(Atoms[i]);
	}
}

void scaleVDW(Real VDWScale, Real VDWOffSet, vector<AtomStruct> &Atoms)
{
	int i, natom=Atoms.size();
	for (i=0;i<natom;i++)
	{
		Atoms[i].vdw=Atoms[i].vdw*VDWScale+VDWOffSet;
	}
}

void addChargeRadii(vector<AtomStruct> &Atoms, Real ChargeRadiiScale)
{
	int natom=Atoms.size();
	//cout <<"In AddChargeRadii"<<endl;
	for (int i=0;i<natom;i++)
	{
		Atoms[i].HydrationRadius+=abs(Atoms[i].charge)*ChargeRadiiScale;
		//cout <<"Atoms["<<i<<"].charge= "<<Atoms[i].charge<<" ChargeRadiiScale= "<<ChargeRadiiScale<<endl;
		//cout <<"Atoms["<<i<<"].HydrationRadius= "<<Atoms[i].HydrationRadius<<endl;
	}
}

void setVDW(string PrmFile, string RtfFile, vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> CharmmAtoms, TempCharmmAtoms;
	ReadRtfFile(RtfFile, CharmmAtoms);
	ReadPrmFile(PrmFile, TempCharmmAtoms);
	setCharmmAtomNames(CharmmAtoms, Atoms);
	charmmAtomNamesToVDW(TempCharmmAtoms, Atoms);
}

void setVDW(vector<AtomStruct> &Atoms, Real vdw)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++) Atoms[i].vdw=vdw;
}

void getCharges(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	int npsf=PsfAtoms.size();
	if (natom>npsf)
	{
		cout <<"ERROR: Atom number mismatch."<<endl;
		cout <<"natom= "<<natom<<" PsfAtoms.size()= "
			<<PsfAtoms.size()<<endl;
		exit(EXIT_FAILURE);
	}

	for (int i=0;i<natom;i++)
	{
		Atoms[i].charge=PsfAtoms[i].charge;
	}
}

void getCharges(string PsfFile, vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> PsfAtoms;
	ReadPsf(PsfFile, PsfAtoms);
	getCharges(PsfAtoms, Atoms);
}

void getCharmmAtomNames(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	for (int i=0;i<natom;i++)
	{
		Atoms[i].CharmmAtomName=PsfAtoms[i].CharmmAtomName;
	}
}

void getCharmmAtomNames(string PsfFile, vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> PsfAtoms;
	ReadPsf(PsfFile, PsfAtoms);
	getCharmmAtomNames(PsfAtoms, Atoms);
}

void getConnectivity(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	//cout <<"natom= "<<natom<<endl;
	for (int i=0;i<natom;i++)
	{
		Atoms[i].connectivity=PsfAtoms[i].connectivity;
	}
	cout <<"Leaving getConnectivity"<<endl;
}

void getConnectivity(string PsfFile, vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> PsfAtoms;
	cout <<"About to enter ReadPsf"<<endl;
	ReadPsf(PsfFile, PsfAtoms);
	cout <<"Read PsfFile"<<endl;
	getConnectivity(PsfAtoms, Atoms);
}

void getPsfInfo(string PsfFile, vector<AtomStruct> &Atoms)
{
	vector<AtomStruct> PsfAtoms;
	ReadPsf(PsfFile, PsfAtoms);
	getConnectivity(PsfAtoms, Atoms);
	getCharmmAtomNames(PsfAtoms, Atoms);
	getCharges(PsfAtoms, Atoms);
}

void setDefaultConnectivity(vector<AtomStruct> &Atoms)
{
        int natom=Atoms.size();

        for (int i=0;i<natom;i++) Atoms[i].connectivity.clear();

        for (int i=0;i<natom;i++)
        {
                for (int j=0;j<natom;j++)
                {
                        if (Atoms[i].AtomName=="N" && Atoms[j].AtomName=="CA")
                        {
                                if (Atoms[i].ResidueNum==Atoms[j].ResidueNum)
                                {
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
                                }
                        }
                        if (Atoms[i].AtomName=="CA" && Atoms[j].AtomName=="C")
                        {
                                if (Atoms[i].ResidueNum==Atoms[j].ResidueNum)
                                {
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
                                }
                        }
                        if (Atoms[i].AtomName=="CA" && Atoms[j].AtomName=="CB")
                        {
                                if (Atoms[i].ResidueNum==Atoms[j].ResidueNum)
                                {
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
                                }
                        }
                        if (Atoms[i].AtomName=="C" && Atoms[j].AtomName=="O")
                        {
                                if (Atoms[i].ResidueNum==Atoms[j].ResidueNum)
                                {
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
                                }
                        }
                        if (Atoms[i].AtomName=="C" && Atoms[j].AtomName=="N")
                        {
                                if (Atoms[i].ResidueNum+1==Atoms[j].ResidueNum)
                                {
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
                                }
                        }
			if (!isCB(Atoms[i]) && Atoms[j].AtomName=="CB")
			{
				if (Atoms[i].ResidueNum==Atoms[j].ResidueNum)
				{
                                        SafePushBack(Atoms[i].connectivity, j, "bond");
                                        SafePushBack(Atoms[j].connectivity, i, "bond");
				}
			}
                }
        }
}


void getFirstLastResidue(vector<AtomStruct> &Atoms, int &FirstResidue, int &LastResidue)
{
	int natom=Atoms.size();
	FirstResidue=Atoms[0].ResidueNum;
	for (int i=natom-1;i>=0;i--)
	{
		if (isProtein(Atoms[i])) 
		{
			LastResidue=Atoms[i].ResidueNum;
			break;
		}
	}
}

void getLastResidue(vector<AtomStruct> &Atoms, int &LastResidue)
{
	int natom=Atoms.size();
	LastResidue=Atoms[natom-1].ResidueNum;
}

void getFirstResidue(vector<AtomStruct> &Atoms, int &FirstResidue)
{
	FirstResidue=Atoms[0].ResidueNum;
}

bool hasNA(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].AtomName=="O1P") return true;
	}
	return false;
}

bool isCompleteResidue(vector<AtomStruct> &Atoms, int ResidueNum)
{
	bool hasCA=false, hasN=false, hasC=false;
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueNum==ResidueNum)
		{
			if (Atoms[i].AtomName=="CA") hasCA=true;
			if (Atoms[i].AtomName=="N") hasN=true;
			if (Atoms[i].AtomName=="C") hasC=true;
		}
	}
	//cout <<"hasCA= "<<hasCA<<" hasN= "<<hasN<<" hasC= "<<hasC<<endl;
	return (hasCA && hasN && hasC);
}

bool hasChainBreaks(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	int FirstResidue=UNK_INT, LastResidue=UNK_INT;

	getFirstLastResidue(Atoms, FirstResidue, LastResidue);
	for (int i=1;i<natom;i++)
	{
		if (Atoms[i].ResidueNum-Atoms[i-1].ResidueNum>1)
		{
			return true;
		}
	}
	for (int i=FirstResidue;i<=LastResidue;i++)
	{
		if (!isCompleteResidue(Atoms, i)) return true;
	}
	return false;
}

bool isValidProtein(vector<AtomStruct> &Atoms)
{
	if (Atoms.size()==0) return false;
	else if (hasChainBreaks(Atoms)) return false;

	return true;
}

bool isValidProtein(string pdbFile)
{
	vector<AtomStruct> Atoms;
	readPdb(pdbFile, Atoms);

	return isValidProtein(Atoms);
}

void renumberResidues(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	int FirstResidue=Atoms[0].ResidueNum;

	for (int i=0;i<natom;i++)
	{
		Atoms[i].ResidueNum-=(FirstResidue-1);
	}

}

void makeResiduesConsecutive(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();
	int residue;
	int firstResidue;
	vector<AtomStruct> tempAtoms;
	tempAtoms=Atoms;
	residue=Atoms[0].ResidueNum;
	firstResidue=residue;

	for (int i=1;i<natom;i++)
	{
		if (Atoms[i].ResidueNum!=Atoms[i-1].ResidueNum)
		{
			residue++;
		}
		tempAtoms[i].ResidueNum=residue;
	}
	for (int i=0;i<natom;i++)
	{
		tempAtoms[i].ResidueNum-=(firstResidue-1);
	}
	Atoms=tempAtoms;
}

void setWeightToBFactor(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Atoms[i].weight=Atoms[i].BFactor;
	}
}

void scaleBFactors(ProteinStruct &Protein, Real scale)
{
	int natom=Protein.Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Protein.Atoms[i].BFactor*=scale;
	}
}

void setBFactors(ProteinStruct &Protein, Real BFactor)
{
	int natom=Protein.Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Protein.Atoms[i].BFactor=BFactor;
	}
}

string getFasta(ProteinStruct &Protein)
{
	string fasta="";
	int natom=Protein.Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Protein.Atoms[i].AtomName=="CA")
		{
			fasta+=OneLetterCode(Protein.Atoms[i].residueid);
		}
	}
	return fasta;
}

string getSubsequence(ProteinStruct &Protein, int residue1, int residue2)
{
	string fasta=getFasta(Protein);
	return fasta.substr(residue1, residue2-residue1);
}

void getSegment(vector<AtomStruct> &Atoms1, int start, int end, vector<AtomStruct> &Atoms2)
{
	int natom=Atoms1.size();

	if (start>=natom || end>=natom || start>end || start<0 || end<0)
	{
		printAtomInfo(Atoms1[start]);
		printAtomInfo(Atoms1[end]);
		string errorStr="natom= "+toStr(natom)+" start= "+toStr(start)+" end= "+toStr(end);
		error(errorStr, __LINE__, __FILE__);
	}

	Atoms2.clear();

	for (int i=start;i<=end;i++)
	{
		SafePushBack(Atoms2, Atoms1[i], "Atoms2");
	}
}

VectorStruct icToXYZ(VectorStruct &v1, VectorStruct &v2, Real angle, Real length, Real dihedralAngle)
{
	VectorStruct CrossProduct;
	VectorStruct v3, v4, v5;
	VectorStruct normV2;

	CrossProduct=calcCrossProduct(v1,v2);
	vectorNormalize(CrossProduct);
	v3=rotateAroundVector(v2, angle, CrossProduct);
	vectorNormalize(v2, normV2);
	v4=rotateAroundVector(v3, dihedralAngle, normV2);
	v5=setLength(v4, length);

	return v5;
}

VectorStruct icToXYZ(VectorStruct &pos1, VectorStruct &pos2, VectorStruct &pos3, Real angle, Real length, Real dihedralAngle)
{
	Real inner=pi-angle;
	VectorStruct v1, v2;
	VectorStruct pOut,vOut;
	v1=pos2-pos1;
	v2=pos3-pos2;

	vOut=icToXYZ(v1, v2, inner, length, dihedralAngle);
	pOut=pos3+vOut;
	return pOut;
}

void icToXYZ(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3, AtomStruct &Atom, Real angle, Real length, Real dihedralAngle)
{
	VectorStruct pos1, pos2, pos3;
	VectorStruct pOut;
	
	pos1.x=Atom1.x;
	pos1.y=Atom1.y;
	pos1.z=Atom1.z;
	
	pos2.x=Atom2.x;
	pos2.y=Atom2.y;
	pos2.z=Atom2.z;
	
	pos3.x=Atom3.x;
	pos3.y=Atom3.y;
	pos3.z=Atom3.z;

	pOut=icToXYZ(pos1, pos2, pos3, angle, length, dihedralAngle);

	Atom.x=pOut.x;
	Atom.y=pOut.y;
	Atom.z=pOut.z;
}

void rotateToAlignWithYAxis(vector<AtomStruct> &Atoms, VectorStruct &v)
{
	Real angle;
	VectorStruct y, cp;

	y.x=0;
	y.y=1.0;
	y.z=0;
	angle=calcAngle(v, y);
	cp=calcCrossProduct(v, y);
	vectorNormalize(cp);
	rotateAtomsAroundVector(Atoms, cp, 0, 0, 0, angle);
}

void convertToFractionalCoordinates(Real dx, Real dy, Real dz, VectorStruct &a, VectorStruct &b, VectorStruct &c, Real &fracX, Real &fracY, Real &fracZ)
{
        Vector variable;
        Matrix coefficient;

        SafeAlloc(variable, 3, "variable");
        Safe2DAlloc(coefficient, 4, 3, "coefficient");
        //cout <<"a= "<<vectorToStr(a)<<endl;
        //cout <<"b= "<<vectorToStr(b)<<endl;
        //cout <<"c= "<<vectorToStr(c)<<endl;
        coefficient[0][0]=a.x;
        coefficient[1][0]=b.x;
        coefficient[2][0]=c.x;
        coefficient[3][0]=dx;

        coefficient[0][1]=a.y;
        coefficient[1][1]=b.y;
        coefficient[2][1]=c.y;
        coefficient[3][1]=dy;

 	coefficient[0][2]=a.z;
        coefficient[1][2]=b.z;
        coefficient[2][2]=c.z;
        coefficient[3][2]=dz;

        equation(coefficient, variable);

        fracX=variable[0];
        fracY=variable[1];
        fracZ=variable[2];

        if (isinf(fracX) || isnan(fracX))
        {
                string errorStr;
                printMatrix(coefficient);
                errorStr="Unable to convert to fractional coordinates";
                error(errorStr, __LINE__, __FILE__);
        }
}

void convertToFractionalCoordinates(Real dx, Real dy, Real dz, Matrix &cartToFrac, Real &fracX, Real &fracY, Real &fracZ)
{
	int Size1, Size2;

	Get2DVectorSize(cartToFrac, Size1, Size2);
	if (Size1==3 && Size2==3)
	{
		fracX=dx*cartToFrac[X][X]+dy*cartToFrac[X][Y]+dz*cartToFrac[X][Z];
		fracY=dx*cartToFrac[Y][X]+dy*cartToFrac[Y][Y]+dz*cartToFrac[Y][Z];
		fracZ=dx*cartToFrac[Z][X]+dy*cartToFrac[Z][Y]+dz*cartToFrac[Z][Z];
	}
	else
	{
		error("cartToFrac not 3 by 3 matrix. It is "+IntToStr(Size1)+" by "+IntToStr(Size2), __LINE__, __FILE__);
	}
}

void convertToFractionalCoordinates(Real dx, Real dy, Real dz, ProteinStruct &Protein, Real &fracX, Real &fracY, Real &fracZ)
{
	int Size1, Size2;

	Get2DVectorSize(Protein.cartToFrac, Size1, Size2);
        if (Size1==3 && Size2==3)
	{
		convertToFractionalCoordinates(dx, dy, dz, Protein.cartToFrac, fracX, fracY, fracZ);
	}
	else
	{
		convertToFractionalCoordinates(dx, dy, dz, Protein.a, Protein.b, Protein.c, fracX, fracY, fracZ);
	}
}

void convertToFractionalCoordinates(vector<AtomStruct> &Atoms, Matrix &cartToFrac)
{
	int natom=Atoms.size();
	int Size1, Size2;
	Real x, y, z;
	Real fracX, fracY, fracZ;

	Get2DVectorSize(cartToFrac, Size1, Size2);
	if (Size1!=3 && Size2!=3)
	{
		string errorStr="cartToFrac matrix should be 3x3 instead it is";
		errorStr+=" "+IntToStr(Size1)+"x"+IntToStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	for (int i=0;i<natom;i++)
	{
		x=Atoms[i].x;
		y=Atoms[i].y;
		z=Atoms[i].z;
		fracX=Atoms[i].frac.x;
		fracY=Atoms[i].frac.y;
		fracZ=Atoms[i].frac.z;
		convertToFractionalCoordinates(x, y, z, cartToFrac, fracX, fracY, fracZ);
		Atoms[i].frac.x=fracX;
		Atoms[i].frac.y=fracY;
		Atoms[i].frac.z=fracZ;
	}
}

void convertToFractionalCoordinates(vector<AtomStruct> &Atoms, VectorStruct &a, VectorStruct &b, VectorStruct &c)
{
	int natom=Atoms.size();
	Real x, y, z;
	Real fracX, fracY, fracZ;

	for (int i=0;i<natom;i++)
	{
		x=Atoms[i].x;
		y=Atoms[i].y;
		z=Atoms[i].z;
		fracX=Atoms[i].frac.x;
		fracY=Atoms[i].frac.y;
		fracZ=Atoms[i].frac.z;
		convertToFractionalCoordinates(x, y, z, a, b, c, fracX, fracY, fracZ);
		Atoms[i].frac.x=fracX;
		Atoms[i].frac.y=fracY;
		Atoms[i].frac.z=fracZ;
	}
}

void convertToFractionalCoordinates(ProteinStruct &Protein)
{
	int Size=Protein.cartToFrac.size();

	if (Size==3) convertToFractionalCoordinates(Protein.Atoms, Protein.cartToFrac);
	else convertToFractionalCoordinates(Protein.Atoms, Protein.a, Protein.b, Protein.c);
}

void convertToCartesianCoordinates(Real fracX, Real fracY, Real fracZ, Matrix &fracToCart, Real &dx, Real &dy, Real &dz)
{
	int Size1, Size2;

	Get2DVectorSize(fracToCart, Size1, Size2);
	if (Size1==3 && Size2==3)
	{
		dx=fracX*fracToCart[X][X]+fracY*fracToCart[X][Y]+fracZ*fracToCart[X][Z];
		dy=fracX*fracToCart[Y][X]+fracY*fracToCart[Y][Y]+fracZ*fracToCart[Y][Z];
		dz=fracX*fracToCart[Z][X]+fracY*fracToCart[Z][Y]+fracZ*fracToCart[Z][Z];
	}
	else
	{
		error("fracToCart not 3 by 3 matrix. It is "+IntToStr(Size1)+" by "+IntToStr(Size2), __LINE__, __FILE__);
	}
}


void convertToCartesianCoordinatesUsingMatrix(ProteinStruct &Protein)
{
	int natom=Protein.Atoms.size();
	int Size1, Size2;
	Real x, y, z;
	Real fracX, fracY, fracZ;
	Matrix fracToCart;

	Get2DVectorSize(Protein.cartToFrac, Size1, Size2);
	if (Size1!=3 && Size2!=3)
	{
		string errorStr="cartToFrac matrix should be 3x3 instead it is";
		errorStr+=" "+IntToStr(Size1)+"x"+IntToStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	calcInverseMatrix(Protein.cartToFrac, fracToCart);

	for (int i=0;i<natom;i++)
	{
		x=Protein.Atoms[i].x;
		y=Protein.Atoms[i].y;
		z=Protein.Atoms[i].z;
		fracX=Protein.Atoms[i].frac.x;
		fracY=Protein.Atoms[i].frac.y;
		fracZ=Protein.Atoms[i].frac.z;
		convertToFractionalCoordinates(fracX, fracY, fracZ, fracToCart, x, y, z);
		Protein.Atoms[i].x=x;
		Protein.Atoms[i].y=y;
		Protein.Atoms[i].z=z;
	}
}

void convertToCartesianCoordinates(ProteinStruct &Protein)
{
	int natom=Protein.Atoms.size();
	VectorStruct a, b, c;
	a=Protein.a;
	b=Protein.b;
	c=Protein.c;
	for (int i=0;i<natom;i++)
	{
		Protein.Atoms[i].x=Protein.Atoms[i].frac.x*a.x+Protein.Atoms[i].frac.y*b.x+Protein.Atoms[i].frac.z*c.x;
		Protein.Atoms[i].y=Protein.Atoms[i].frac.x*a.y+Protein.Atoms[i].frac.y*b.y+Protein.Atoms[i].frac.z*c.y;
		Protein.Atoms[i].z=Protein.Atoms[i].frac.x*a.z+Protein.Atoms[i].frac.y*b.z+Protein.Atoms[i].frac.z*c.z;
	}
}

void calcUnitCellVectors(Real XBoxLength, Real YBoxLength, Real ZBoxLength, Real alpha, Real beta, Real gamma, VectorStruct &a, VectorStruct &b, VectorStruct &c)
{
	alpha=alpha*DEGREE_TO_RAD;
        beta=beta*DEGREE_TO_RAD;
        gamma=gamma*DEGREE_TO_RAD;

        a.x=XBoxLength;
        a.y=0;
        a.z=0;

        b.x=cos(gamma);
        b.y=sin(gamma);
        b.z=0;

        c.x=cos(beta);
        c.y=(cos(alpha)-b.x*cos(beta))/b.y;
        c.z=sqrt(1.0-c.x*c.x-c.y*c.y);

        b=setLength(b, YBoxLength);
        c=setLength(c, ZBoxLength);
}

void calcUnitCellVectors(ProteinStruct &Protein)
{
	Real XBoxLength, YBoxLength, ZBoxLength;
	Real alpha, beta, gamma;

	XBoxLength=Protein.XBoxLength;
	YBoxLength=Protein.YBoxLength;
	ZBoxLength=Protein.ZBoxLength;
	alpha=Protein.alpha;
	beta=Protein.beta;
	gamma=Protein.gamma;


	calcUnitCellVectors(XBoxLength, YBoxLength, ZBoxLength, alpha, beta, gamma, Protein.a, Protein.b, Protein.c);
}

void calcFracToCart(VectorStruct &a, VectorStruct &b, VectorStruct &c, Matrix &fracToCart)
{
	int Size1=0, Size2=0;

	Size1=fracToCart.size();

	if (Size1!=0)
	{
		Size2=fracToCart[0].size();
	}
	if (Size1!=3 || Size2!=3) Safe2DAlloc(fracToCart, 3, 3, "fracToCart");

	fracToCart[X][X]=a.x;
	fracToCart[X][Y]=b.x;
	fracToCart[X][Z]=c.x;

	fracToCart[Y][X]=a.y;
	fracToCart[Y][Y]=b.y;
	fracToCart[Y][Z]=c.y;

	fracToCart[Z][X]=a.z;
	fracToCart[Z][Y]=b.z;
	fracToCart[Z][Z]=c.z;
}

void calcCartToFrac(ProteinStruct &Protein)
{
	calcUnitCellVectors(Protein);
	calcFracToCart(Protein.a, Protein.b, Protein.c, Protein.fracToCart);
	calcInverseMatrix(Protein.fracToCart, Protein.cartToFrac);
}

/*
void findCoreAtoms(vector<AtomStruct> &Atoms)
{
	vector<bool> core;
	int natom, nresidue, n;
	int coreCriteria=7;
	Real dist, cutOff=6.0;
	Real cutOff2=cutOff*cutOff;
	vector<AtomStruct> caAtoms;
	vector<AtomStruct> coreAtoms, surfaceAtoms;

	caAtoms=Atoms;
	removeNonCA(caAtoms);
	nresidue=caAtoms.size();
	natom=Atoms.size();
	SafeAlloc(core, false, nresidue, "core");

	for (int i=0;i<nresidue;i++)
	{
		n=0;
		for (int j=0;j<nresidue;j++)
		{
			if (i!=j)
			{
				dist=atomSquareDistance(caAtoms[i], caAtoms[j]);
				if (dist<cutOff2) n++;
			}
			if (n>=coreCriteria) core[i]=true;
		}
	}
	for (int i=0;i<nresidue;i++)
	{
		if (core[i])
		{
			for (int j=0;j<natom;j++)
			{
				if (Atoms[j].ResidueNum==i) Atoms[j].core=true;
			}
		}
	}
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].core) SafePushBack(coreAtoms, Atoms[i], "coreAtoms");
		else SafePushBack(surfaceAtoms, Atoms[i], "surfaceAtoms");
	}
	//removeNonCA(coreAtoms);
	//removeNonCA(surfaceAtoms);
	PrintPdb("/home/jouko/project/mr/pdb_out/coreAtoms.pdb", coreAtoms);
	PrintPdb("/home/jouko/project/mr/pdb_out/surfaceAtoms.pdb", surfaceAtoms);
}
*/

void findCoreAtoms(vector<AtomStruct> &Atoms, Real defaultVDW, Real ASACutoff)
{
	//For Calpha atoms
	//Probably need a better method
	int natom, nresidue;
	Real maxASA;
	vector<Real> asa, zscore;
	Matrix distMat;
	vector<AtomStruct> caAtoms;
	vector<AtomStruct> coreAtoms, surfaceAtoms;

	maxASA=4.0*pi*defaultVDW*defaultVDW;
	caAtoms=Atoms;
	removeNonCA(caAtoms);
	setVDW(caAtoms, defaultVDW);
	calcDistanceMatrix(caAtoms, distMat);
	nresidue=caAtoms.size();
	natom=Atoms.size();
	SafeAlloc(asa, nresidue, "asa");
	for (int i=0;i<nresidue;i++) asa[i]=ASAOfAtomLCPO(caAtoms, i, distMat);
	for (int i=0;i<nresidue;i++)
	{
		if (asa[i]/maxASA<ASACutoff)
		{
			for (int j=0;j<natom;j++)
			{
				if (Atoms[j].ResidueNum==i) Atoms[j].core=true;
			}
		}
	}
/*
	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].core) SafePushBack(coreAtoms, Atoms[i], "coreAtoms");
		else SafePushBack(surfaceAtoms, Atoms[i], "surfaceAtoms");
	}
	removeNonCA(coreAtoms);
	removeNonCA(surfaceAtoms);
	PrintPdb("/home/jouko/project/mr/pdb_out/coreAtoms.pdb", coreAtoms);
	PrintPdb("/home/jouko/project/mr/pdb_out/surfaceAtoms.pdb", surfaceAtoms);
*/
}

void findCoreAtomsSharke(vector<AtomStruct> &Atoms, Real defaultVDW, Real ASACutoff)
{
	//For Calpha atoms
	//Probably need a better method
	int natom, nresidue;
	Real maxASA;
	vector<Real> asa, zscore;
	vector<AtomStruct> caAtoms;
	vector<AtomStruct> coreAtoms, surfaceAtoms;

	maxASA=4.0*pi*(defaultVDW+probeRadius)*(defaultVDW+probeRadius);
	caAtoms=Atoms;
	removeNonCA(caAtoms);
	setVDW(caAtoms, defaultVDW);
	nresidue=caAtoms.size();
	natom=Atoms.size();
	SafeAlloc(asa, nresidue, "asa");
	for (int i=0;i<nresidue;i++) asa[i]=ASAOfAtomSharkeRupley(caAtoms, i);
	for (int i=0;i<nresidue;i++)
	{
		if (asa[i]/maxASA<=ASACutoff)
		{
			for (int j=0;j<natom;j++)
			{
				if (Atoms[j].ResidueNum-1==i) 
				{
					Atoms[j].core=true;
				}
			}
		}
	}
}

void findCoreAtoms(vector<AtomStruct> &Atoms)
{
	Real ASAcutoff=0.20;
	Real defaultVDW=4.0;
	findCoreAtoms(Atoms, defaultVDW, ASAcutoff);
}

bool isBonded(AtomStruct &Atom, int i)
{
	int nconnect=Atom.connectivity.size();
	for (int j=0;j<nconnect;j++) if (Atom.connectivity[j]==i) return true;
	return false;	
}

bool areSomeOccupanciesNonZero(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].Occupancy!=0) return true;
	}
	return false;
}

void checkOccupancies(vector<AtomStruct> &Atoms)
{
	int natom=Atoms.size();

	if (!areSomeOccupanciesNonZero(Atoms))
	{
		for (int i=0;i<natom;i++)
		{
			Atoms[i].Occupancy=1.0;
		}
	}
}

void parsePdb(string PdbFile, ProteinStruct &Protein, bool BoolRemoveWaters, bool BoolRemoveUnknownResidues)
{
	bool BoolRenumberResidues=false, BoolRemoveDuplicates=false;
	string str;
	cout <<"In parsePdb"<<endl;
	cout <<"PdbFile= "<<PdbFile<<endl;
	cout <<"BoolRemoveWaters= "<<BoolRemoveWaters<<endl;
	cout <<"BoolRemoveUnknownResidues= "<<BoolRemoveUnknownResidues<<endl;
	cout <<"PdbFile= "<<PdbFile<<endl;
	str=GetExtension(PdbFile);
	cout <<"str= "<<str<<" PdbFile= "<<PdbFile<<endl;
	if (str=="pdb") readPdb(PdbFile, Protein);
	else if (str=="gro") readGro(PdbFile, Protein.Atoms);
	else ReadXYZ(PdbFile, Protein.Atoms);
	cout <<"Atoms.size()= "<<Protein.Atoms.size()<<endl;
	if (BoolRemoveWaters) removeWaters(Protein.Atoms);
	cout <<"Atoms.size()= "<<Protein.Atoms.size()<<endl;
	if (BoolRemoveUnknownResidues) removeUnknownResidues(Protein.Atoms);
	if (BoolRemoveDuplicates) removeDuplicateAtoms(Protein.Atoms);
	if (BoolRenumberResidues) renumberResidues(Protein.Atoms);
	cout <<"Atoms.size()= "<<Protein.Atoms.size()<<endl;
	assignAtomIDs(Protein.Atoms);
	GetResidueIDs(Protein.Atoms);
	getAtomType(Protein.Atoms);
	getAtomType2(Protein.Atoms);
	checkOccupancies(Protein.Atoms);
}

void readFasta(string fastaFile, vector<string> &residues)
{
	string residue;
	vector<string> lines;
	int nline, length;

	readLines(fastaFile, lines, "fastaFile");

	nline=lines.size();
	cout <<"nline= "<<nline<<endl;
	for (int i=0;i<nline;i++)
	{
		cout <<lines[i]<<endl;
		if (lines[i].substr(0,1)!=">")
		{
			length=lines[i].length();
			cout <<"length= "<<length<<endl;
			for (int j=0;j<length;j++)
			{
				residue=OneLetterToThreeLetterCode(lines[i].substr(j,1));
				cout <<residue<<endl;
				SafePushBack(residues, residue, "residues");
			}

		}
	}
	cout <<"Leaving readFasta"<<endl;
}

void getEquivalentAtoms(ProteinStruct &Protein1, ProteinStruct &Protein2, vector< vector<int> > &pairs)
{
        string name;
        int natom=Protein1.Atoms.size();
        int residue, index;
        vector<int> temp;

        SafeAlloc(temp, 2, "temp");
        for (int i=0;i<natom;i++)
        {
                residue=Protein1.Atoms[i].ResidueNum;
                name=Protein1.Atoms[i].AtomName;
                index=getAtomIndex(Protein2.Atoms, residue, name);
                if (index!=UNK_INT)
                {
                        temp[0]=i;
                        temp[1]=index;
                        SafePushBack(pairs, temp, "pairs");
                }
        }
}

void getMatchingAtoms(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, vector<AtomStruct> &AtomsOut)
{
        int natom1=Atoms1.size();
        int natom2=Atoms2.size();
        //int firstResidue1;
        //int firstResidue2;

        if (natom1==0 || natom2==0)
        {
                string errorStr="natom1= "+toStr(natom1);
                errorStr+=" natom2= "+toStr(natom2);
        }
        //firstResidue1=Atoms1[0].ResidueNum;
        //firstResidue2=Atoms2[0].ResidueNum;
        AtomsOut.clear();
        for (int i=0;i<natom1;i++)
        {
                for (int j=0;j<natom2;j++)
                {
			if (Atoms1[i].AtomName==Atoms2[j].AtomName && Atoms1[i].ResidueNum==Atoms2[j].ResidueNum && Atoms1[i].ChainName==Atoms2[j].ChainName)
			{
				cout <<endl;
				printAtomInfo(Atoms1[i]);
				printAtomInfo(Atoms2[j]);
				cout <<endl;
			}
                        if (Atoms1[i].AtomName==Atoms2[j].AtomName && Atoms1[i].ResidueNum==Atoms2[j].ResidueNum && Atoms1[i].ChainName==Atoms2[j].ChainName && Atoms1[i].ResidueName==Atoms2[j].ResidueName)
                        //if (Atoms1[i].AtomName==Atoms2[j].AtomName && Atoms1[i].ResidueNum-firstResidue1==Atoms2[j].ResidueNum-firstResidue2 && Atoms1[i].ChainName==Atoms2[j].ChainName && Atoms1[i].ResidueName==Atoms2[j].ResidueName)
                        {
				//cout <<"SafePushBack"<<endl;
                                SafePushBack(AtomsOut, Atoms1[i], "AtomsOut");
                        }
                }
        }
}

void getMatchingAtomsIgnoreChain(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, vector<AtomStruct> &AtomsOut)
{
        int natom1=Atoms1.size();
        int natom2=Atoms2.size();
        //int firstResidue1;
        //int firstResidue2;

        if (natom1==0 || natom2==0)
        {
                string errorStr="natom1= "+toStr(natom1);
                errorStr+=" natom2= "+toStr(natom2);
        }
        //firstResidue1=Atoms1[0].ResidueNum;
        //firstResidue2=Atoms2[0].ResidueNum;
        AtomsOut.clear();
        for (int i=0;i<natom1;i++)
        {
                for (int j=0;j<natom2;j++)
                {
                        if (Atoms1[i].AtomName==Atoms2[j].AtomName && Atoms1[i].ResidueNum==Atoms2[j].ResidueNum && Atoms1[i].ResidueName==Atoms2[j].ResidueName)
                        //if (Atoms1[i].AtomName==Atoms2[j].AtomName && Atoms1[i].ResidueNum-firstResidue1==Atoms2[j].ResidueNum-firstResidue2 && Atoms1[i].ChainName==Atoms2[j].ChainName && Atoms1[i].ResidueName==Atoms2[j].ResidueName)
                        {
                                SafePushBack(AtomsOut, Atoms1[i], "AtomsOut");
                        }
                }
        }
}

void renameResidues(vector<AtomStruct> &Atoms, string oldName, string newName)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		if (Atoms[i].ResidueName==oldName) Atoms[i].ResidueName=newName;
	}
}

Real calcAveOccupancy(ProteinStruct &Protein)
{
	int natom=Protein.Atoms.size();
	Real sum=0;

	for (int i=0;i<natom;i++)
	{
		sum+=Protein.Atoms[i].Occupancy;
	}
	return sum/Real(natom);
}

void setSegmentOccupancies(vector<AtomStruct> &Atoms, int start, int end, Real occupancy)
{
        for (int i=start;i<=end;i++)
        {
                if (occupancy==1.0) Atoms[i].Occupancy=occupancy;
		else Atoms[i].Occupancy=0;
        }
}

void setSegmentOccupancies(ProteinStruct &Protein, Vector &degOfFreedom, int nprot)
{
        int ndof=degOfFreedom.size();
        int nsegment=ndof-6*nprot;
        int natom=Protein.Atoms.size();
        int start, end;

        for (int i=0;i<nsegment;i++)
        {
                start=int(Real(natom)*Real(i)/Real(nsegment));
                end=int(Real(natom)*Real(i+1)/Real(nsegment))-1;
                setSegmentOccupancies(Protein.Atoms, start, end, degOfFreedom[nprot*6+i]);
        }
}

void dofToProteins(ProteinStruct &referenceProtein, vector<Real> &degOfFreedom, DofStruct &dofType, vector<ProteinStruct> &Proteins)
{
	int ndof=degOfFreedom.size();
	int segment=0;
	int nsegment=dofType.OccupancySegments;
	cout <<"Before SafeAlloc"<<endl;
	SafeAlloc(Proteins, referenceProtein, dofType.nprot, "Proteins");
	for (int i=0;i<dofType.nprot;i++)
	{
		cout <<"i= "<<i<<endl;
	        rotateAtoms(Proteins[i].Atoms, 0, 0, degOfFreedom[6*i+Z]);
                rotateAtoms(Proteins[i].Atoms, 0, degOfFreedom[6*i+Y], 0);
                rotateAtoms(Proteins[i].Atoms, degOfFreedom[6*i+X], 0, 0);
                moveAtoms(Proteins[i].Atoms, degOfFreedom[6*i+3], degOfFreedom[6*i+4], degOfFreedom[6*i+5]);
	}
	for (int i=0;i<ndof;i++)
	{
		cout <<"i= "<<i<<endl;
		cout <<"dofType.dofType.size()= "<<dofType.dofType.size()<<endl;
		if (dofType.dofType[i]==OCCUPANCY)
		{
			for (int j=0;j<dofType.nprot;j++)
			{
				cout <<"j= "<<j<<endl;
				setSegmentOccupancies(Proteins[j].Atoms, degOfFreedom[i], segment, nsegment);
				segment++;
			}
		}
	}
	cout <<"Leaving dofToProteins"<<endl;
}

void dofToProteins(ProteinStruct &referenceProtein, Matrix &degOfFreedoms, DofStruct &dofType, vector< vector<ProteinStruct> > &Proteins)
{
	int Size=degOfFreedoms.size();
	vector<ProteinStruct> tempProteins;
	cout <<"Size= "<<Size<<endl;
	for (int i=0;i<Size;i++)
	{
		dofToProteins(referenceProtein, degOfFreedoms[i], dofType, tempProteins);
		SafePushBack(Proteins, tempProteins, "Proteins");
	}
}

void lineToDofStruct(string line, DofStruct &dofType)
{
	vector<string> str;
	int nstr;

	dofType.OccupancySegments=0;

	Tokenize2(line, "\t", str);
	nstr=str.size();
	cout <<"nstr= "<<nstr<<endl;
	for (int i=0;i<nstr;i++)
	{
		cout <<"str= "<<str[i]<<endl;
		if (str[i]=="RX") SafePushBack(dofType.dofType, RX, "dofType");
		if (str[i]=="RY") SafePushBack(dofType.dofType, RY, "dofType");
		if (str[i]=="RZ") SafePushBack(dofType.dofType, RZ, "dofType");
		if (str[i]=="DX") SafePushBack(dofType.dofType, DX, "dofType");
		if (str[i]=="DY") SafePushBack(dofType.dofType, DY, "dofType");
		if (str[i]=="DZ") SafePushBack(dofType.dofType, DZ, "dofType");
		if (str[i]=="O") 
		{
			SafePushBack(dofType.dofType, OCCUPANCY, "dofType");
			dofType.OccupancySegments++;
		}
	}
}

void readDofFile(string dofFile, Matrix &degreeOfFreedoms, DofStruct &dofType)
{
	vector<string> lines, str;
	int nstr, nline;
	Vector temp;

	readLines(dofFile, lines, "dofFile");

	nline=lines.size();
	dofType.nprot=getValueOfReal("NumCopies=", lines[0]);
	cout <<"dofType.nprot= "<<dofType.nprot<<endl;
	lineToDofStruct(lines[0], dofType);
	cout <<"nline= "<<nline<<endl;
	for (int i=1;i<nline;i++)
	{
		Tokenize2(lines[i], "\t", str);
		nstr=str.size();
		temp.clear();
		for (int j=0;j<nstr-2;j++)
		{
			SafePushBack(temp, toReal(str[j]), "temp");
		}
		SafePushBack(degreeOfFreedoms, temp, "degreeOfFreedoms");
	}
	cout <<"degreeOfFreedoms.size()= "<<degreeOfFreedoms.size()<<endl;
}

void dofToProteins(string pdbReferenceFile, string dofFile, vector< vector<ProteinStruct> > &Proteins)
{
	ProteinStruct referenceProtein;
	Matrix degOfFreedoms;
	DofStruct dofType;

	readPdb(pdbReferenceFile, referenceProtein);
	cout <<"nsym= "<<referenceProtein.symmetryOperations.size()<<endl;
	readDofFile(dofFile, degOfFreedoms, dofType);
	dofToProteins(referenceProtein, degOfFreedoms, dofType, Proteins);
}

void setOccupancies(vector<AtomStruct> &Atoms, Real occupancy)
{
	int natom=Atoms.size();

	for (int i=0;i<natom;i++)
	{
		Atoms[i].Occupancy=occupancy;
	}
}

void setVdwByAtomType(vector<AtomStruct> &Atoms)
{
        int natom=Atoms.size();

        for (int i=0;i<natom;i++)
        {
                if (Atoms[i].atomid==0) Atoms[i].vdw=0.5;
                else Atoms[i].vdw=1.5;
        }
}

