/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _AtomUtils_include_
#define _AtomUtils_include_
#include "ReadGro.h"

const int RX=0;
const int RY=1;
const int RZ=2;
const int DX=3;
const int DY=4;
const int DZ=5;
const int OCCUPANCY=6;

void copyAtom(const AtomStruct &Atom, AtomStruct &Atom2);
Real calcAngle(VectorStruct &v1, VectorStruct &v2);
Real calcAngle(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3);
Real calcAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3);
void scaleBFactors(ProteinStruct &Protein, Real scale);
void setBFactors(ProteinStruct &Protein, Real BFactor);
void centerAtom(vector<AtomStruct> &Atoms, int index);
void applyPeriodicBoundaryConditionsRectalinear(ProteinStruct &Protein);
void applyPeriodicBoundaryConditionsToFractionalCoordinates(vector<AtomStruct> &Atoms);
void applyPeriodicBoundaryConditions(ProteinStruct &Protein);
void applyPeriodicBoundaryConditions(vector<ProteinStruct> &Proteins);
void initializeAtom(AtomStruct &Atom);
void setAtomTypeNames(vector<string> &AtomTypeName);
void moveAtoms(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, Real dx, Real dy, Real dz);
void moveAtoms(vector<AtomStruct> &Atoms, Real dx, Real dy, Real dz, bool boolPrint=false);
void moveAtom(AtomStruct &Atom, Real dx, Real dy, Real dz);
void moveAtom(AtomStruct &Atom, VectorStruct &v);
void moveAtoms(vector<AtomStruct> &Atoms, Vector &d);
void mergeProteins(const vector<ProteinStruct> &Proteins, vector<AtomStruct> &Atoms);
void calcDistMatrix(vector< vector<Real> > &DistMatrix, vector<AtomStruct> &Atoms);
Real atomSquareDistance(AtomStruct &Atom1, AtomStruct &Atom2);
Real atomDistance(AtomStruct &Atom1, AtomStruct &Atom2);
Real atomDistance(vector<AtomStruct> &Atoms, int atom1, int atom2);
Real atomDistance(Real x, Real y, Real z, AtomStruct &Atom);
void calcDistanceMatrix(vector<AtomStruct> &Atoms, Matrix &DistanceMatrix);
void updateDistanceMatrix(vector<AtomStruct> &Atoms, Matrix &DistanceMatrix, int start1, int end1, int start2, int end2);
void updateDistanceMatrix(ProteinStruct &Protein, vector<int> &start, vector<int> &end);
void updateAtomDistances(vector<AtomStruct> &Atoms, Matrix &DistMatrix, int atomNum);
void updateLocalDistanceMatrix(ProteinStruct &Protein, int residue1, int residue2);
void updateDistanceMatrixForPivot(ProteinStruct &Protein, int residue, int angleType);
int findFirstAtomWithResidueNum(vector<AtomStruct> &Atoms, int residue);
int findLastAtomWithResidueNum(vector<AtomStruct> &Atoms, int residue);
void updateDistanceMatrix(ProteinStruct &Protein, int residue1, int residue2);
void rotateAroundX(AtomStruct &Atom, Real theta);
void rotateAroundY(AtomStruct &Atom, Real theta);
void rotateAroundZ(AtomStruct &Atom, Real theta);
void rotateAroundX(vector<AtomStruct> &Atoms, Real theta);
void rotateAroundY(vector<AtomStruct> &Atoms, Real theta);
void rotateAroundZ(vector<AtomStruct> &Atoms, Real theta);
void calcRotationMatrix(VectorStruct &v, Real angle, Real RotationMatrix[3][3]);
void calcRotationMatrix(VectorStruct &v, Real angle, Matrix &RotationMatrix);
void calcRotationMatrixDerivative(VectorStruct &v, Matrix &RotationMatrix);
void applyRotationMatrix(Real &x, Real &y, Real &z, Real RotationMatrix[3][3]);
void applyRotationMatrix(Real &x, Real &y, Real &z, Matrix &RotationMatrix);
void applyRotationMatrix(AtomStruct &Atom, Real RotationMatrix[3][3]);
void applyRotationMatrix(AtomStruct &Atom, Matrix &RotationMatrix);
void applyRotationMatrixToVector(VectorStruct &v, Real RotationMatrix[3][3]);
void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Real RotationMatrix[3][3]);
void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Matrix &RotationMatrix);
VectorStruct atomVector(AtomStruct &Atom1, AtomStruct &Atom2);
void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, int Atom, Real RotationMatrix[3][3]);
void applyRotationMatrixToAtoms(vector<AtomStruct> &Atoms, Real x, Real y, Real z, Real RotationMatrix[3][3]);
void applyRotationMatrixToAtom(vector<AtomStruct> &Atoms, int atom1, int atom2, Real RotationMatrix[3][3]);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, Real x, Real y, Real z, Real angle);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, int Atom, Real angle);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, VectorStruct &v, Real angle);
VectorStruct rotateAroundVector(VectorStruct v, Real angle, VectorStruct &rotationV);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, vector<bool> &use, Real phi, Real psi, Real theta);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, Real phi, Real psi, Real theta);
Real calcAngle(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3);
VectorStruct calcCrossProduct(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3);
Real calcSign(VectorStruct &v1, VectorStruct &plane1, VectorStruct &plane2);
Real calcDihedralAngle(VectorStruct &v1, VectorStruct &v2, VectorStruct &v3);
Real calcDihedralAngle(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3, AtomStruct &Atom4);
Real calcDihedralAngle(int Atom1, int Atom2, int Atom3, int Atom4, vector<AtomStruct> &Atoms);
Real calcPhi(vector<AtomStruct> &Atoms, int ResidueNum);
Real calcPsi(vector<AtomStruct> &Atoms, int ResidueNum);
Real calcOmega(vector<AtomStruct> &Atoms, int ResidueNum);
Real calcDihedralAngle(vector<AtomStruct> &Atoms, int AngleType, int ResidueNum);
void calcDihedrals(vector<AtomStruct> &Atoms, vector<Real> &dihedral);
void rotateAtomsAroundVector(vector<AtomStruct> &Atoms, int Atom1, int Atom2, Real angle);
void changeDihedralAngle(vector<AtomStruct> &Atoms, int atom2, int atom3, vector<bool> &moved, VectorStruct &v, Real dAngle);
void changeDihedralAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, int atom4, Real dAngle);
void setDihedralAngle(vector<AtomStruct> &Atoms, int Atom1, int Atom2, int Atom3, int Atom4, Real DihedralAngle);
void rotateDihedralAngleBy(vector<AtomStruct> &Atoms, int atom1, int atom2, Real deltaAngle);
void getPhiDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum);
void getPsiDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum);
void getOmegaDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int ResidueNum);
void getDihedralAtoms(vector<AtomStruct> &Atoms, int &Atom1, int &Atom2, int &Atom3, int &Atom4, int AngleType, int ResidueNum);
void setDihedralAngle(vector<AtomStruct> &Atoms, int AngleType, int ResidueNum, Real DihedralAngle);
void dihedralIndexToResidueAndAngleType(int dihedralIndex, int &residue, int &angleType);
void printDihedral(vector<Real> &dihedral);
void printDihedral(vector<AtomStruct> &Atoms);
void setDihedralAngle(vector<AtomStruct> &Atoms, int dihedralIndex, Real DihedralAngle);
void rotateDihedralAngleBy(vector<AtomStruct> &Atoms, int dihedralIndex, Real deltaAngle);
Real calcDihedralAngle(vector<AtomStruct> &Atoms, int dihedralIndex);
void setDihedralAngles(vector<AtomStruct> &Atoms, vector<Real> &dihedrals);
int phiPsiToBasin(Real phi, Real psi);
string basinTypeToString(int basinType);
string phiPsiToBasinString(vector<Real> &phi, vector<Real> &psi);
void rotateAtoms(vector<AtomStruct> &Atoms, vector<bool> &use, Real theta, Real phi, Real psi);
void rotateAtoms(vector<AtomStruct> &Atoms, Real theta, Real phi, Real psi);
void rotateAtomsNoCenter(vector<AtomStruct> &Atoms, Real theta, Real phi, Real psi);
void rotateAtom(AtomStruct &Atom, Real theta, Real phi, Real psi);
void RotateAtom(AtomStruct &Atom, Real theta, Real phi, Real psi);
void doubleCrank(vector<AtomStruct> &Atoms, int Residue, Real dAngle);
void doubleCrank(vector<AtomStruct> &Atoms, int Residue, int AngleType, Real dAngle);
void pivot(vector<AtomStruct> &Atoms, int Residue, int AngleType, Real dAngle);
VectorStruct findNormVector(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3);
void getAngles(ProteinStruct &Protein);
void changeBondAngle(vector<AtomStruct> &Atoms, int atom2, int atom3, vector<bool> &moved, VectorStruct &v, Real dAngle);
void changeBondAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, Real dAngle);
void changeBondAngle(ProteinStruct &Protein, int bondIndex, Real dAngle);
void setBondAngle(vector<AtomStruct> &Atoms, int atom1, int atom2, int atom3, Real angle);
int getAtomIndex(vector<AtomStruct> &Atoms, int residue, string AtomName);
int getNumResidues(vector<AtomStruct> &Atoms);
void getResidues(vector<AtomStruct> &Atoms, vector<int> &ResidueTypes);
void getWaterMolecule(vector<AtomStruct> &Atoms, int index, vector<AtomStruct> &WaterMolecule);
bool isWater(AtomStruct &Atom);
bool isWaterOxygen(AtomStruct &Atom);
bool isIon(AtomStruct &Atom);
bool isProtein(AtomStruct &Atom);
bool isBackbone(AtomStruct &Atom);
bool isDuplicate(AtomStruct &Atom, vector<AtomStruct> &Atoms, int index);
bool isSameResidue(AtomStruct &Atom1, AtomStruct &Atom2);
bool isNextResidue(AtomStruct &Atom1, AtomStruct &Atom2);
bool isPreviousResidue(AtomStruct &Atom1, AtomStruct &Atom2);
void removeNonHetatm(vector<AtomStruct> &Atoms);
void removeNonCA(vector<AtomStruct> &Atoms);
void removeNonCaAndOp(vector<AtomStruct> &Atoms);
void removeDuplicateAtoms(vector<AtomStruct> &Atoms);
void removeProteinAtoms(vector<AtomStruct> &Atoms);
void removeNonProteinAtoms(vector<AtomStruct> &Atoms);
void removeWaters(vector<AtomStruct> &Atoms);
void removeUnknownResidues(vector<AtomStruct> &Atoms);
vector<AtomStruct> removeHydrogensVector(vector<AtomStruct> &Atoms1);
void removeSideChains(vector<AtomStruct> &Atoms);
void removeHydrogens(vector<AtomStruct> &Atoms);
void removeNonBackbone(vector<AtomStruct> &Atoms);
void removeResiduesOtherThan(vector<AtomStruct> &Atoms, string residue);
void removeResiduesWithName(vector<AtomStruct> &Atoms, string residue);
void splitIntoChains(ProteinStruct &Protein, vector<ProteinStruct> &Proteins);
void getChain(vector<AtomStruct> &Atoms, string chain);
int findNumProteinAtoms(vector<AtomStruct> &Atoms);
int countIons(vector<AtomStruct> &Atoms);
void setSolute(vector<AtomStruct> &Atoms);
void setAllToSolute(vector<AtomStruct> &Atoms);
Real calcDipoleMoment(vector<AtomStruct> &Atoms);
bool calcCenterCa(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave);
bool calcCenter(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave);
bool calcCenter(vector<AtomStruct> &Atoms, vector<bool> &use, Real &xave, Real &yave, Real &zave);
bool calcCenterSelection(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave, string selection);
void calcCenterAll(vector<AtomStruct> &Atoms, Real &xave, Real &yave, Real &zave);
VectorStruct findLongestVector(vector<AtomStruct> &Atoms, int &atom1, int &atom2);
PosStruct findLongestOrthogonalAxes(vector<AtomStruct> &Atoms, int atom1, int atom2);
Real calcUnitCellVolume(ProteinStruct &Protein);
void center(vector<AtomStruct> &Atoms);
void centerAll(vector<AtomStruct> &Atoms);
void setAtomElectrons(vector<AtomStruct> &Atoms);
void center(vector<AtomStruct> &Atoms, vector<bool> &use);
void selectAtomsByName(vector<AtomStruct> &Atoms, vector<AtomStruct> &SelectedAtoms, string SelectionName);
void selectAtomsByName(vector<AtomStruct> &Atoms, string SelectionName);
void selectAtomsByName(vector<AtomStruct> &Atoms, string selectionName, vector<bool> &use);
void removeSelection(vector<AtomStruct> &Atoms, string selection);
void removeResidue(vector<AtomStruct> &Atoms, string selection);
void changeBondLength(vector<AtomStruct> &Atoms, int atom, VectorStruct &v, vector<bool> &moved);
void changeBondLength(vector<AtomStruct> &Atoms, int atom1, int atom2, Real dLength);
void setBondLength(vector<AtomStruct> &Atoms, int atom1, int atom2, Real d);
void centerAtomsInBox(vector<AtomStruct> &Atoms, string SelectionName);
void centerAtomsInBox(vector<AtomStruct> &Atoms, Real &XMove, Real &YMove, Real &ZMove);
void centerAtomsInBox(vector<AtomStruct> &Atoms);
void allignCenters(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2);
Real calcRg(vector<AtomStruct> &Atoms);
void setNumElectrons(Real NumElectrons[]);
void setAtomElectrons(vector<AtomStruct> &Atoms, Real NumElectrons[]);
Real calcProteinElectrons(vector<AtomStruct> &Atoms);
void applyPeriodicBoundaryConditions(vector<AtomStruct> &Atoms, Real xmin, Real ymin, Real zmin, Real xmax, Real ymax, Real zmax);
void applyPeriodicBoundaryConditions(vector<AtomStruct> &Atoms, Real XBoxLength, Real YBoxLength, Real ZBoxLength);
void GetAtomsWithinRadius(vector<AtomStruct> &Atoms, Real xgrid, Real ygrid, Real zgrid, Real radius);
void Image(vector<AtomStruct> &Atoms, Real &XBoxLength, Real &YBoxLength, Real &ZBoxLength, int NumPerEdge);
void Image(vector<AtomStruct> &Atoms, int axis, Real &Length, int NumPerEdge);
void calcBondLengths(vector<AtomStruct> &Atoms, Vector &lengths);
void charmmAlias(AtomStruct &Atom);
void atomAlias(vector<AtomStruct> &Atoms);
void setCharmmAtomNames(vector<AtomStruct> &CharmmAtoms, vector<AtomStruct> &Atoms);
void charmmAtomNameToVDW(vector<AtomStruct> &CharmmAtoms, AtomStruct &Atom);
void charmmAtomNamesToVDW(vector<AtomStruct> &CharmmAtoms, vector<AtomStruct> &Atoms);
void ScaleVDW(Real VDWScale, Real VDWOffSet, vector<AtomStruct> &Atoms);
void AddChargeRadii(vector<AtomStruct> &Atoms, Real ChargeRadiiScale);
void setVDW(string PrmFile, string RtfFile, vector<AtomStruct> &Atoms);
void setVDW(vector<AtomStruct> &Atoms, Real vdw);
void getCharges(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms);
void getCharges(string PsfFile, vector<AtomStruct> &Atoms);
void getCharmmAtomNames(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms);
void getCharmmAtomNames(string PsfFile, vector<AtomStruct> &Atoms);
void getConnectivity(vector<AtomStruct> &PsfAtoms, vector<AtomStruct> &Atoms);
void getConnectivity(string PsfFile, vector<AtomStruct> &Atoms);
void GetPsfInfo(string PsfFile, vector<AtomStruct> &Atoms);
void setDefaultConnectivity(vector<AtomStruct> &Atoms);
void getFirstLastResidue(vector<AtomStruct> &Atoms, int &FirstResidue, int &LastResidue);
void getLastResidue(vector<AtomStruct> &Atoms, int &LastResidue);
void getFirstResidue(vector<AtomStruct> &Atoms, int &FirstResidue);
bool hasNA(vector<AtomStruct> &Atoms);
bool isCompleteResidue(vector<AtomStruct> &Atoms, int ResidueNum);
bool hasChainBreaks(vector<AtomStruct> &Atoms);
bool isValidProtein(vector<AtomStruct> &Atoms);
bool isValidProtein(string pdbFile);
void renumberResidues(vector<AtomStruct> &Atoms);
void makeResiduesConsecutive(vector<AtomStruct> &Atoms);
void setWeightToBFactor(vector<AtomStruct> &Atoms);
string getFasta(ProteinStruct &Protein);
string getSubsequence(ProteinStruct &Protein, int residue1, int residue2);
void getSegment(vector<AtomStruct> &Atoms1, int start, int end, vector<AtomStruct> &Atoms2);
VectorStruct icToXYZ(VectorStruct &v1, VectorStruct &v2, Real angle, Real length, Real dihedralAngle);
VectorStruct icToXYZ(VectorStruct &pos1, VectorStruct &pos2, VectorStruct &pos3, Real angle, Real length, Real dihedralAngle);
void icToXYZ(AtomStruct &Atom1, AtomStruct &Atom2, AtomStruct &Atom3, AtomStruct &Atom, Real angle, Real length, Real dihedralAngle);
void rotateToAlignWithYAxis(vector<AtomStruct> &Atoms, VectorStruct &v);
void convertToFractionalCoordinates(Real dx, Real dy, Real dz, VectorStruct &a, VectorStruct &b, VectorStruct &c, Real &fracX, Real &fracY, Real &fracZ);
void convertToFractionalCoordinates(Real dx, Real dy, Real dz, Matrix &cartToFrac, Real &fracX, Real &fracY, Real &fracZ);
void convertToFractionalCoordinates(Real dx, Real dy, Real dz, ProteinStruct &Protein, Real &fracX, Real &fracY, Real &fracZ);
void convertToFractionalCoordinates(ProteinStruct &Protein);
void convertToCartesianCoordinates(Real fracX, Real fracY, Real fracZ, Matrix &fracToCart, Real &dx, Real &dy, Real &dz);
void convertToCartesianCoordinates(ProteinStruct &Protein);
void calcUnitCellVectors(ProteinStruct &Protein);
void calcUnitCellVectors(Real XBoxLength, Real YBoxLength, Real ZBoxLength, Real alpha, Real beta, Real gamma, VectorStruct &a, VectorStruct &b, VectorStruct &c);
void findCoreAtoms(vector<AtomStruct> &Atoms);
void findCoreAtoms(vector<AtomStruct> &Atoms, Real defaultVDW, Real ASACutoff);
void findCoreAtomsSharke(vector<AtomStruct> &Atoms, Real defaultVDW, Real ASACutoff);
void calcCartToFrac(ProteinStruct &Protein);
void calcFracToCart(VectorStruct &a, VectorStruct &b, VectorStruct &c, Matrix &fracToCart);
void calcMomentOfInertia(vector<AtomStruct> &Atoms, Matrix &momentOfInertia);
void alignWithYAxis(vector<AtomStruct> &Atoms);
void parsePdb(string PdbFile, ProteinStruct &Protein, bool BoolremoveWaters, bool BoolremoveUnknownResidues);
void applySymmetryOperation(Vector &vIn, SymmetryOperationStruct &symmetryOperation, Vector &vOut);
AtomStruct applySymmetryOperation(AtomStruct Atom, SymmetryOperationStruct &symmetryOperation);
void applySymmetryOperation(AtomStruct Atom1, SymmetryOperationStruct &symmetryOperation, AtomStruct &Atom2);
void applySymmetryOperation(vector<AtomStruct> &Atoms1, SymmetryOperationStruct &symmetryOperation, vector<AtomStruct> &Atoms2);
void applySymmetryOperation(vector<AtomStruct> &Atoms, SymmetryOperationStruct &symmetryOperation);
void applySymmetryOperationFast(vector<AtomStruct> &Atoms, SymmetryOperationStruct &symmetryOperation);
vector<AtomStruct> makeUnitCell(ProteinStruct &Protein);
void makeUnitCell(ProteinStruct &Protein, ProteinStruct &UnitCell);
Real calcFurthestDistance(vector<AtomStruct> &Atoms);
void makeUnitCellAndImagesLib(vector<ProteinStruct> &Proteins, vector<ProteinStruct> &allProteins);
void moveIntoUnitCell(ProteinStruct &Protein);
void readFasta(string fastaFile, vector<string> &residues);
bool isBonded(AtomStruct &Atom, int i);
void getEquivalentAtoms(ProteinStruct &Protein1, ProteinStruct &Protein2, vector< vector<int> > &pairs);
void getMatchingAtoms(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, vector<AtomStruct> &AtomsOut);
void getMatchingAtomsIgnoreChain(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, vector<AtomStruct> &AtomsOut);
void dofToProteins(string pdbReferenceFile, string dofFile, vector< vector<ProteinStruct> > &Proteins);
void setOccupancies(vector<AtomStruct> &Atoms, Real occupancy);
void setSegmentOccupancies(ProteinStruct &Protein, Vector &degOfFreedom, int nprot);
Real calcAveOccupancy(ProteinStruct &Protein);
void setVdwByAtomType(vector<AtomStruct> &Atoms);
void renameResidues(vector<AtomStruct> &Atoms, string oldName, string newName);
#endif
