#ifndef _MathUtils_include_
#define _MathUtils_include_

# include "TypeDef.h"
# include "VectorManip.h"
# include "Structures.h"
# include "Constants.h"
# include "IOUtils.h"
# include "MathStructures.h"
void printMatrix(Matrix &Mat);
void printMatrix(Real **Mat, int Size1, int Size2);
string vectorToStr(VectorStruct &v);
template <class Matrix_T>
void createMatrix(Matrix_T &Matrix, int Size1, int Size2);
template <class Matrix_T>
void deleteMatrix(Matrix_T &Matrix);
bool isEven(int x);
bool isOdd(int x);
bool isEqual(Real x, Real y, Real d);
bool isEqual(Real x, Real y);
bool vectorEqual(vector<Real> &v1, vector<Real> &v2, Real d);
bool vectorEqual(vector<int> &v1, vector<int> &v2);
bool matrixEqual(Matrix &mat1, Matrix &mat2);
bool isPositiveDefinite(Matrix &mat);
bool quadraticEquation(Real a, Real b, Real c, Real &low, Real &high);
void CubicEquation(Real a, Real b, Real c, Real d, Real &x1, Real &x2, Real &x3);
Real solveTrig(Real a, Real b, Real c);
void solveTrig(Real a, Real b, Real c, Real &theta1, Real &theta2);
Real power(Real x, Real y);
Real vectorLength(VectorStruct &v);
Real calcLength(Vector &v);
void vectorNormalize(VectorStruct &v);
void vectorNormalize(VectorStruct &v1, VectorStruct &v2);
VectorStruct setLength(VectorStruct &v, Real length);
void setLength(Vector &v, Real length);
VectorStruct calcCrossProduct(VectorStruct &v1, VectorStruct &v2);
Real calcDotProduct(Vector &v1, Vector &v2);
Real calcDotProduct(VectorStruct &v1, Vector &v2);
Real calcAngle(VectorStruct &v1, VectorStruct &v2);
Real calcAngle(Vector &v1, Vector &v2);
Real calcDistance(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2);
Real calcDistance(Vector &v1, Vector &v2);
Real calcProjection(VectorStruct &v1, VectorStruct &v2);
Real calcDeltaAngle(Real angle1, Real angle2);
void calcEulerAngles(Real x, Real y, Real z, Real &theta, Real &phi);
void calcPolarCoordinates(VectorStruct v, Real &ry, Real &rz);
void polar2Cartesian(Real &x, Real &y, Real &z, Real &ry, Real &rz);
void polar2Cartesian(VectorStruct &v, Real &ry, Real &rz);
template<class pos>
void rotateAroundX(pos &coord, Real theta);
template<class pos>
void rotateAroundY(pos &coord, Real theta);
template<class pos>
void rotateAroundZ(pos &coord, Real theta);
void calcRotationMatrixAroundX(Real theta, Matrix &rotationMatrix);
void calcRotationMatrixAroundY(Real theta, Matrix &rotationMatrix);
void calcRotationMatrixAroundZ(Real theta, Matrix &rotationMatrix);
Real coth(Real x);
Real csch(Real x);
Real calcBesselRecursive(long double r, int l);
Real calcBessel(Real r, int l, vector<long double> sterm);
void calcSTerm(int l, vector<long double> &sterm);
Real calcBessel(Real r, int l);
Real calcBesselI0(Real x);
Real calcLogBesselI0(Real x);
void calcSphericalHarmonic(Real phi, Real theta, int lvalue, int mvalue, Real &realOut, Real &imagOut);
void calcFactorials(int maxFactorial, Vector &factorials);
Real calcWigner_dElement(int j, int m1, int m2, Real theta, Vector &factorials, Vector &cosPowers, Vector &sinPowers);
Real calcWigner_dElement(int j, int m1, int m2, Real theta);
void initializeNegativeExpLookUp(LookUpStruct &lookUp, Real max, Real binSize);
void initializeNegativeExpLookUp(LookUpStruct &lookUp);
Real getNegativeExpLookUp(LookUpStruct &lookUp, Real x);
void initializeExpLookUp(LookUpStruct &lookUp, Real max, Real binSize);
void initializeExpLookUp(LookUpStruct &lookUp);
Real getExpLookUp(LookUpStruct &lookUp, Real x);
void initializeCosLookUp_Old(LookUpStruct &lookUp, Real binSize);
void initializeCosLookUp(LookUpStruct &lookUp, Real binSize);
void initializeCosLookUp(LookUpStruct &lookUp);
void initializeSinLookUp_Old(LookUpStruct &lookUp, Real binSize);
void initializeSinLookUp(LookUpStruct &lookUp, Real binSize);
void initializeSinLookUp(LookUpStruct &lookUp);
Real getSinLookUp(LookUpStruct &lookUp, Real x);
Real getCosLookUp(LookUpStruct &lookUp, Real x);
void getTrigLookUp_Old(LookUpStruct &lookUp, Real x, Real &cosx, Real &sinx);
void getTrigLookUp(LookUpStruct &lookUp, Real &x, Real &cosx, Real &sinx);
Real calcVolume(VectorStruct &a, VectorStruct &b, VectorStruct &c);
void calcHistogram(Vector &v, Vector &histogram, Real binSize);
void calcHistogram(Vector &v, Vector &histogram, Real binSize, Real min, int nbin);
void calcHistogram(Vector &v, Vector &histogram, Real &binSize, Real &min);
void equation(long double **coefficient, long double variable[], int nequations);
void equation2(long double coefficient[][500], long double variable[], int nequations);
template <typename T>
void swapEquations(vector< vector<T> > &coefficient, int i, int j);
template <typename T>
void makeDiagonalNonzero(vector< vector<T> > &coefficient, int n);
void equation(Matrix &coefficient, Vector &variable);
void gaussJordanElimination(long double **coefficient, long double **InverseMatrix, int nequations);
void gaussJordanElimination(Matrix coefficient, Matrix &InverseMatrix);
void calcInverseMatrix(Matrix coefficient, Matrix &InverseMatrix);
Real getMatrixElement(Real **Matrix1, Real **Matrix2, int m, int n, int Size);
void matrixMultiply(Real **Matrix1, Real **Matrix2, Real **MatrixOut, int Size1, int Size2);
Real getMatrixElement(Real Matrix1[3][3], Real Matrix2[3][3], int m, int n, int Size);
void matrixMultiply(Real Matrix1[3][3], Real Matrix2[3][3], Real MatrixOut[3][3]);
Real getMatrixElement(Matrix Matrix1, Matrix Matrix2, int m, int n);
Real getMatrixElement(vector<Real> &v, Matrix &mat, int n);
void matrixMultiply(Matrix Matrix1, Matrix Matrix2, Matrix &MatrixOut);
void matrixMultiplyFast(Vector &v, Matrix &mat, Vector &vOut);
void matrixMultiply(vector<Real> &v, Matrix &mat, vector<Real> &vOut);
void matrixMultiply(Matrix &mat, vector<Real> &v, vector<Real> &vOut);
void matrixMultiply(Real x1, Real y1, Real z1, Matrix &mat, Real &x2, Real &y2, Real &z2);
void subtractMatrix(Matrix &mat1, Matrix &mat2, Matrix &diffMat);
Real calcTrace(Matrix &mat);
void transpose(Matrix Mat, Matrix &Matrixtranspose);
template<class Matrix_T>
void transpose(Matrix_T &Matrix);
void swap(Real &x, Real &y);
template <class Matrix_T>
void transpose(Matrix_T &Matrix, int Size1, int Size2);
template <class Matrix_T>
void transpose2(Matrix_T &Matrix, int Size1, int Size2);
void normalizeMatrix(Matrix &mat);
void calcEigenvector(Matrix &mat, Vector &eigenvector);
Real calcSum(vector<Real> &x);
Real calcSumSqr(vector<Real> &x);
template<class T>
T calcAverage(vector<T> &x);
template<class T>
T calcAverage(vector<T> &x, int first, int last);
Real calcAverage(vector<Real> &x);
Real calcAverage(vector<Real> &x, int first, int last);
Real calcVariance(vector<Real> &x);
Real calcStandardDeviation(vector<Real> &x);
Real calcError(Vector &x);
Real variance(vector< vector<Real> > &x);
template <class T>
Real calcCovariance(vector<T> &x, vector<T> &y);
template<class Matrix_T>
void calcCovarianceMatrix(Matrix_T Matrix, Matrix_T &CovarianceMatrix);
void calcZScore(vector<Real> &x, vector<Real> &z);
Real interpolate(Real x1, Real y1, Real x2, Real y2, Real x);
Real interpolate3D(vector< vector< vector<PosStruct> > > &x, Array3D &y, vector<Real> &pos);
Real calcAreaUnderCurve(vector<Real> &x, vector<Real> &y);
void calcBinAverage(Real binSize, vector<Real> &x, vector<Real> &y, vector<Real> &ave);
bool isNonzero(Vector &v);
void normalize(vector<Real> &v);
void setAverageTo(vector<Real> &v, Real x);
Real getMin(Real x1, Real x2);
Real getMax(Real x1, Real x2);
int getMin(int x1, int x2);
int getMax(int x1, int x2);
Real calcMonotonicity(Vector &x, Vector &y);
template <class Vector_T>
bool Issorted(Vector_T &v);
template <class Vector_T>
void sort(Vector_T &v);
bool isSorted(Vector &v);
void sort(Vector &v);
void sort(vector<int> &v);
Real calcGaussian(Real x, Real stdDev);
Real calcGaussian(Real x, Real mean, Real stdDev);
void initializeRandom(int rannum);
int randInt(int Max);
int randInt(int Min, int Max);
Real randDouble(Real Max);
Real randDouble(Real Min, Real Max);
Real randDoubleRef(Real &Min, Real &Max);
Real randGaussian(Real stdDev);
Real randGaussian(Real mean, Real stdDev);
Real randAngle(Real mean, Real width);
VectorStruct randVector(Real length);
VectorStruct randVector();
void createRandomVector(Vector &r, int Size);
template <typename T>
void linearRegression(vector<T> x, vector<T> y, T &slope, T &yintercept);
void linearRegression(Vector &x, Vector &y, Real &slope, Real &yintercept);
Real calcAbsSum(vector<Real> &v);
void abs(vector<Real> &v);
Real calcAverage(vector<Real> &x, vector<Real> &y);
void calcStdDevAndAverage(vector<Real> &x, vector<Real> &y, Real &ave, Real &stdDev);
Real calcScaleFactor(vector<Real> &y1, vector<Real> &y2);
void calcGaussian(vector<Real> &x, vector<Real> &gaussian, Real mean, Real width);
Real calcScaleForGaussian(vector<Real> &x, vector<Real> &y, Real mean, Real width);
Real calcSqrDiff(vector<Real> &y1, vector<Real> &y2);
void vectorMultiply(Real scale, vector<Real> &v);
void multiply(Real scale, vector<Real> &v1, vector<Real> &v2);
Real gaussianFitScore(vector<Real> &x, vector<Real> &y, Real mean, Real width);
Real gaussianFitScore(vector<Real> coefficient, xyStruct xy);
void fitGaussian(vector<Real> &x, vector<Real> &y, Real &scale, Real &mean, Real &width);
void fitGaussian(vector<Real> &x, vector<Real> &y, Real &mean, Real &width);
Real calcPower(Real x, int degree);
void calcPowers(Real x, int degree, Vector &powers);
Real calcOverallCorrelation(Matrix &data, Vector &z, Vector &variable);
Real fitCurves(Matrix &curves, Vector &y, Vector &variable);
void fitQuadratic(Real x1, Real y1, Real x2, Real y2, Real x3, Real y3, Real &a, Real &b, Real &c);
void fitGaussian2(vector<Real> &x, vector<Real> &y, Real &scale, Real &mean, Real &width);
void fitGaussian2(vector<Real> &x, vector<Real> &y, Real &mean, Real &width);
void FFT(Real InputReal[], Real InputImag[], Real OutputReal[], Real OutputImag[], int max);
void printArray(Real **output, int MAX);
void printTensor(Real ***output, int MAX);
void FFT2D(Real **InputReal, Real **InputImag, Real **OutputReal, Real **OutputImag, int max);
void FFT3D(Real ***InputReal, Real ***InputImag, Real ***OutputReal, Real ***OutputImag, int max);
void calcFTElement(Real ***InputReal, Real ***InputImag, Real &OutputReal, Real &OutputImag, int i, int j, int k, int max);
void FT3D(Real ***InputReal, Real ***InputImag, Real ***OutputReal, Real ***OutputImag, int max);
void fitLine(vector<Real> &x, vector<Real> &y, Real &a, Real &b);
//template <typename T>
void multiVariableRegression(Vector &z, Matrix &data, Vector &variable);
Real calcSxy(vector<Real> &x, vector<Real> &y, Real a, Real b);
Real calcSy(vector<Real> &x, vector<Real> &y);
Real correlation(vector<Real> &x, vector<Real> &y);
Real correlation(vector<Real> &x, vector<Real> &y);
Real correlation(vector<Real> &x, vector<Real> &y, Real a, Real b);
//template <typename T>
Real calcSxy(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable);
//template <typename T>
Real calcSy(vector<Real> &y);
Real calcCorrelationFromFit(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable);
Real linearRegression(vector<Real> &x, vector<Real> &y, Real &slope, Real &yintercept, Real &slopeError, Real &yError);
//template <typename T>
Real calcCorrelation(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable);
Real calcPearson(Vector &x, Vector &y);
Real roundReal(Real x, Real nearest);
bool isLastPermutation(vector<int> &permutation);
int findLast(vector<int> &permutation);
int findSecondToLast(vector<int> &permutation);
int countNumAfter(vector<int> &permutation, int last);
void getNextPermutation(vector<int> &permutation);
Real nChooseK(int n, int k);
Real updatePrincipleComponent(Matrix &data, Vector &r);
void calcPrincipleComponent(Matrix &data, Vector &r);
void removeAverage(Matrix &data);
void removeAverage(Vector &v);
Real calcPrincipleComponentWeight(Vector &data, Vector &r);
void subtractPrincipleComponent(Vector &data, Vector &r);
void subtractPrincipleComponent(Matrix &data, Vector &r);
void calcPca(Matrix data, Matrix &pca);
vector<Real> addVectors(vector<Real> &v1, vector<Real> &v2);
bool isZero(Vector &v);
#endif
