/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MinMax_included_
#define _MinMax_included_

# include "Structures.h"
# include "TypeDef.h"
# include "AtomUtils.h"
# include "VectorManip.h"

	template <class T>
T findMax(vector<T> &v)
{
        int Size=v.size();
        T max=v[0];

        for (int i=1;i<Size;i++)
        {
                if (v[i]>max) max=v[i];
        }
        return max;
}

	template <class T>
int findMax(vector<T> &v, T &max)
{
        int Size=v.size();
	int nmax=0;
        max=v[0];
        for (int i=1;i<Size;i++)
        {
                if (v[i]>max) 
		{
			max=v[i];
			nmax=i;
		}
        }
        return nmax;
}

	template <class T>
T findMin(vector<T> &v)
{
        int Size=v.size();
        T min=v[0];

        for (int i=1;i<Size;i++)
        {
                if (v[i]<min) min=v[i];
        }
        return min;
}

	template <class T>
int findMin(vector<T> &v, T &min)
{
        int Size=v.size(), lowest;
        min=v[0];
        lowest=0;
        for (int i=1;i<Size;i++)
        {
                if (v[i]<min) 
                {
                        min=v[i];
                        lowest=i;
                }
        }
        return lowest;
}

template <class T>
void MinMax(vector<T> &v, T &min, T &max)
{
        int Size=v.size();

        min=v[0];
        max=v[0];
        for (int i=0;i<Size;i++)
        {
                if (v[i]<min) min=v[i];
                else if (v[i]>max) max=v[i];
        }
}
/*
template <class T>
void MinMax(T &XMin, T &YMin, Real &ZMin, Real &XMax, Real &YMax, Real &ZMax, vector<AtomStruct> &Atoms)
{
	int n, natom;
        cout <<"In MinMax"<<endl;
	natom=Atoms.size();
	XMin=Atoms[0].x;
	YMin=Atoms[0].y;
	ZMin=Atoms[0].z;
	XMax=Atoms[0].x;
	YMax=Atoms[0].y;
	ZMax=Atoms[0].z;
	for (n=0;n<natom;n++)
	{
                //cout <<"n= "<<n<<endl;
		if (Atoms[n].x<XMin) XMin=Atoms[n].x;
		if (Atoms[n].y<YMin) YMin=Atoms[n].y;
		if (Atoms[n].z<ZMin) ZMin=Atoms[n].z;
		if (Atoms[n].x>XMax) XMax=Atoms[n].x;
		if (Atoms[n].y>YMax) YMax=Atoms[n].y;
		if (Atoms[n].z>ZMax) ZMax=Atoms[n].z;
	}
}
*/

template <class Vector_T>
void MinMax(Real &XMin, Real &YMin, Real &ZMin, Real &XMax, Real &YMax, Real &ZMax, Vector_T &pos)
{
	int n, natom;
	natom=pos.size();
	XMin=pos[0].x;
	YMin=pos[0].y;
	ZMin=pos[0].z;
	XMax=pos[0].x;
	YMax=pos[0].y;
	ZMax=pos[0].z;
	for (n=0;n<natom;n++)
	{
		if (pos[n].x<XMin) XMin=pos[n].x;
		if (pos[n].y<YMin) YMin=pos[n].y;
		if (pos[n].z<ZMin) ZMin=pos[n].z;
		if (pos[n].x>XMax) XMax=pos[n].x;
		if (pos[n].y>YMax) YMax=pos[n].y;
		if (pos[n].z>ZMax) ZMax=pos[n].z;
	}
}

template <class T>
void MinMax(T &XMin, T &YMin, T &ZMin, T &XMax, T &YMax, T &ZMax, lattice &cubes)
{
        int MaxXBin, MaxYBin, MaxZBin;
        Get3DVectorSize(cubes, MaxXBin, MaxYBin, MaxZBin, "cubes in MinMax");
	
        XMin=cubes[0][0][0].x;
	YMin=cubes[0][0][0].y;
	ZMin=cubes[0][0][0].z;
	XMax=cubes[0][0][0].x;
	YMax=cubes[0][0][0].y;
	ZMax=cubes[0][0][0].z;

        for (int i=0;i<MaxXBin;i++)
        {
                for (int j=0;j<MaxYBin;j++)
                {
                        for (int k=0;k<MaxZBin;k++)
                        {
		                if (cubes[i][j][k].x<XMin) XMin=cubes[i][j][k].x;
		                if (cubes[i][j][k].y<YMin) YMin=cubes[i][j][k].y;
		                if (cubes[i][j][k].z<ZMin) ZMin=cubes[i][j][k].z;
		                if (cubes[i][j][k].x>XMax) XMax=cubes[i][j][k].x;
		                if (cubes[i][j][k].y>YMax) YMax=cubes[i][j][k].y;
		                if (cubes[i][j][k].z>ZMax) ZMax=cubes[i][j][k].z;
                        }
                }
        }
}

template <class T>
void MinMaxProtein(T &XMin, T &YMin, T &ZMin, T &XMax, T &YMax, T &ZMax, vector<AtomStruct> Atoms)
{
	removeUnknownResidues(Atoms);
	removeWaters(Atoms);
	MinMax(XMin, YMin, ZMin, XMax, YMax, ZMax, Atoms);
	cout <<"XMin= "<<XMin<<" YMin= "<<YMin<<" ZMin= "<<ZMin<<endl;
	cout <<"XMax= "<<XMax<<" YMax= "<<YMax<<" ZMax= "<<ZMax<<endl;
}

#endif
