/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _ReadGofRFile_included_
#define _ReadGofRFile_included_

# include "IOUtils.h"
# include "VectorManip.h"
# include "TypeDef.h"

const Real UNK_DENSITY=-666.0;

void ReadGofRFile(string GofRFile, Real contrast, Matrix &DensityMatrix, Real RecBin)
{
	bool valid;
	char CharLine[1000], chargofrfile[1000];
	char *Space;
	int m, n;
	int NumSolventTypes, MaxRecord;
	int distance;
	string line, str;
	Real dist;

	cout <<"In ReadGofRFile"<<endl;

	NumSolventTypes=2000;
	MaxRecord=1000;
	strcpy(chargofrfile, GofRFile.c_str());
	valid=exist(chargofrfile);
	if (!valid)
	{
		cout <<"Error. GofRFile "<<GofRFile<<" cannot be opened."<<endl;
		exit(EXIT_FAILURE);
	}

	fstream gofr;
	gofr.open(chargofrfile,ios::in);
	cout <<"chargofrfile= "<<chargofrfile<<endl;
	getline(gofr, line);
	while (true)
	{
		if (line.substr(0,3)=="END") break;
		if (gofr.eof()) break;
		if (line.substr(0,15)=="NumSolventTypes")
		{
			NumSolventTypes=StrToInt(line.substr(16,10));
			cout <<"NumSolventTypes= "<<NumSolventTypes<<endl;
		}

		if (line.substr(0,9)=="MaxRecord")
		{
			MaxRecord=StrToInt(line.substr(10,10));
			cout <<"NumSolventTypes= "<<NumSolventTypes<<endl;
			cout <<"MaxRecord= "<<MaxRecord<<endl;
			CreateMatrix(DensityMatrix, NumSolventTypes, MaxRecord);
			cout <<"After CreateMatrix"<<endl;
			for (m=0;m<NumSolventTypes;m++)
			{
				for (n=0;n<MaxRecord;n++)
				{
					if (n<30) DensityMatrix[m][n]=UNK_DENSITY;
					else DensityMatrix[m][n]=UNK_DENSITY;
					//cout <<"DensityMatrix["<<m<<"]["<<n<<"]= "<<DensityMatrix[m][n]<<endl;
				}
			}
		}
		//cout <<"After Initialized DensityMatrix"<<endl;
		if (line.substr(0,8)=="AtomType")
		{
			n=StrToInt(line.substr(9,15));
			str=line.substr(0,10);
			distance=0;
			//cout <<line<<endl;
			//cout <<line.substr(9,15)<<endl;
			//cout <<"n= "<<n<<endl;
			while (true)
			{
				getline(gofr,line);
				if (line.substr(0,1)==" ") cout <<"Break found"<<endl;
				if (line.substr(0,8)=="AtomType")
				{
					n++;
					break;
				}
				strcpy(CharLine, line.c_str());
				Space=strchr(CharLine,' ');
				if (Space==NULL) break;
				if (line.substr(0,1)==" " || line.substr(0,3)=="END") break;
				line=" " + line + " ";
				dist=StrToFloat(GetWord(line, 1));
				distance=int(dist/RecBin+0.5);
				str=GetWord(line, 2);
				DensityMatrix[n][distance]=StrToFloat(str)-contrast;
				//cout <<line<<endl;
				//cout <<dist<<" "<<str<<endl;
				//cout <<"DensityMatrix["<<n<<"]["<<distance<<"]= "<<DensityMatrix[n][distance]<<endl;
			}
		}
		getline(gofr, line);
	}
	for (n=0;n<NumSolventTypes;n++)
	{
		for (distance=0;distance<MaxRecord;distance++)
		{
			//cout <<"DensityMatrix["<<n<<"]["<<distance<<"]= "<<DensityMatrix[n][distance]<<endl;
		}
	}
	gofr.close();	
}

void ReadGofRFile(string GofRFile, Array3D &DensityMatrix, Real RecBin)
{
	char CharLine[1000];
	char *Space;
	int NumSolventTypes, MaxRecord;
	int type, distance, Size;
	string line;
        vector<string> str;
        vector<Real> v;
	Real dist;
        fstream gofr;

	cout <<"In ReadGofRFile"<<endl;

	NumSolventTypes=2000;
	MaxRecord=1000;
	type=0;

        OpenFile(GofRFile, gofr, "3D gofr file"); 
        
	getline(gofr, line);
	while (true)
	{
		if (line.substr(0,3)=="END") break;
		if (gofr.eof()) break;
		if (line.substr(0,15)=="NumSolventTypes")
		{
			NumSolventTypes=StrToInt(line.substr(16,10));
			cout <<"NumSolventTypes= "<<NumSolventTypes<<endl;
		}

		if (line.substr(0,9)=="MaxRecord")
		{
			MaxRecord=StrToInt(line.substr(10,10));
			cout <<"NumSolventTypes= "<<NumSolventTypes<<endl;
			cout <<"MaxRecord= "<<MaxRecord<<endl;
			Safe3DAlloc(DensityMatrix, NumSolventTypes, MaxRecord, 0, "3D density matrix");
			cout <<"After CreateMatrix"<<endl;
		}
		//cout <<"After Initialized DensityMatrix"<<endl;
		if (line.substr(0,8)=="AtomType")
		{
			type=StrToInt(line.substr(9,15));
			//str=line.substr(0,10);
			distance=0;
			//cout <<line<<endl;
			//cout <<line.substr(9,15)<<endl;
			//cout <<"n= "<<n<<endl;
			while (true)
			{
				getline(gofr,line);
				if (line.substr(0,1)==" ") cout <<"Break found"<<endl;
				if (line.substr(0,8)=="AtomType")
				{
					type++;
					break;
				}
				strcpy(CharLine, line.c_str());
				Space=strchr(CharLine,' ');
				if (Space==NULL) break;
				if (line.substr(0,1)==" " || line.substr(0,3)=="END") break;
				line=" " + line + " ";
                                Tokenize2(line, " ", str);
				dist=StrToFloat(str[0]);
				distance=int(dist/RecBin+0.5);
                                Size=str.size();
                                v.clear();
				for (int n=1;n<Size-2;n++) SafePushBack(v, Real(StrToFloat(str[n])/StrToFloat(str[Size-2])), "v in ReadGofRFile");
                                //printVector(v);
                                //printVector(str);
                                //cout <<line<<endl;
				DensityMatrix[type][distance]=v;
				//cout <<line<<endl;
				//cout <<dist<<" "<<str<<endl;
				//cout <<"DensityMatrix["<<type<<"]["<<distance<<"]= "<<DensityMatrix[type][distance]<<endl;
			}
		}
		getline(gofr, line);
	}
	gofr.close();	
}
#endif
