/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <vector>
# include <algorithm>

using namespace std;

# include "ResidueCode.h"
# include "Structures.h"
# include "error.h"

string OneLetterCode(int ResidueID)
{
        if (ResidueID==GLY) return "G";
        else if (ResidueID==ALA) return "A";
        else if (ResidueID==VAL) return "V";
        else if (ResidueID==LEU) return "L";
        else if (ResidueID==ILE) return "I";
        else if (ResidueID==SER) return "S";
        else if (ResidueID==THR) return "T";
        else if (ResidueID==CYS) return "C";
        else if (ResidueID==MET) return "M";
        else if (ResidueID==ASP) return "D";
        else if (ResidueID==GLU) return "E";
        else if (ResidueID==ASN) return "N";
        else if (ResidueID==GLN) return "Q";
        else if (ResidueID==LYS) return "K";
        else if (ResidueID==ARG) return "R";
        else if (ResidueID==PHE) return "F";
        else if (ResidueID==TYR) return "Y";
        else if (ResidueID==PRO) return "P";
        else if (ResidueID==HIS) return "H";
        else if (ResidueID==TRP) return "W";
        else
        {
                cout <<"ERROR: Unknown ResidueID "<<ResidueID<<endl;
                exit(EXIT_FAILURE);
        }
}

string OneLetterToThreeLetterCode(string residue)
{
	if (residue=="G") return "GLY";
	else if (residue=="A") return "ALA";
	else if (residue=="V") return "VAL";
	else if (residue=="L") return "LEU";
	else if (residue=="I") return "ILE";
	else if (residue=="S") return "SER";
	else if (residue=="T") return "THR";
	else if (residue=="C") return "CYS";
	else if (residue=="M") return "MET";
	else if (residue=="D") return "ASP";
	else if (residue=="E") return "GLU";
	else if (residue=="N") return "ASN";
	else if (residue=="Q") return "GLN";
	else if (residue=="K") return "LYS";
	else if (residue=="R") return "ARG";
	else if (residue=="F") return "PHE";
	else if (residue=="Y") return "TYR";
	else if (residue=="P") return "PRO";
	else if (residue=="H") return "HIS";
	else if (residue=="W") return "TRP";
	else
	{
		error("Unknown residue "+residue, __LINE__, __FILE__);
	}
	return "UNK";
}

int GetResidueID(string StrResidueID)
{
	bool verbose=false;
	if (StrResidueID=="GLY") return GLY;
	else if (StrResidueID=="ALA") return ALA;
	else if (StrResidueID=="VAL") return VAL;
	else if (StrResidueID=="LEU") return LEU;
	else if (StrResidueID=="ILE") return ILE;
	else if (StrResidueID=="SER") return SER;
	else if (StrResidueID=="THR") return THR;
	else if (StrResidueID=="CYS") return CYS;
	else if (StrResidueID=="MET") return MET;
	else if (StrResidueID=="ASP") return ASP;
	else if (StrResidueID=="GLU") return GLU;
	else if (StrResidueID=="ASN") return ASN;
	else if (StrResidueID=="GLN") return GLN;
	else if (StrResidueID=="LYS") return LYS;
	else if (StrResidueID=="ARG") return ARG;
	else if (StrResidueID=="PHE") return PHE;
	else if (StrResidueID=="TYR") return TYR;
	else if (StrResidueID=="PRO") return PRO;
	else if (StrResidueID=="HIS") return HIS;
	else if (StrResidueID=="HSE") return HIS;
	else if (StrResidueID=="HSD") return HIS;
	else if (StrResidueID=="TRP") return TRP;
	else if (StrResidueID=="HEM") return HEM;
	else if (StrResidueID=="WAT") return WAT;
	else if (StrResidueID=="TIP3") return WAT;
	else if (StrResidueID=="TIP4") return WAT;
	else if (StrResidueID=="TIP5") return WAT;
	else if (StrResidueID=="G") return GLY;
	else if (StrResidueID=="A") return ALA;
	else if (StrResidueID=="V") return VAL;
	else if (StrResidueID=="L") return LEU;
	else if (StrResidueID=="I") return ILE;
	else if (StrResidueID=="S") return SER;
	else if (StrResidueID=="T") return THR;
	else if (StrResidueID=="C") return CYS;
	else if (StrResidueID=="M") return MET;
	else if (StrResidueID=="D") return ASP;
	else if (StrResidueID=="E") return GLU;
	else if (StrResidueID=="N") return ASN;
	else if (StrResidueID=="Q") return GLN;
	else if (StrResidueID=="K") return LYS;
	else if (StrResidueID=="R") return ARG;
	else if (StrResidueID=="F") return PHE;
	else if (StrResidueID=="Y") return TYR;
	else if (StrResidueID=="P") return PRO;
	else if (StrResidueID=="H") return HIS;
	else if (StrResidueID=="W") return TRP;
	else
	{
		return UNK;
		if (verbose) cout <<"Warning unknown resiude "<<StrResidueID<<endl;
	}
}

void GetResidueIDs(vector<AtomStruct> &Atoms)
{
	int i, natom=Atoms.size();
	for (i=0;i<natom;i++) Atoms[i].residueid=GetResidueID(Atoms[i].ResidueName);
}

