/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <fstream>
# include <string>
# include <cstring>
# include <sstream>
# include <time.h>
# include <vector>
# include <iomanip>
# include <complex>
# include <numeric>

using namespace std;

# include "Structures.h"
# include "Constants.h"
# include "TypeDef.h"

ProteinStruct& ProteinStruct::operator = (const ProteinStruct &Protein)
{
	name=Protein.name;
	bravisLatticeCentering=Protein.bravisLatticeCentering;
	spaceGroup=Protein.spaceGroup;
        FirstResidue=Protein.FirstResidue;
        LastResidue=Protein.LastResidue;
        NumResidues=Protein.NumResidues;
	symmetry1=Protein.symmetry1;
	symmetry2=Protein.symmetry2;
	symmetry3=Protein.symmetry3;
	numPerUnitCell=Protein.numPerUnitCell;
	intSpaceGroup=Protein.intSpaceGroup;
        ResidueType=Protein.ResidueType;
        XBoxLength=Protein.XBoxLength;
        YBoxLength=Protein.YBoxLength;
        ZBoxLength=Protein.ZBoxLength;
	alpha=Protein.alpha;
	beta=Protein.beta;
	gamma=Protein.gamma;
	a=Protein.a;
	b=Protein.b;
	c=Protein.c;
	center=Protein.center;
	translation=Protein.translation;
	principalAxes=Protein.principalAxes;
	rotationMatrix=Protein.rotationMatrix;
        phi=Protein.phi;
        psi=Protein.psi;
        omega=Protein.omega;
        dihedral=Protein.dihedral;
        bondAngles=Protein.bondAngles;
	DistMatrix=Protein.DistMatrix;
	cartToFrac=Protein.cartToFrac;
	fracToCart=Protein.fracToCart;
	symmetryOperations=Protein.symmetryOperations;
        Atoms=Protein.Atoms;
        return *this;
}

bool AtomStruct::operator== (AtomStruct Atom)
{
	if (HetAtom!=Atom.HetAtom) return false;
	if (AtomNumber!=Atom.AtomNumber) return false;
	if (ResidueNum!=Atom.ResidueNum) return false;
	if (atomid!=Atom.atomid) return false;
	if (AtomType!=Atom.AtomType) return false;
	if (AtomType2!=Atom.AtomType2) return false;
	if (residueid!=Atom.residueid) return false;
	//if (atomtype!=Atom.atomtype) return false;
	if (weight!=Atom.weight) return false;
	if (Electrons!=Atom.Electrons) return false;
	if (charge!=Atom.charge) return false;
	if (mass!=Atom.mass) return false;
	//if (partialcharge!=Atom.partialcharge) return false;
	if (vdw!=Atom.vdw) return false;
	//if (epsilon!=Atom.epsilon) return false;
	if (epsilon14!=Atom.epsilon14) return false;
	if (vdw14!=Atom.vdw14) return false;
	if (HydrationRadius!=Atom.HydrationRadius) return false;
	if (x!=Atom.x) return false;
	if (y!=Atom.y) return false;
	if (z!=Atom.z) return false;

	return true;
}

bool AtomStruct::operator!= (AtomStruct Atom)
{
	if (HetAtom!=Atom.HetAtom) return true;
	if (solute!=Atom.solute) return true;
	if (AtomNumber!=Atom.AtomNumber) return true;
	if (ResidueNum!=Atom.ResidueNum) return true;
	if (atomid!=Atom.atomid) return true;
	if (AtomType!=Atom.AtomType) return true;
	if (AtomType2!=Atom.AtomType2) return true;
	if (ParmType!=Atom.ParmType) return true;
	if (residueid!=Atom.residueid) return true;
	//if (atomtype!=Atom.atomtype) return true;
	if (weight!=Atom.weight) return true;
	if (Electrons!=Atom.Electrons) return true;
	if (charge!=Atom.charge) return true;
	if (mass!=Atom.mass) return true;
	if (x!=Atom.x) return true;
	if (y!=Atom.y) return true;
	if (z!=Atom.z) return true;
	//if (partialcharge!=Atom.partialcharge) return true;
	if (vdw!=Atom.vdw) return true;
	//if (epsilon!=Atom.epsilon) return true;
	if (vdw14!=Atom.vdw14) return true;
	if (epsilon14!=Atom.epsilon14) return true;
	if (Occupancy!=Atom.Occupancy) return true;
	if (BFactor!=Atom.BFactor) return true;
	if (AtomName!=Atom.AtomName) return true;
	if (CharmmAtomName!=Atom.CharmmAtomName) return true;
	if (ResidueName!=Atom.ResidueName) return true;
	if (ChainName!=Atom.ChainName) return true;
	if (SegID!=Atom.SegID) return true;
	if (ID!=Atom.ID) return true;
	if (HydrationRadius!=Atom.HydrationRadius) return true;

	return false;
	
}

bool CompareAtoms(AtomStruct Atom1, AtomStruct Atom2)
{
	if (Atom1==Atom2) return true;
	else return false;
}

ProteinStruct::ProteinStruct(void)
{
	spaceGroup="";

	XBoxLength=0;
	YBoxLength=0;
	ZBoxLength=0;

	alpha=0;
	beta=0;
	gamma=0;

	symmetry1=0;
	symmetry2=0;
	symmetry3=0;

	a.x=0;
	a.y=0;
	a.z=0;
	
	b.x=0;
	b.y=0;
	b.z=0;
	
	c.x=0;
	c.y=0;
	c.z=0;

	
}

AtomStruct::AtomStruct(void)
{
	HetAtom=false;
	solute=true;
	core=false;
        AtomNumber=0;
	ResidueNum=0;
	atomid=0;
	AtomType=0;
	AtomType2=0;
	ParmType=0;
	residueid=0;
        ResidencyTime=0;
        xbin=-1;
        ybin=-1;
        zbin=-1;
	//atomtype=0;
	weight=1.0;
	Electrons=0;
	charge=0;
	mass=0;
	//partialcharge=0;
	vdw=0;
	//epsilon=0;
	epsilon14=0;
	vdw14=0;
	HydrationRadius=0;
	x=0;
	y=0;
	z=0;
	Occupancy=0;
	BFactor=0;
	AtomName="";
	CharmmAtomName="";
	duplicate="";
	ResidueName="";
	ChainName="A";
	SegID="";
	ID="";
}

CubeStruct& CubeStruct::operator = (const CubeStruct &cube)
{
	angular=cube.angular;
	phobic=cube.phobic;
	concave=cube.concave;
        occupied=cube.occupied;
	x=cube.x;
	y=cube.y;
	z=cube.z;
	density=cube.density;
        entropy=cube.entropy;
	MinDist=cube.MinDist;
        deg=cube.deg;
	AllAtomType=cube.AllAtomType;
	AtomType=cube.AtomType;
	ElementType=cube.ElementType;
	IntDist=cube.IntDist;
	LocatedIn=cube.LocatedIn;
	nearest=cube.nearest;
	SecondNearest=cube.SecondNearest;
        IntOrientation=cube.IntOrientation;
        ResidencyTimes=cube.ResidencyTimes;
	return *this;
}

CubeStruct::CubeStruct(void)
{
	angular=false;
	phobic=false;
	concave=false;
        occupied=false;
	x=0;
	y=0;
	z=0;
	density=0;
	entropy=0;
        MinDist=0;
        deg=0;
	AllAtomType=0;
        AtomType=0;
	ElementType=0;
	IntDist=0;
	LocatedIn=InProtein;
	nearest=0;
	SecondNearest=0;
}

void InitializeCube(CubeStruct &Cube)
{
	Cube.angular=false;
	Cube.phobic=false;
	Cube.concave=false;
	Cube.x=0;
	Cube.y=0;
	Cube.z=0;
	Cube.density=0;
        Cube.entropy=0;
	Cube.MinDist=1000.0;
        Cube.deg=0;
	Cube.AllAtomType=0;
	Cube.AtomType=0;
	Cube.ElementType=0;
	Cube.IntDist=0;
	Cube.LocatedIn=InProtein;
	Cube.nearest=0;
	Cube.SecondNearest=0;
	//for (int i=0;i<NumAtomTypes;i++) Cube.NumAtoms[i]=0;
}

void PrintCubeInfo(CubeStruct &Cube)
{
	cout <<"x= "<<Cube.x<<" y= "<<Cube.y<<" z= "<<Cube.z<<endl;
	cout <<"density= "<<Cube.density<<" MinDist= "<<Cube.MinDist
	<<" Cube.IntDist= "<<Cube.IntDist<<endl;
	cout <<"deg= "<<Cube.deg<<endl;
        cout <<"AllAtomType= "<<Cube.AllAtomType<<" AtomType= "<<Cube.AtomType
	<<" ElementType= "<<Cube.ElementType<<endl;
	cout <<"LocatedIn= "<<Cube.LocatedIn<<" nearest= "<<Cube.nearest
	<<" SecondNearest= "<<Cube.SecondNearest<<endl;
	cout <<"angular= "<<Cube.angular<<" phobic= "<<Cube.phobic
	<<" concave= "<<Cube.concave<<endl;
}

void CopyCube(CubeStruct Cube1, CubeStruct &Cube2)
{
	Cube2.angular=Cube1.angular;
	Cube2.phobic=Cube1.phobic;
	Cube2.concave=Cube1.concave;
	Cube2.x=Cube1.x;
	Cube2.y=Cube1.y;
	Cube2.z=Cube1.z;
	Cube2.density=Cube1.density;
	Cube2.entropy=Cube1.entropy;
        Cube2.MinDist=Cube1.MinDist;
        Cube2.deg=Cube1.deg;
	Cube2.AllAtomType=Cube1.AllAtomType;
	Cube2.AtomType=Cube1.AtomType;
	Cube2.ElementType=Cube1.ElementType;
	Cube2.IntDist=Cube1.IntDist;
	Cube2.nearest=Cube1.nearest;
	Cube2.SecondNearest=Cube1.SecondNearest;
        Cube2.IntOrientation=Cube1.IntOrientation;
        Cube2.ResidencyTimes=Cube1.ResidencyTimes;
}

