/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <iostream>
# include <fstream>
# include <string>
# include <cstring>
# include <cmath>
# include <sstream>
# include <time.h>
# include <vector>
# include <iomanip>
# include <complex>
# include <numeric>
# include <sys/time.h>
# include <algorithm>

using namespace std;

# include "StringUtils.h"
# include "LinkedList.h"
# include "IOUtils.h"
# include "TypeDef.h"
# include "time.h"
# include "MathUtils.h"
# include "error.h"

Real RFactor(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error)
{
        Real rfactor, scale, SumDiff, sum;
        SumDiff=0;
        sum=0;
        scale=expi[0]/i[0];
        //cout <<"scale= "<<scale<<endl;
        for (int n=0;n<points;n++) i[n]*=scale;
        for (int n=beginfit;n<endfit;n++)
        {
                SumDiff+=abs(expi[n]-i[n]);
                sum+=(expi[n]+i[n]);
                //cout <<"SumDiff= "<<SumDiff<<" sum= "<<sum<<" expi["<<n<<"]= "<<expi[n]<<" i["<<n<<"]= "<<i[n]<<endl;
        }
        rfactor=SumDiff/sum;
        return rfactor;
}

Real RFactorNoScale(vector<Real> &i, vector<Real> &expi)
{
	int npoint=i.size();
	Real sumDiff=0, sum=0, rfactor;
	for (int j=0;j<npoint;j++)
	{
		//sumDiff+=abs(sqrt(expi[j])-sqrt(i[j]));
		//sum+=sqrt(expi[j]);
		sumDiff+=abs(expi[j]-i[j]);
		sum+=expi[j];
		//cout <<expi[j]<<"\t"<<i[j]<<endl;
	}
	//endProgram(__LINE__, __FILE__);
	rfactor=sumDiff/sum;
	return rfactor;
}

Real RFactor(vector<Real> &i, vector<Real> &expi, Real scale)
{
	int npoint=i.size();
	Real sumDiff=0, sum=0, rfactor;
	for (int j=0;j<npoint;j++)
	{
		//sumDiff+=abs(sqrt(expi[j])-sqrt(i[j]));
		//sum+=sqrt(expi[j]);
		//cout <<"j= "<<j<<" expi= "<<expi[j]<<" i= "<<i[j]*scale<<endl;
		sumDiff+=abs(expi[j]-i[j]*scale);
		sum+=expi[j];
	}
	rfactor=sumDiff/sum;
	return rfactor;
}

Real RFactor(vector<Real> &i, vector<Real> &expi)
{
	int npoint=i.size();
	int npoint2=expi.size();
	Real r, minr, scale;
	//Real bestScale;
	//Real rp, rm, d, scalep, scalem;
	//Real e=1.0e-14;
	vector<Real> tempA, a, expA;
	timeval start, end;
	gettimeofday(&start, NULL);	
	if (npoint!=npoint2 || npoint==0)
	{
		string errorStr="npoint= "+toStr(npoint)+" npoint2= "+toStr(npoint2);
		error(errorStr, __LINE__, __FILE__);
	}

	SafeAlloc(expA, npoint, "tempA");
	SafeAlloc(a, npoint, "a");
	//cout <<"expi\ti"<<endl;
	for (int j=0;j<npoint;j++)
	{
		expA[j]=sqrt(expi[j]);
		a[j]=sqrt(i[j]);
		//cout <<expi[j]<<"\t"<<i[j]<<"\t"<<expA[j]<<"\t"<<a[j]<<endl;
	}

	scale=expA[0]/a[0];
	//bestScale=scale;
	multiply(scale, a, tempA);
	minr=RFactorNoScale(tempA, expA);
	for (int j=0;j<npoint;j++)
	{
		scale=expA[j]/a[j];
		multiply(scale, a, tempA);
		r=RFactorNoScale(tempA, expA);
		//cout <<"scale= "<<scale<<" r= "<<r<<endl;
		if (r<minr) 
		{
			minr=r;
			//bestScale=scale;
		}
	}
/*
	d=bestScale*0.0000001;
	scalep=bestScale+d;
	scalem=bestScale-d;
	multiply(scalep, a, tempA);
	rp=RFactorNoScale(tempA, expA);
	multiply(scalem, a, tempA);
	rm=RFactorNoScale(tempA, expA);
	if (minr-rm>e || minr-rp>e)
	{
		string errorStr="Best R-factor not found.";
		errorStr+="rp= "+toStr(rp)+" rm= "+toStr(rm);
		errorStr+=" minr= "+toStr(minr);
		errorStr+=" minr-rp= "+toStr(minr-rp);
		errorStr+=" minr-rm= "+toStr(minr-rm);
		error(errorStr, __LINE__, __FILE__);
	}
*/
	gettimeofday(&end, NULL);
	//cout <<"RFactor took "<<calcTimeDiff(start, end)<<endl;
	return minr;
}

Real calcRatioOfAverages(Vector &f, Vector &expF)
{
	Real ave1, ave2, scale;

	ave1=calcAverage(f);
	ave2=calcAverage(expF);
	scale=ave2/ave1;
	return scale;
}

Real RFactorFast(vector<Real> &i, vector<Real> &expF)
{
	int npoint=i.size();
	int npoint2=expF.size();
	Real r, scale;
	vector<Real> tempA, a;
	if (npoint!=npoint2 || npoint==0)
	{
		string errorStr="npoint= "+toStr(npoint)+" npoint2= "+toStr(npoint2);
		error(errorStr, __LINE__, __FILE__);
	}
	SafeAlloc(a, npoint, "a");
	for (int j=0;j<npoint;j++)
	{
		a[j]=sqrt(i[j]);
	}
	scale=calcRatioOfAverages(a, expF);
	r=RFactor(a, expF, scale);
	return r;
}



/*
Real RFactor(vector<Real> &i, vector<Real> &expi)
{
	int npoint=i.size();
	int npoint2=expi.size();
	Real r, minr, scale, bestScale;
	Real rp, rm, d, scalep, scalem;
	vector<Real> tempI;
	
	if (npoint!=npoint2)
	{
		cout <<"ERROR: The number of experimental points does not match the number of calculated points"<<endl;
		exit(EXIT_FAILURE);
	}

	scale=expi[0]/i[0];
	bestScale=scale;
	multiply(scale, i, tempI);
	minr=RFactorNoScale(tempI, expi);
	for (scale=0;scale<2.5;scale+=0.00001)
	{
		multiply(scale, i, tempI);
		r=RFactorNoScale(tempI, expi);
		cout <<"scale= "<<scale<<" r= "<<r<<endl;
		if (r<minr) 
		{
			minr=r;
			bestScale=scale;
		}
	}
	d=bestScale*0.0000001;
	scalep=bestScale+d;
	scalem=bestScale-d;
	multiply(scalep, i, tempI);
	rp=RFactorNoScale(tempI, expi);
	multiply(scalem, i, tempI);
	rm=RFactorNoScale(tempI, expi);
	cout <<"rm= "<<rm<<" scalem= "<<scalem<<endl;
	cout <<"minr= "<<minr<<" bestScale= "<<bestScale<<endl;
	cout <<"rp= "<<rp<<" scalep= "<<scalep<<endl;
	return minr;
}
*/
/*
Real RFactor(vector<Real> &i, vector<Real> &expi)
{
	int npoint=i.size();
	int npoint2=expi.size();
	Real r, isqr=0, crossterm=0, scale;
	vector<Real> tempI;
	
	if (npoint!=npoint2)
	{
		cout <<"ERROR: The number of experimental points does not match the number of calculated points"<<endl;
		exit(EXIT_FAILURE);
	}

        for (int j=0;j<npoint;j++)
        {
                isqr+=i[j]*i[j];
                crossterm+=i[j]*expi[j];
        }
        if (isqr!=0)
        {
                scale=crossterm/isqr;
        }
	multiply(scale, i, tempI);
	r=RFactorNoScale(tempI, expi);
	return r;
}
*/

Real calcSqrDiff(vector<Real> &i, vector<Real> &expi, Real scale)
{
	int npoint=i.size();
	Real score=0;
	for (int j=0;j<npoint;j++)
	{
		score+=(expi[j]-i[j]*scale)*(expi[j]-i[j]*scale);
	}
	//score/=Real(npoint)*expi[0]*expi[0];
	return score;
}

Real calcScaleForSqrDiff(vector<Real> &i, vector<Real> &expi)
{
	int npoint=i.size();
	int npoint2=expi.size();
	Real isqr=0, crossterm=0, scale=0;
	vector<Real> tempI;
	if (npoint!=npoint2 || npoint==0)
	{
		string errorStr;
		errorStr="The # of experimental points, ";
		errorStr+=IntToStr(npoint2)+", does not match the # of ";
		errorStr+="calculated points, "+IntToStr(npoint);
	}
        for (int j=0;j<npoint;j++)
        {
                isqr+=i[j]*i[j];
                crossterm+=i[j]*expi[j];
        }
        if (isqr!=0)
        {
                scale=crossterm/isqr;
        }
	else
	{
		string errorStr="Intensity is zero. isqr= "+RealToStr(isqr);
		errorStr+=" crossterm= "+RealToStr(crossterm);
		error(errorStr, __LINE__, __FILE__);
	}
	//cout <<"1-c= "<<1.0-scale<<endl;
	//scale=6.90086e-08;
	return scale;
}

Real calcBestSqrDiff(vector<Real> &i, vector<Real> &expi)
{
	//int npoint=i.size();
	Real scale=calcScaleForSqrDiff(i, expi);
/*
	Vector a, expa;
	
	SafeAlloc(a, npoint, "a");
	SafeAlloc(expa, npoint, "expa");
	for (int j=0;j<npoint;j++)
	{
		a[j]=sqrt(i[j]);
		expa[j]=sqrt(expi[j]);
	}
	scale=calcRatioOfAverages(a, expa);
	scale*=scale;
*/
	return calcSqrDiff(i, expi, scale);
}

Real MatchI0(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error)
{
	Real chisqr;
        Real scale;
        Real sumsqr=0;
        Real * inverror2;

        inverror2=new Real[endfit+1];

        for (int t=0;t<endfit+1;t++)
        {
                inverror2[t]=0;
        }

        //cout <<"error[0]="<<error[0]<<" error[1]="<<error[1]<<endl;

        for (int t=0;t<endfit+1;t++)
        {
                //cout <<"In normalize.  expi["<<t<<"]="<<expi[t]<<endl;
                if (error[t]!=0)
                {
                        inverror2[t]=1.0/(error[t]*error[t]);
                }
                else
                {
                        if (expi[t]!=0)
                        {
                                inverror2[t]=1.0/(expi[t]*expi[t]*0.02*0.02);
                        }
                        else
                        {
                                inverror2[t]=0.01;
                                //cout <<"Error.  Experimental intensity is 0"<<endl;
                        }
                }
        }

        scale=expi[0]/i[0];
        //cout <<"scale= "<<scale<<endl;
        for (int t=0;t<points;t++)
        {
                i[t]=i[t]*scale;
        }

        for (int t=beginfit;t<endfit+1;t++)
        {
                sumsqr=sumsqr+(i[t]-expi[t])*(i[t]-expi[t])*inverror2[t];
        }

        chisqr=sumsqr/float(endfit-beginfit+1);

        delete [] inverror2;

        return chisqr;
}

Real NoScaling(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error)
{
        Real chisqr;
        Real sumsqr;
        Real * inverror2;

        inverror2=new Real[endfit+1];

        for (int t=0;t<endfit+1;t++)
        {
                inverror2[t]=0;
        }

        //cout <<"error[0]="<<error[0]<<" error[1]="<<error[1]<<endl;

        for (int t=0;t<endfit+1;t++)
        {
                //cout <<"In normalize.  expi["<<t<<"]="<<expi[t]<<endl;
                if (error[t]!=0)
                {
                        inverror2[t]=1.0/(error[t]*error[t]);
                }
                else
                {
                        if (expi[t]!=0)
                        {
                                inverror2[t]=1.0/(expi[t]*expi[t]*0.02*0.02);
                        }
                        else
                        {
                                inverror2[t]=0.01;
                                //cout <<"Error.  Experimental intensity is 0"<<endl;
                        }
                }
        }

        sumsqr=0;

        for (int t=beginfit;t<endfit+1;t++)
        {
                sumsqr=sumsqr+(i[t]-expi[t])*(i[t]-expi[t])*inverror2[t];
        }

        chisqr=sumsqr/float(endfit-beginfit+1);

        delete [] inverror2;

        return chisqr;
}

Real log(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error)
{
        int t;
        Real chisqr;
        Real sumsqr;

        sumsqr=0;

        for (t=beginfit;t<endfit+1;t++)
        {
                sumsqr=sumsqr+(log(i[t]/expi[t]))*(log(i[t]/expi[t]));
        }

        chisqr=sumsqr/Real(endfit-beginfit+1);

        return chisqr;
}

Real CalcLogNormalization(vector<Real> &expi, vector<Real> &i, int beginfit, int endfit)
{
        int nexp=expi.size();
        int npoint=i.size();
        Real sum=0;

        if (nexp<endfit) endfit=nexp;
        if (npoint<endfit) endfit=npoint;

        for (int t=beginfit;t<endfit;t++)
        {
                cout <<"sum= "<<sum<<" expi["<<t<<"]= "<<expi[t]<<" i= "<<i[t]<<endl;
                if (expi[t]>0 && i[t]>0) sum+=log(expi[t]/i[t]);
        }
        sum/=(endfit+1-beginfit);
        return exp(sum);
}

void ScaleIntensity(Real scale, vector<Real> &i)
{
        int points=i.size();
        for (int t=0;t<points;t++)
        {
                //cout <<"i["<<t<<"]= "<<i[t]<<endl;
                i[t]*=scale;
                //cout <<"i["<<t<<"]= "<<i[t]<<endl;
        }
}

Real log_normalize(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, vector<Real> &error)
{
        int numpoints=0;
        Real chisqr;
        Real scale;
        Real sumsqr;
        //cout <<"In log_normalize"<<endl;
        sumsqr=0;
        scale=CalcLogNormalization(expi, i, beginfit, endfit);
        cout <<"scale= "<<scale<<endl;
        ScaleIntensity(scale, i);
        for (int t=beginfit;t<endfit;t++)
        {
                if (expi[t]>0)
                {
                        cout <<"expi["<<t<<"]= "<<expi[t]<<" i= "<<i[t]<<" sqr= "<<(log(i[t]/expi[t]))*(log(i[t]/expi[t]))<<endl;
                        sumsqr=sumsqr+(log(i[t]/expi[t]))*(log(i[t]/expi[t]));
                        numpoints++;
                }
        }
        //cout <<"numpoints= "<<numpoints<<endl;
        //cout <<"endfit-beginfit= "<<endfit-beginfit<<endl;
        chisqr=sumsqr/Real(endfit-beginfit);

        return chisqr;
}

Real normalize(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error)
{
        int t;
        Real chisqr;
        Real isqr;
        Real crossterm;
        Real scale;
        Real sumsqr;
        Real * inverror2;

        inverror2=new Real[endfit+1];

        for (t=0;t<endfit+1;t++)
        {
                inverror2[t]=0;
        }

        //cout <<"error[0]="<<error[0]<<" error[1]="<<error[1]<<endl;

        for (t=0;t<endfit+1;t++)
        {
                //cout <<"In normalize.  expi["<<t<<"]="<<expi[t]<<endl;
                if (error[t]!=0)
                {
                        inverror2[t]=1.0/(error[t]*error[t]);
                }
                else
                {
                        if (expi[t]!=0)
                        {
                                inverror2[t]=1.0/(expi[t]*expi[t]*0.02*0.02);
                        }
                        else
                        {
                                inverror2[t]=0.01;
                                //cout <<"Error.  Experimental intensity is 0"<<endl;
                        }
                }
        }

        isqr=0;
        crossterm=0;
        sumsqr=0;

        for (t=beginfit;t<endfit+1;t++)
        {
                //cout <<"In normalize.  i["<<t<<"]="<<i[t]<<endl;
                isqr+=i[t]*i[t]*inverror2[t];
                crossterm+=i[t]*expi[t]*inverror2[t];
                //cout <<"i["<<t<<"]= "<<i[t]<<" expi["<<t<<"]= "<<expi[t]<<endl;
        }
        //cout <<"crossterm= "<<crossterm<<" isqr= "<<isqr<<endl;
        if (isqr!=0)
        {
                scale=crossterm/isqr;
        }
        else
        {
                scale=1.0;
                cout <<"Error scalling\n";
        }
        //cout <<"scale= "<<scale<<endl;
        for (t=0;t<points;t++)
        {
                i[t]=i[t]*scale;
        }

        for (t=beginfit;t<endfit+1;t++)
        {
                sumsqr=sumsqr+(i[t]-expi[t])*(i[t]-expi[t])*inverror2[t];
        }

        chisqr=sumsqr/float(endfit-beginfit+1);

        delete [] inverror2;

        return chisqr;
}

Real normalize(vector<Real> &i, vector<Real> &expi, int beginfit, int endfit, int points, vector<Real> &error, string option)
{
        Real chisqr;
        if (option=="normalize") chisqr=normalize(i, expi, beginfit, endfit, points, error);
        else if (option=="NoScaling") chisqr=NoScaling(i, expi, beginfit, endfit, points, error);
        else if (option=="RFactor") chisqr=RFactor(i, expi, beginfit, endfit, points, error);
        else if (option=="MatchI0") chisqr=MatchI0(i, expi, beginfit, endfit, points, error);
        else if (option=="log") chisqr=log(i, expi, beginfit, endfit, points, error);
        else if (option=="log_normalize") chisqr=log_normalize(i, expi, beginfit, endfit, error);
        else
        {
                cout <<"ERROR: Unrecognized option "<<option<<endl;
                exit(EXIT_FAILURE);
        }
        //chisqr=NoScaling(i, expi, beginfit, endfit, points, error);
        cout <<"chisqr= "<<chisqr<<endl;

        return chisqr;
}
