#ifndef _FlexibleEditing_include_
#define _FlexibleEditing_include_
void readSegmentsFile(string segmentsFile, vector<int> &segmentStart, vector<int> &segmentEnd);
void intToBinary(int integer, Vector &binary);
void residueSegmentToAtomSegment(vector<AtomStruct> &Atoms, int &segmentStart, int &segmentEnd);
void residueSegmentsToAtomSegments(vector<AtomStruct> &Atoms, vector<int> &segmentStart, vector<int> &segmentEnd);
void setSegmentOccupancies(ProteinStruct &Protein, Vector &segmentOccupancies, vector<int> &segmentStart, vector<int> &segmentEnd);
void setSegmentOccupancies(ProteinStruct &Protein, Vector &segmentOccupancies, vector<int> &segmentStart, vector<int> &segmentEnd, Real defaultOccupancy);
void setSegmentOccupancies(ProteinStruct &Protein, Vector &degOfFreedom, XRayParamStruct &params);
void setSegmentOccupancies(ProteinStruct &Protein, Vector &degOfFreedom);
void setSegmentOccupancyToOne(vector<AtomStruct> &Atoms, int segmentStart, int segmentEnd);
void setFixedAndSegmentOccupancyToOne(vector<AtomStruct> &Atoms, vector<int> &segmentStart, vector<int> &segmentEnd, int segment);
void setNonFixedOccupancyToZero(ProteinStruct &Protein, vector<int> &segmentStart, vector<int> &segmentEnd);
void calcSegmentScattering(XRayStruct &xray, ProteinStruct &Protein, XRayParamStruct &params, int segment, Vector &real, Vector &imag);
void calcSegmentScattering(XRayStruct &xray, ProteinStruct &Protein, XRayParamStruct &params, int segmentStart, int segmentEnd, Vector &real, Vector &imag);
void addSegmentScattering(Vector &segmentOccupancies, Matrix &real, Matrix &imag, XRayStruct &xray);
void addSegmentScattering(Vector &segmentOccupancies, Matrix &real, Matrix &imag, Vector &fixedReal, Vector &fixedImag, XRayStruct &xray);
void calcSegmentClashScores(ProteinStruct &Protein, XRayParamStruct &params, Matrix &intraSegmentClashes, Array3D &interSegmentClashes);
void calcSegmentClashScores(ProteinStruct &Protein, XRayParamStruct &params, vector<int> &segmentStart, vector<int> &segmentEnd, Real &fixedClash, Vector &fixedNonFixedClashes, Vector &intraSegmentClashes, Matrix &interSegmentClashes);
Real calcClashScore(Vector &segmentOccupancies, Matrix &intraSegmentClashes, Array3D &interSegmentClashes, XRayParamStruct &params, Real &clashCC, Real &clashCS, Real &clashSS);
Real calcClashScore(Vector &segmentOccupancies, Real fixedClash, Vector &fixedNonFixedClashes, Vector &intraSegmentClashes, Matrix &interSegmentClashes);
Real calcAveOccupancy(Vector &segmentOccupancies);
Real optimizeSegmentOccupanciesAfterMR(XRayStruct &xray, XRayStruct &expXRay, ProteinStruct &Protein, XRayParamStruct &params);
Real optimizeSegmentOccupanciesAfterMR(XRayStruct &xray, XRayStruct &expXRay, ProteinStruct &Protein, XRayParamStruct &params, vector<int> &segmentStart, vector<int> &segmentEnd);
Real optimizeSegmentsFromFileAfterMR(XRayStruct &xray, XRayStruct &expXRay, ProteinStruct &Protein, XRayParamStruct &params);
void setIncorrectAtomsToZero(ProteinStruct &Protein, XRayParamStruct &params);
void readProbabilitiesFile(string probabilitesFile, Real &binSize, Array3D &probabilities, Array3D &trimerProbability);
void readProbabilitiesFile(string probabilitesFile, Real &binSize, Array3D &probabilities, Array3D &trimerProbability, Real estimatedRmsd);
Real probabilityToOccupancy(Array3D &probabilities, Real binSize, Real clashScore, Real xrayScore);
void probabilitesToOccupancies(Array3D &probabilities, Real binSize, Vector &residueClashScore, Vector &residueXRayScore, vector<AtomStruct> &Atoms);
int getFeature(Real clashScore, Real xrayScore);
void getFeatureSequence(Vector &clashScore, Vector &xrayScore, vector<int> &feature);
void calcOccupancies(Array3D &probabilities, Array3D &trimerProbabilities, Real binSize, Vector &residueClashScore, Vector &residueXRayScore, Vector &occupancies);
void assignOccupancies(Vector &occupancies, vector<AtomStruct> &Atoms);
void probabilitiesToOccupancies(Array3D &probabilities, Array3D &trimerProbabilities, Real binSize, Vector &residueClashScore, Vector &residueXRayScore, vector<AtomStruct> &Atoms);
void calcResidueClashScore(ProteinStruct Protein, XRayParamStruct &params, Vector &residueClashScore);
void subtractResidueFromScattering(ProteinStruct &Protein, XRayStruct &xray, int residue);
Real calcResidueXRayScore(ProteinStruct &Protein, XRayStruct xray, XRayStruct &expXRay, int residue, XRayParamStruct &params);
void calcResidueXRayScore(ProteinStruct Protein, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params, Vector &residueXRayScore);
Real calcResidueDensity(ProteinStruct &Protein, int residue, LatticeStruct &lattice, LatticeStruct &expLattice);
Real calcResidueDensity(ProteinStruct &Protein, int residue, XRayStruct &xray, XRayStruct &combinedXRay, Real calcScale, Real expScale);
void calcResidueDensity(ProteinStruct &Protein, XRayStruct &xray, XRayStruct &expXRay, LatticeStruct &lattice, LatticeStruct &expLattice, Vector &residueDensity);
void calcResidueDensityScore(ProteinStruct &Protein, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params, Vector &residueDensity);
void setOccupancies(ProteinStruct &Protein, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params, Real estimatedRmsd);
void setOccupancies(ProteinStruct &Protein, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params);
Real estimateRmsdFromAveOccupancy(Real aveOccupancy);
Real estimateRmsdFromAveOccupancy(Real aveOccupancy, Real residues);
Real calcAverageOccupancy(vector<AtomStruct> &Atoms);
Real estimateRmsdFromAveOccupancy(vector<AtomStruct> &Atoms);
Real estimateRmsd(ProteinStruct Protein, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params);
Real findMinEstimatedRmsd(vector<ProteinStruct> &Proteins, XRayStruct &xray, XRayStruct &expXRay, XRayParamStruct &params);
Real findMinEstimatedRmsd(string pdbPathsFile, XRayStruct xray, XRayStruct expXRay, XRayParamStruct &params);
#endif
